@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -61,6 +61,7 @@ ROCSPARSE_DO_ROUTINE(csrcolor)					\
 ROCSPARSE_DO_ROUTINE(csric0)					\
 ROCSPARSE_DO_ROUTINE(csrilu0)					\
 ROCSPARSE_DO_ROUTINE(csritilu0)					\
+ROCSPARSE_DO_ROUTINE(csritilu0_ex)				\
 ROCSPARSE_DO_ROUTINE(csrgeam)					\
 ROCSPARSE_DO_ROUTINE(csrgemm)					\
 ROCSPARSE_DO_ROUTINE(csrgemm_reuse)				\
@@ -122,9 +123,15 @@ ROCSPARSE_DO_ROUTINE(sctr)					\
 ROCSPARSE_DO_ROUTINE(sddmm)					\
 ROCSPARSE_DO_ROUTINE(sparse_to_dense_coo)			\
 ROCSPARSE_DO_ROUTINE(sparse_to_dense_csc)			\
- ROCSPARSE_DO_ROUTINE(sparse_to_dense_csr)			\
- ROCSPARSE_DO_ROUTINE(sparse_to_sparse)				\
- ROCSPARSE_DO_ROUTINE(extract)
+ROCSPARSE_DO_ROUTINE(sparse_to_dense_csr)			\
+ROCSPARSE_DO_ROUTINE(sparse_to_sparse)				\
+ROCSPARSE_DO_ROUTINE(extract)					\
+ROCSPARSE_DO_ROUTINE(v2_spmv_bsr)				\
+ROCSPARSE_DO_ROUTINE(v2_spmv_csr)				\
+ROCSPARSE_DO_ROUTINE(v2_spmv_csc)				\
+ROCSPARSE_DO_ROUTINE(v2_spmv_coo)				\
+ROCSPARSE_DO_ROUTINE(v2_spmv_coo_aos)				\
+ROCSPARSE_DO_ROUTINE(v2_spmv_ell)
 // clang-format on
 
 template <std::size_t N, typename T>
