@@ -197,6 +197,10 @@ INSTANTIATE_TIJ(int8_t, int32_t, int32_t);
 INSTANTIATE_TIJ(int8_t, int64_t, int32_t);
 INSTANTIATE_TIJ(int8_t, int64_t, int64_t);
 
+INSTANTIATE_TIJ(_Float16, int32_t, int32_t);
+INSTANTIATE_TIJ(_Float16, int64_t, int32_t);
+INSTANTIATE_TIJ(_Float16, int64_t, int64_t);
+
 INSTANTIATE_TIJ(float, int32_t, int32_t);
 INSTANTIATE_TIJ(float, int64_t, int32_t);
 INSTANTIATE_TIJ(float, int64_t, int64_t);
@@ -216,6 +220,9 @@ INSTANTIATE_TIJ(rocsparse_double_complex, int64_t, int64_t);
 INSTANTIATE_TI(int8_t, int32_t);
 INSTANTIATE_TI(int8_t, int64_t);
 
+INSTANTIATE_TI(_Float16, int32_t);
+INSTANTIATE_TI(_Float16, int64_t);
+
 INSTANTIATE_TI(float, int32_t);
 INSTANTIATE_TI(float, int64_t);
 
