@@ -114,6 +114,11 @@ inline rocsparseio_type type_tconvert<int64_t>()
     return rocsparseio_type_int64;
 };
 template <>
+inline rocsparseio_type type_tconvert<_Float16>()
+{
+    return rocsparseio_type_float16;
+};
+template <>
 inline rocsparseio_type type_tconvert<float>()
 {
     return rocsparseio_type_float32;
@@ -283,6 +288,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
             }
 
             case rocsparseio_type_int8:
+            case rocsparseio_type_float16:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -309,6 +315,12 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_coo(I* row_ind, I
                 break;
             }
 
+            case rocsparseio_type_float16:
+            {
+                rocsparse_importer_copy_mixed_arrays(NNZ, val, (const _Float16*)tmp_val);
+                break;
+            }
+
             case rocsparseio_type_float32:
             {
                 //
@@ -485,6 +497,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
             }
 
             case rocsparseio_type_int8:
+            case rocsparseio_type_float16:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -510,6 +523,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
                 break;
             }
             case rocsparseio_type_int8:
+            case rocsparseio_type_float16:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -537,6 +551,13 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_gebsx(I* ptr, J*
                 break;
             }
 
+            case rocsparseio_type_float16:
+            {
+                rocsparse_importer_copy_mixed_arrays(
+                    NNZB * row_block_dim * col_block_dim, val, (const _Float16*)tmp_val);
+                break;
+            }
+
             case rocsparseio_type_float32:
             {
                 rocsparse_importer_copy_mixed_arrays(
@@ -694,6 +715,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
             }
 
             case rocsparseio_type_int8:
+            case rocsparseio_type_float16:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -719,6 +741,7 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
                 break;
             }
             case rocsparseio_type_int8:
+            case rocsparseio_type_float16:
             case rocsparseio_type_float32:
             case rocsparseio_type_float64:
             case rocsparseio_type_complex32:
@@ -745,6 +768,12 @@ rocsparse_status rocsparse_importer_rocsparseio::import_sparse_csx(I* ptr, J* in
                 break;
             }
 
+            case rocsparseio_type_float16:
+            {
+                rocsparse_importer_copy_mixed_arrays(NNZ, val, (const _Float16*)tmp_val);
+                break;
+            }
+
             case rocsparseio_type_float32:
             {
                 rocsparse_importer_copy_mixed_arrays(NNZ, val, (const float*)tmp_val);
@@ -806,6 +835,10 @@ INSTANTIATE_TIJ(int8_t, int32_t, int32_t);
 INSTANTIATE_TIJ(int8_t, int64_t, int32_t);
 INSTANTIATE_TIJ(int8_t, int64_t, int64_t);
 
+INSTANTIATE_TIJ(_Float16, int32_t, int32_t);
+INSTANTIATE_TIJ(_Float16, int64_t, int32_t);
+INSTANTIATE_TIJ(_Float16, int64_t, int64_t);
+
 INSTANTIATE_TIJ(float, int32_t, int32_t);
 INSTANTIATE_TIJ(float, int64_t, int32_t);
 INSTANTIATE_TIJ(float, int64_t, int64_t);
@@ -825,6 +858,9 @@ INSTANTIATE_TIJ(rocsparse_double_complex, int64_t, int64_t);
 INSTANTIATE_TI(int8_t, int32_t);
 INSTANTIATE_TI(int8_t, int64_t);
 
+INSTANTIATE_TI(_Float16, int32_t);
+INSTANTIATE_TI(_Float16, int64_t);
+
 INSTANTIATE_TI(float, int32_t);
 INSTANTIATE_TI(float, int64_t);
 
