@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,7 +49,7 @@ namespace fs = std::experimental::filesystem;
 #include "rocsparse_clients_envariables.hpp"
 
 // Parse YAML data
-static std::string rocsparse_parse_yaml(const std::string& yaml)
+static std::string rocsparse_parse_yaml(const std::string& yaml, const char* include_path)
 {
 #ifdef WIN32
     // Generate "/tmp/rocsparse-XXXXXX" like file name
@@ -64,9 +64,26 @@ static std::string rocsparse_parse_yaml(const std::string& yaml)
 
     fs::path tmpname = fs::temp_directory_path() / uniquestr;
 
-    auto exepath = rocsparse_exepath();
-    auto cmd     = exepath + "rocsparse_gentest.py --template " + exepath
-               + "rocsparse_template.yaml -o " + tmpname.string() + " " + yaml;
+    auto        exepath       = rocsparse_exepath();
+    const char* matrices_path = rocsparse_clients_matrices_dir_get(false);
+    auto        cmd           = exepath + "rocsparse_gentest.py ";
+    if(include_path != nullptr)
+    {
+        cmd += " -I ";
+        cmd += include_path;
+    }
+    else
+    {
+        cmd += " -I ./ ";
+    }
+
+    if(matrices_path && matrices_path[0] != '\0')
+    {
+        cmd += " -m ";
+        cmd += matrices_path;
+    }
+    cmd += " --template " + exepath + "rocsparse_template.yaml -o " + tmpname.string() + " " + yaml;
+
     std::cerr << cmd << std::endl;
 
     int status = std::system(cmd.c_str());
@@ -86,10 +103,29 @@ static std::string rocsparse_parse_yaml(const std::string& yaml)
         perror("Cannot open temporary file");
         exit(EXIT_FAILURE);
     }
-    auto exepath = rocsparse_exepath();
-    auto cmd     = exepath + "rocsparse_gentest.py --template " + exepath
-               + "rocsparse_template.yaml -o " + tmp + " " + yaml;
+    auto        exepath       = rocsparse_exepath();
+    const char* matrices_path = rocsparse_clients_matrices_dir_get(false);
+    auto        cmd           = exepath + "rocsparse_gentest.py ";
+    if(include_path != nullptr)
+    {
+        cmd += " -I ";
+        cmd += include_path;
+    }
+    else
+    {
+        cmd += " -I ./ ";
+    }
+
+    if(matrices_path && matrices_path[0] != '\0')
+    {
+        cmd += " -m ";
+        cmd += matrices_path;
+    }
+
+    cmd += " --template " + exepath + "rocsparse_template.yaml -o " + tmp + " " + yaml;
+
     std::cerr << cmd << std::endl;
+
     int status = system(cmd.c_str());
     if(status == -1 || !WIFEXITED(status) || WEXITSTATUS(status))
         exit(EXIT_FAILURE);
@@ -108,7 +144,6 @@ bool rocsparse_parse_data(int& argc, char** argv, const std::string& default_fil
 #ifdef ROCSPARSE_WITH_MEMSTAT
     const char* memory_report_filename = nullptr;
 #endif
-    std::string reproducibility_filename("rocsparse_reproducibility.json");
 
     // Scan, process and remove any --yaml or --data options
     std::string command = argv[0];
@@ -116,10 +151,69 @@ bool rocsparse_parse_data(int& argc, char** argv, const std::string& default_fil
     {
         command += std::string(" ") + argv[i];
     }
+    const char* include_path = nullptr;
     rocsparse_reproducibility_t::instance().config().set_command(command);
+
+    //
+    // Disable debug argument verbose
+    //
+    {
+        static constexpr const char* option_force_debug_arguments_verbose
+            = "--force-debug-arguments-verbose";
+        bool enable_debug_arguments_verbose = false;
+        for(int iarg = 0; iarg < argc; ++iarg)
+        {
+            if(!strcmp(argv[iarg], option_force_debug_arguments_verbose))
+            {
+                enable_debug_arguments_verbose = true;
+                (void)memcpy(argv + iarg, argv + iarg + 1, (argc - iarg) * sizeof(char*));
+                argv[--argc] = nullptr;
+                break;
+            }
+        }
+
+        if(enable_debug_arguments_verbose == false)
+        {
+            rocsparse_disable_debug_arguments_verbose();
+            std::cout << "rocsparse-test: debug arguments verbose is disabled for testings (use "
+                      << option_force_debug_arguments_verbose << " to skip the disabling)"
+                      << std::endl;
+        }
+    }
+
+    //
+    // Disable debug warnings
+    //
+    {
+        static constexpr const char* option_force_warning  = "--force-warnings";
+        bool                         enable_debug_warnings = false;
+        for(int iarg = 0; iarg < argc; ++iarg)
+        {
+            if(!strcmp(argv[iarg], option_force_warning))
+            {
+                enable_debug_warnings = true;
+                (void)memcpy(argv + iarg, argv + iarg + 1, (argc - iarg) * sizeof(char*));
+                argv[--argc] = nullptr;
+                break;
+            }
+        }
+
+        if(enable_debug_warnings == false)
+        {
+            rocsparse_disable_debug_warnings();
+            std::cout << "rocsparse-test: warnings are disabled for testings (use "
+                      << option_force_warning << " to skip the disabling)" << std::endl;
+        }
+    }
+
     for(int i = 1; argv[i]; ++i)
     {
-        if(!strcmp(argv[i], "--data") || (yaml |= !strcmp(argv[i], "--yaml")))
+
+        if(!strcmp(argv[i], "-I"))
+        {
+            include_path = argv[++i];
+        }
+        else if(!strcmp(argv[i], "--data") || (yaml |= !strcmp(argv[i], "--yaml")))
         {
             if(filename != "")
             {
@@ -224,59 +318,17 @@ bool rocsparse_parse_data(int& argc, char** argv, const std::string& default_fil
             }
             rocsparse_reproducibility_t::instance().config().set_info_level(r_level);
         }
-        else if(!strcmp(argv[i], "--rocsparse-clients-enable-test-debug-arguments"))
-        {
-            rocsparse_clients_envariables::set(rocsparse_clients_envariables::TEST_DEBUG_ARGUMENTS,
-                                               true);
-        }
-        else if(!strcmp(argv[i], "--rocsparse-clients-disable-test-debug-arguments"))
-        {
-            rocsparse_clients_envariables::set(rocsparse_clients_envariables::TEST_DEBUG_ARGUMENTS,
-                                               false);
-        }
-        else if(!strcmp(argv[i], "--rocsparse-disable-debug"))
-        {
-            rocsparse_disable_debug();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-enable-debug"))
-        {
-            rocsparse_enable_debug();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-enable-debug-verbose"))
-        {
-            rocsparse_enable_debug_verbose();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-disable-debug-verbose"))
-        {
-            rocsparse_disable_debug_verbose();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-enable-debug-arguments"))
-        {
-            rocsparse_enable_debug_arguments();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-disable-debug-arguments"))
-        {
-            rocsparse_disable_debug_arguments();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-enable-debug-arguments-verbose"))
-        {
-            rocsparse_enable_debug_arguments_verbose();
-        }
-        else if(!strcmp(argv[i], "--rocsparse-disable-debug-arguments-verbose"))
-        {
-            rocsparse_disable_debug_arguments_verbose();
-        }
         else
         {
             *argv_p++ = argv[i];
             if(!help && (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")))
             {
                 help = true;
-                std::cout
-                    << "\n"
-                    << argv[0]
-                    << " [ --data <path> | --yaml <path> ] [--matrices-dir <path>]<options> ...\n"
-                    << std::endl;
+                std::cout << "\n"
+                          << argv[0]
+                          << " [ --data <path> | --yaml <path> ] [--matrices-dir <path>] [-I "
+                             "<path>] <options> ...\n"
+                          << std::endl;
 
                 std::cout << "" << std::endl;
                 std::cout << "Rocsparse reproducibility options:" << std::endl;
@@ -289,49 +341,6 @@ bool rocsparse_parse_data(int& argc, char** argv, const std::string& default_fil
                              "json file."
                           << std::endl;
                 std::cout << "" << std::endl;
-                std::cout << "Rocsparse clients debug options:" << std::endl;
-                std::cout << "--rocsparse-clients-enable-test-debug-arguments   enable rocsparse "
-                             "clients test debug arguments, it discards any environment variable "
-                             "definition of ROCSPARSE_CLIENTS_TEST_DEBUG_ARGUMENTS."
-                          << std::endl;
-                std::cout << "--rocsparse-clients-disable-test-debug-arguments  disable rocsparse "
-                             "clients test debug arguments, it discards any environment variable "
-                             "definition of ROCSPARSE_CLIENTS_TEST_DEBUG_ARGUMENTS."
-                          << std::endl;
-                std::cout << "" << std::endl;
-                std::cout << "Rocsparse debug options:" << std::endl;
-                std::cout << "--rocsparse-enable-debug                     enable rocsparse debug, "
-                             "it discards any environment variable definition of ROCSPARSE_DEBUG."
-                          << std::endl;
-                std::cout
-                    << "--rocsparse-disable-debug                    disable rocsparse debug, it "
-                       "discards any environment variable definition of ROCSPARSE_DEBUG."
-                    << std::endl;
-                std::cout << "--rocsparse-enable-debug-verbose             enable rocsparse debug "
-                             "verbose, it discards any environment variable definition of "
-                             "ROCSPARSE_DEBUG_VERBOSE."
-                          << std::endl;
-                std::cout << "--rocsparse-disable-debug-verbose            disable rocsparse debug "
-                             "verbose, it discards any environment variable definition of "
-                             "ROCSPARSE_DEBUG_VERBOSE"
-                          << std::endl;
-                std::cout << "--rocsparse-enable-debug-arguments           enable rocsparse debug "
-                             "arguments, it discards any environment variable definition of "
-                             "ROCSPARSE_DEBUG_ARGUMENTS."
-                          << std::endl;
-                std::cout << "--rocsparse-disable-debug-arguments          disable rocsparse debug "
-                             "arguments, it discards any environment variable definition of "
-                             "ROCSPARSE_DEBUG_ARGUMENTS."
-                          << std::endl;
-                std::cout << "--rocsparse-enable-debug-arguments-verbose   enable rocsparse debug "
-                             "arguments verbose, it discards any environment variable definition "
-                             "of ROCSPARSE_DEBUG_ARGUMENTS_VERBOSE"
-                          << std::endl;
-                std::cout << "--rocsparse-disable-debug-arguments-verbose  disable rocsparse debug "
-                             "arguments verbose, it discards any environment variable definition "
-                             "of ROCSPARSE_DEBUG_ARGUMENTS_VERBOSE"
-                          << std::endl;
-                std::cout << "" << std::endl;
                 std::cout << "" << std::endl;
                 std::cout << "Specific environment variables:" << std::endl;
                 for(const auto v : rocsparse_clients_envariables::s_var_bool_all)
@@ -369,7 +378,7 @@ bool rocsparse_parse_data(int& argc, char** argv, const std::string& default_fil
         filename = default_file;
 
     if(yaml)
-        filename = rocsparse_parse_yaml(filename);
+        filename = rocsparse_parse_yaml(filename, include_path);
 
     if(filename != "")
     {
