@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #pragma once
 
-#include <rocsparse.hpp>
+#include "rocsparse.hpp"
 
 #include "auto_testing_bad_arg.hpp"
 #include "gbyte.hpp"
@@ -280,45 +280,17 @@ void testing_csx2dense(const Arguments& arg, FUNC1& csx2dense, FUNC2& dense2csx)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
-
-        //
-        // Warm-up
-        //
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-
-            CHECK_ROCSPARSE_ERROR(csx2dense(handle,
-                                            M,
-                                            N,
-                                            descr,
-                                            d_csx_val,
-                                            d_csx_row_col_ptr,
-                                            d_csx_col_row_ind,
-                                            (T*)d_dense_val,
-                                            LD));
-        }
-
-        double gpu_time_used = get_time_us();
-        {
-            //
-            // Performance run
-            //
-            for(int iter = 0; iter < number_hot_calls; ++iter)
-            {
-                CHECK_ROCSPARSE_ERROR(csx2dense(handle,
-                                                M,
-                                                N,
-                                                descr,
-                                                d_csx_val,
-                                                d_csx_row_col_ptr,
-                                                d_csx_col_row_ind,
-                                                (T*)d_dense_val,
-                                                LD));
-            }
-        }
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg, [&] {
+            return csx2dense(handle,
+                             M,
+                             N,
+                             descr,
+                             d_csx_val,
+                             d_csx_row_col_ptr,
+                             d_csx_col_row_ind,
+                             (T*)d_dense_val,
+                             LD);
+        });
 
         double gbyte_count = csx2dense_gbyte_count<DIRA, T>(M, N, nnz);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
