@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
 #ifndef UTILITY_HPP
 #define UTILITY_HPP
 
+#include "rocsparse_clients_routine_trace.hpp"
 #include "rocsparse_matrix.hpp"
 #include "rocsparse_test.hpp"
 
@@ -88,6 +89,12 @@ inline rocsparse_datatype get_datatype<uint32_t>(void)
     return rocsparse_datatype_u32_r;
 }
 
+template <>
+inline rocsparse_datatype get_datatype<_Float16>(void)
+{
+    return rocsparse_datatype_f16_r;
+}
+
 template <>
 inline rocsparse_datatype get_datatype<float>(void)
 {
@@ -136,17 +143,21 @@ inline constexpr size_t rocsparse_datatype_sizeof(rocsparse_datatype datatype_)
 {
     switch(datatype_)
     {
+    case rocsparse_datatype_f16_r:
+    {
+        return sizeof(_Float16);
+    }
     case rocsparse_datatype_f32_r:
     {
         return sizeof(float);
     }
     case rocsparse_datatype_i8_r:
     {
-        return sizeof(char);
+        return sizeof(int8_t);
     }
     case rocsparse_datatype_u8_r:
     {
-        return sizeof(unsigned char);
+        return sizeof(uint8_t);
     }
     case rocsparse_datatype_i32_r:
     {
@@ -172,6 +183,12 @@ inline constexpr size_t rocsparse_datatype_sizeof(rocsparse_datatype datatype_)
     return static_cast<size_t>(0);
 }
 
+inline std::ostream& operator<<(std::ostream& os_, const _Float16& that_)
+{
+    os_ << (float)that_;
+    return os_;
+}
+
 /*! \brief  local handle which is automatically created and destroyed  */
 class rocsparse_local_handle
 {
@@ -182,6 +199,8 @@ public:
         : capture_started(false)
         , graph_testing(false)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_handle(&this->handle);
         if(status != rocsparse_status_success)
         {
@@ -192,6 +211,8 @@ public:
         : capture_started(false)
         , graph_testing(arg.graph_test)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_handle(&this->handle);
         if(status != rocsparse_status_success)
         {
@@ -200,6 +221,8 @@ public:
     }
     ~rocsparse_local_handle()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_handle(this->handle);
     }
 
@@ -215,6 +238,8 @@ public:
 
     void rocsparse_stream_begin_capture()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         if(!(this->graph_testing))
         {
             return;
@@ -236,6 +261,8 @@ public:
 
     void rocsparse_stream_end_capture(rocsparse_int runs = 1)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         if(!(this->graph_testing))
         {
             return;
@@ -286,6 +313,8 @@ class rocsparse_local_mat_descr
 public:
     rocsparse_local_mat_descr()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_mat_descr(&this->descr);
         if(status != rocsparse_status_success)
         {
@@ -295,6 +324,8 @@ public:
 
     ~rocsparse_local_mat_descr()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_mat_descr(this->descr);
     }
 
@@ -317,6 +348,8 @@ class rocsparse_local_mat_info
 public:
     rocsparse_local_mat_info()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_mat_info(&this->info);
         if(status != rocsparse_status_success)
         {
@@ -325,12 +358,16 @@ public:
     }
     ~rocsparse_local_mat_info()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_mat_info(this->info);
     }
 
     // Sometimes useful to reset local info
     void reset()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_mat_info(this->info);
         const rocsparse_status status = rocsparse_create_mat_info(&this->info);
         if(status != rocsparse_status_success)
@@ -358,6 +395,8 @@ class rocsparse_local_color_info
 public:
     rocsparse_local_color_info()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_color_info(&this->info);
         if(status != rocsparse_status_success)
         {
@@ -366,12 +405,16 @@ public:
     }
     ~rocsparse_local_color_info()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_color_info(this->info);
     }
 
     // Sometimes useful to reset local info
     void reset()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_color_info(this->info);
         const rocsparse_status status = rocsparse_create_color_info(&this->info);
         if(status != rocsparse_status_success)
@@ -415,6 +458,8 @@ class rocsparse_local_hyb_mat
 public:
     rocsparse_local_hyb_mat()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_hyb_mat(&this->hyb);
         if(status != rocsparse_status_success)
         {
@@ -423,6 +468,8 @@ public:
     }
     ~rocsparse_local_hyb_mat()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         rocsparse_destroy_hyb_mat(this->hyb);
     }
 
@@ -451,6 +498,8 @@ public:
                           rocsparse_index_base idx_base,
                           rocsparse_datatype   compute_type)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_spvec_descr(
             &this->descr, size, nnz, indices, values, idx_type, idx_base, compute_type);
         if(status != rocsparse_status_success)
@@ -460,6 +509,8 @@ public:
     }
     ~rocsparse_local_spvec()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         if(this->descr != nullptr)
         {
             rocsparse_destroy_spvec_descr(this->descr);
@@ -493,6 +544,8 @@ public:
                           rocsparse_index_base idx_base,
                           rocsparse_datatype   compute_type)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_coo_descr(&this->descr,
                                                                    m,
                                                                    n,
@@ -532,6 +585,8 @@ public:
                           rocsparse_index_base idx_base,
                           rocsparse_datatype   compute_type)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_coo_aos_descr(
             &this->descr, m, n, nnz, coo_ind, coo_val, idx_type, idx_base, compute_type);
         if(status != rocsparse_status_success)
@@ -557,10 +612,11 @@ public:
                           rocsparse_indextype  row_col_ind_type,
                           rocsparse_index_base idx_base,
                           rocsparse_datatype   compute_type,
-                          rocsparse_format     format)
+                          bool                 csc_format = false)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
 
-        if(format == rocsparse_format_csr)
+        if(csc_format == false)
         {
             const rocsparse_status status = rocsparse_create_csr_descr(&this->descr,
                                                                        m,
@@ -580,7 +636,6 @@ public:
         }
         else
         {
-            assert(format == rocsparse_format_csc);
             const rocsparse_status status = rocsparse_create_csc_descr(&this->descr,
                                                                        m,
                                                                        n,
@@ -615,8 +670,7 @@ public:
                                 get_indextype<J>(),
                                 h.base,
                                 get_datatype<T>(),
-                                (DIRECTION_ == rocsparse_direction_row) ? rocsparse_format_csr
-                                                                        : rocsparse_format_csc)
+                                (DIRECTION_ == rocsparse_direction_column))
     {
     }
 
@@ -634,6 +688,7 @@ public:
                           rocsparse_datatype   compute_type,
                           rocsparse_format     format)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
 
         if(format == rocsparse_format_bsr)
         {
@@ -690,6 +745,8 @@ public:
                           rocsparse_index_base idx_base,
                           rocsparse_datatype   compute_type)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_bell_descr(&this->descr,
                                                                     m,
                                                                     n,
@@ -716,6 +773,8 @@ public:
                           rocsparse_index_base idx_base,
                           rocsparse_datatype   compute_type)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_ell_descr(
             &this->descr, m, n, ell_col_ind, ell_val, ell_width, idx_type, idx_base, compute_type);
         if(status != rocsparse_status_success)
@@ -733,6 +792,8 @@ public:
 
     ~rocsparse_local_spmat()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         if(this->descr != nullptr)
             rocsparse_destroy_spmat_descr(this->descr);
     }
@@ -756,6 +817,8 @@ class rocsparse_local_dnvec
 public:
     rocsparse_local_dnvec(int64_t size, void* values, rocsparse_datatype compute_type)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status
             = rocsparse_create_dnvec_descr(&this->descr, size, values, compute_type);
         if(status != rocsparse_status_success)
@@ -772,6 +835,8 @@ public:
 
     ~rocsparse_local_dnvec()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         if(this->descr != nullptr)
             rocsparse_destroy_dnvec_descr(this->descr);
     }
@@ -800,6 +865,8 @@ public:
                           rocsparse_datatype compute_type,
                           rocsparse_order    order)
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         const rocsparse_status status = rocsparse_create_dnmat_descr(
             &this->descr, rows, cols, ld, values, compute_type, order);
         if(status != rocsparse_status_success)
@@ -816,6 +883,8 @@ public:
 
     ~rocsparse_local_dnmat()
     {
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+
         if(this->descr != nullptr)
             rocsparse_destroy_dnmat_descr(this->descr);
     }
@@ -850,4 +919,113 @@ std::string rocsparse_exepath();
 /*! \brief Return path where the test data file (rocsparse_test.data) is located */
 std::string rocsparse_datapath();
 
+namespace rocsparse_clients
+{
+
+    struct timer
+    {
+    private:
+        hipEvent_t m_start, m_stop;
+
+    public:
+        timer();
+        void  start();
+        float stop();
+        ~timer();
+    };
+
+    /*! \brief  repeated performance runs of a function with a set of arguments that computes
+   * the median of the mean of the wall-clock time.
+   */
+    template <typename T, typename... ARG>
+    double run_benchmark(const Arguments& arguments, T func, ARG&&... arg)
+    {
+        if(arguments.iters_inner == 0)
+        {
+            std::cerr << "error " << __FUNCTION__ << ": arguments.iters_inner is zero, exit program"
+                      << std::endl;
+            exit(rocsparse_status_invalid_value);
+        }
+
+        if(arguments.iters == 0)
+        {
+            std::cerr << "error " << __FUNCTION__ << ": arguments.iters is zero, exit program."
+                      << std::endl;
+            exit(rocsparse_status_invalid_value);
+        }
+
+        const int32_t n_cold_calls = 2;
+        const int32_t n_sub_calls  = arguments.iters_inner;
+        const int32_t n_calls      = arguments.iters;
+
+        for(int32_t iter = 0; iter < n_cold_calls; ++iter)
+        {
+            const rocsparse_status status = func(arg...);
+            if(status != rocsparse_status_success)
+            {
+                std::cerr << "error " << __FUNCTION__ << ": cold call failed." << std::endl;
+                exit(rocsparse_status_invalid_value);
+            }
+        }
+
+        std::vector<double> gpu_time(n_calls);
+
+        rocsparse_clients::timer t;
+        for(int32_t iter = 0; iter < n_calls; ++iter)
+        {
+            t.start();
+            for(int32_t sub_iter = 0; sub_iter < n_sub_calls; ++sub_iter)
+            {
+                (void)func(arg...);
+            }
+            const double t_microseconds = (t.stop() * 1000);
+            gpu_time[iter]              = t_microseconds / n_sub_calls;
+        }
+
+        std::sort(gpu_time.begin(), gpu_time.end());
+        const int32_t mid = n_calls / 2;
+        return (n_calls % 2 == 0) ? (gpu_time[mid] + gpu_time[mid - 1]) / 2 : gpu_time[mid];
+    }
+
+}
+
+#define ROCSPARSE_CLIENTS_RUN_BENCHMARK(arguments_, gpu_time_used_, func_)                         \
+    if(arguments_.iters_inner == 0)                                                                \
+    {                                                                                              \
+        std::cerr << "error " << __FUNCTION__ << ": arguments_.iters_inner is zero." << std::endl; \
+        CHECK_ROCSPARSE_ERROR(rocsparse_status_invalid_value);                                     \
+    }                                                                                              \
+                                                                                                   \
+    if(arguments_.iters == 0)                                                                      \
+    {                                                                                              \
+        std::cerr << "error " << __FUNCTION__ << ": arguments_.iters is zero." << std::endl;       \
+        CHECK_ROCSPARSE_ERROR(rocsparse_status_invalid_value);                                     \
+    }                                                                                              \
+                                                                                                   \
+    const int32_t n_cold_calls = 2;                                                                \
+    const int32_t n_sub_calls  = arguments_.iters_inner;                                           \
+    const int32_t n_calls      = arguments_.iters;                                                 \
+    for(int32_t iter = 0; iter < n_cold_calls; ++iter)                                             \
+    {                                                                                              \
+        CHECK_ROCSPARSE_ERROR(func_);                                                              \
+    }                                                                                              \
+                                                                                                   \
+    std::vector<double> gpu_time(n_calls);                                                         \
+                                                                                                   \
+    rocsparse_clients::timer t;                                                                    \
+    for(int32_t iter = 0; iter < n_calls; ++iter)                                                  \
+    {                                                                                              \
+        t.start();                                                                                 \
+        for(int32_t iter2 = 0; iter2 < n_sub_calls; ++iter2)                                       \
+        {                                                                                          \
+            (void)(func_);                                                                         \
+        }                                                                                          \
+        const double t_microseconds = (t.stop() * 1000);                                           \
+        gpu_time[iter]              = t_microseconds / n_sub_calls;                                \
+    }                                                                                              \
+                                                                                                   \
+    std::sort(gpu_time.begin(), gpu_time.end());                                                   \
+    const int32_t mid = n_calls / 2;                                                               \
+    gpu_time_used_ = (n_calls % 2 == 0) ? (gpu_time[mid] + gpu_time[mid - 1]) / 2 : gpu_time[mid];
+
 #endif // UTILITY_HPP
