@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -52,16 +52,16 @@ void testing_csrmm_bad_arg(const Arguments& arg)
     const T*             csr_val     = (const T*)0x4;
     const rocsparse_int* csr_row_ptr = (const rocsparse_int*)0x4;
     const rocsparse_int* csr_col_ind = (const rocsparse_int*)0x4;
-    const T*             B           = (const T*)0x4;
+    const T*             dense_B     = (const T*)0x4;
     rocsparse_int        ldb         = safe_size;
     const T              local_beta  = static_cast<T>(2);
     const T*             beta        = &local_beta;
-    T*                   C           = (T*)0x4;
+    T*                   dense_C     = (T*)0x4;
     rocsparse_int        ldc         = safe_size;
 
-#define PARAMS                                                                                  \
-    handle, trans_A, trans_B, m, n, k, nnz, alpha, descr, csr_val, csr_row_ptr, csr_col_ind, B, \
-        ldb, beta, C, ldc
+#define PARAMS                                                                               \
+    handle, trans_A, trans_B, m, n, k, nnz, alpha, descr, csr_val, csr_row_ptr, csr_col_ind, \
+        dense_B, ldb, beta, dense_C, ldc
 
     bad_arg_analysis(rocsparse_csrmm<T>, PARAMS);
 
@@ -318,24 +318,24 @@ void testing_csrmm(const Arguments& arg)
         {
             host_dense_matrix<T> hC_copy(hC);
             // CPU csrmm
-            host_csrmm<T, rocsparse_int, rocsparse_int>(M,
-                                                        N,
-                                                        K,
-                                                        transA,
-                                                        transB,
-                                                        *h_alpha,
-                                                        hA.ptr,
-                                                        hA.ind,
-                                                        hA.val,
-                                                        hB,
-                                                        hB.ld,
-                                                        order,
-                                                        *h_beta,
-                                                        hC,
-                                                        hC.ld,
-                                                        order,
-                                                        base,
-                                                        false);
+            host_csrmm<T, rocsparse_int, rocsparse_int, T, T, T>(M,
+                                                                 N,
+                                                                 K,
+                                                                 transA,
+                                                                 transB,
+                                                                 *h_alpha,
+                                                                 hA.ptr,
+                                                                 hA.ind,
+                                                                 hA.val,
+                                                                 hB,
+                                                                 hB.ld,
+                                                                 order,
+                                                                 *h_beta,
+                                                                 hC,
+                                                                 hC.ld,
+                                                                 order,
+                                                                 base,
+                                                                 false);
             hC.near_check(dC);
             dC = hC_copy;
         }
@@ -351,26 +351,11 @@ void testing_csrmm(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_csrmm<T>(PARAMS(h_alpha, dA, dB, h_beta, dC)));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_csrmm<T>(PARAMS(h_alpha, dA, dB, h_beta, dC)));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(
+            arg, rocsparse_csrmm<T>, PARAMS(h_alpha, dA, dB, h_beta, dC));
 
         double gflop_count = csrmm_gflop_count<rocsparse_int, rocsparse_int>(
             N, dA.nnz, dC.m * dC.n, *h_beta != static_cast<T>(0));
