@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 #include <tuple>
 
-template <typename I, typename T>
+template <typename I, typename A, typename B, typename C, typename T>
 void testing_spmm_batched_coo_bad_arg(const Arguments& arg)
 {
     static const size_t safe_size = 100;
@@ -41,8 +41,8 @@ void testing_spmm_batched_coo_bad_arg(const Arguments& arg)
     void*                coo_val     = (void*)0x4;
     void*                coo_row_ind = (void*)0x4;
     void*                coo_col_ind = (void*)0x4;
-    void*                B           = (void*)0x4;
-    void*                C           = (void*)0x4;
+    void*                dense_B     = (void*)0x4;
+    void*                dense_C     = (void*)0x4;
     size_t*              buffer_size = (size_t*)0x4;
     void*                temp_buffer = (void*)0x4;
     rocsparse_operation  trans_A     = rocsparse_operation_none;
@@ -54,6 +54,9 @@ void testing_spmm_batched_coo_bad_arg(const Arguments& arg)
     rocsparse_spmm_stage stage       = rocsparse_spmm_stage_compute;
 
     rocsparse_indextype itype = get_indextype<I>();
+    rocsparse_datatype  atype = get_datatype<A>();
+    rocsparse_datatype  btype = get_datatype<B>();
+    rocsparse_datatype  ctype = get_datatype<C>();
     rocsparse_datatype  ttype = get_datatype<T>();
 
     T alpha = static_cast<T>(1.0);
@@ -61,9 +64,9 @@ void testing_spmm_batched_coo_bad_arg(const Arguments& arg)
 
     // SpMM structures
     rocsparse_local_spmat local_mat_A(
-        m, k, nnz, coo_row_ind, coo_col_ind, coo_val, itype, base, ttype);
-    rocsparse_local_dnmat local_mat_B(k, n, k, B, ttype, order_B);
-    rocsparse_local_dnmat local_mat_C(m, n, m, C, ttype, order_C);
+        m, k, nnz, coo_row_ind, coo_col_ind, coo_val, itype, base, atype);
+    rocsparse_local_dnmat local_mat_B(k, n, k, dense_B, btype, order_B);
+    rocsparse_local_dnmat local_mat_C(m, n, m, dense_C, ctype, order_C);
 
     rocsparse_spmat_descr mat_A = local_mat_A;
     rocsparse_dnmat_descr mat_B = local_mat_B;
@@ -130,7 +133,7 @@ void testing_spmm_batched_coo_bad_arg(const Arguments& arg)
 #undef PARAMS
 }
 
-template <typename I, typename T>
+template <typename I, typename A, typename B, typename C, typename T>
 void testing_spmm_batched_coo(const Arguments& arg)
 {
     I                    M               = arg.M;
@@ -154,6 +157,9 @@ void testing_spmm_batched_coo(const Arguments& arg)
 
     // Index and data type
     rocsparse_indextype itype = get_indextype<I>();
+    rocsparse_datatype  atype = get_datatype<A>();
+    rocsparse_datatype  btype = get_datatype<B>();
+    rocsparse_datatype  ctype = get_datatype<C>();
     rocsparse_datatype  ttype = get_datatype<T>();
 
     // Create rocsparse handle
@@ -169,12 +175,12 @@ void testing_spmm_batched_coo(const Arguments& arg)
     }
 
     // Allocate host memory for matrix
-    rocsparse_matrix_factory<T, I, I> matrix_factory(arg);
+    rocsparse_matrix_factory<A, I, I> matrix_factory(arg);
 
     // Allocate host memory for matrix
     host_vector<I> hcoo_row_ind_temp;
     host_vector<I> hcoo_col_ind_temp;
-    host_vector<T> hcoo_val_temp;
+    host_vector<A> hcoo_val_temp;
 
     int64_t nnz_A;
     matrix_factory.init_coo(hcoo_row_ind_temp,
@@ -216,7 +222,7 @@ void testing_spmm_batched_coo(const Arguments& arg)
     // Allocate host memory for all batches of A matrix
     host_dense_vector<I> hcoo_row_ind(batch_count_A * nnz_A);
     host_dense_vector<I> hcoo_col_ind(batch_count_A * nnz_A);
-    host_dense_vector<T> hcoo_val(batch_count_A * nnz_A);
+    host_dense_vector<A> hcoo_val(batch_count_A * nnz_A);
 
     for(I i = 0; i < batch_count_A; i++)
     {
@@ -229,23 +235,23 @@ void testing_spmm_batched_coo(const Arguments& arg)
     }
 
     // Allocate host memory for vectors
-    host_vector<T> hB(batch_count_B * nnz_B);
-    host_vector<T> hC_1(batch_count_C * nnz_C);
+    host_vector<B> hB(batch_count_B * nnz_B);
+    host_vector<C> hC_1(batch_count_C * nnz_C);
 
     // Initialize data on CPU
-    rocsparse_init<T>(hB, batch_count_B * nnz_B, 1, 1);
-    rocsparse_init<T>(hC_1, batch_count_C * nnz_C, 1, 1);
+    rocsparse_init<B>(hB, batch_count_B * nnz_B, 1, 1);
+    rocsparse_init<C>(hC_1, batch_count_C * nnz_C, 1, 1);
 
-    host_vector<T> hC_2(hC_1);
-    host_vector<T> hC_gold(hC_1);
+    host_vector<C> hC_2(hC_1);
+    host_vector<C> hC_gold(hC_1);
 
     // Allocate device memory
     device_vector<I> dcoo_row_ind(hcoo_row_ind);
     device_vector<I> dcoo_col_ind(hcoo_col_ind);
-    device_vector<T> dcoo_val(hcoo_val);
-    device_vector<T> dB(hB);
-    device_vector<T> dC_1(hC_1);
-    device_vector<T> dC_2(hC_2);
+    device_vector<A> dcoo_val(hcoo_val);
+    device_vector<B> dB(hB);
+    device_vector<C> dC_1(hC_1);
+    device_vector<C> dC_2(hC_2);
     device_vector<T> dalpha(1);
     device_vector<T> dbeta(1);
 
@@ -253,20 +259,20 @@ void testing_spmm_batched_coo(const Arguments& arg)
     CHECK_HIP_ERROR(hipMemcpy(dbeta, &hbeta, sizeof(T), hipMemcpyHostToDevice));
 
     // Create descriptors
-    rocsparse_local_spmat A(
-        A_m, A_n, nnz_A, dcoo_row_ind, dcoo_col_ind, dcoo_val, itype, base, ttype);
+    rocsparse_local_spmat mat_A(
+        A_m, A_n, nnz_A, dcoo_row_ind, dcoo_col_ind, dcoo_val, itype, base, atype);
 
     ldb = std::max(int64_t(1), ldb);
     ldc = std::max(int64_t(1), ldc);
 
-    rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, order_B);
-    rocsparse_local_dnmat C1(C_m, C_n, ldc, dC_1, ttype, order_C);
-    rocsparse_local_dnmat C2(C_m, C_n, ldc, dC_2, ttype, order_C);
+    rocsparse_local_dnmat mat_B(B_m, B_n, ldb, dB, btype, order_B);
+    rocsparse_local_dnmat mat_C1(C_m, C_n, ldc, dC_1, ctype, order_C);
+    rocsparse_local_dnmat mat_C2(C_m, C_n, ldc, dC_2, ctype, order_C);
 
-    CHECK_ROCSPARSE_ERROR(rocsparse_coo_set_strided_batch(A, batch_count_A, batch_stride_A));
-    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(B, batch_count_B, batch_stride_B));
-    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(C1, batch_count_C, batch_stride_C));
-    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(C2, batch_count_C, batch_stride_C));
+    CHECK_ROCSPARSE_ERROR(rocsparse_coo_set_strided_batch(mat_A, batch_count_A, batch_stride_A));
+    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(mat_B, batch_count_B, batch_stride_B));
+    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(mat_C1, batch_count_C, batch_stride_C));
+    CHECK_ROCSPARSE_ERROR(rocsparse_dnmat_set_strided_batch(mat_C2, batch_count_C, batch_stride_C));
 
     // Query SpMM buffer
     size_t buffer_size;
@@ -274,10 +280,10 @@ void testing_spmm_batched_coo(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_buffer_size,
@@ -292,10 +298,10 @@ void testing_spmm_batched_coo(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_preprocess,
@@ -310,10 +316,10 @@ void testing_spmm_batched_coo(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       &halpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       &hbeta,
-                                                      C1,
+                                                      mat_C1,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
@@ -326,10 +332,10 @@ void testing_spmm_batched_coo(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       dalpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       dbeta,
-                                                      C2,
+                                                      mat_C2,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
@@ -341,81 +347,55 @@ void testing_spmm_batched_coo(const Arguments& arg)
         hC_2.transfer_from(dC_2);
 
         // CPU coomm_batched
-        host_coomm_batched<T, I>(A_m,
-                                 N,
-                                 A_n,
-                                 nnz_A,
-                                 batch_count_A,
-                                 batch_stride_A,
-                                 trans_A,
-                                 trans_B,
-                                 halpha,
-                                 hcoo_row_ind.data(),
-                                 hcoo_col_ind.data(),
-                                 hcoo_val.data(),
-                                 hB.data(),
-                                 ldb,
-                                 batch_count_B,
-                                 batch_stride_B,
-                                 order_B,
-                                 hbeta,
-                                 hC_gold.data(),
-                                 ldc,
-                                 batch_count_C,
-                                 batch_stride_C,
-                                 order_C,
-                                 base);
+        host_coomm_batched<T, I, A, B, C>(A_m,
+                                          N,
+                                          A_n,
+                                          nnz_A,
+                                          batch_count_A,
+                                          batch_stride_A,
+                                          trans_A,
+                                          trans_B,
+                                          halpha,
+                                          hcoo_row_ind.data(),
+                                          hcoo_col_ind.data(),
+                                          hcoo_val.data(),
+                                          hB.data(),
+                                          ldb,
+                                          batch_count_B,
+                                          batch_stride_B,
+                                          order_B,
+                                          hbeta,
+                                          hC_gold.data(),
+                                          ldc,
+                                          batch_count_C,
+                                          batch_stride_C,
+                                          order_C,
+                                          base);
 
-        hC_gold.near_check(hC_1, get_near_check_tol<T>(arg));
-        hC_gold.near_check(hC_2, get_near_check_tol<T>(arg));
+        hC_gold.near_check(hC_1, get_near_check_tol<C>(arg));
+        hC_gold.near_check(hC_2, get_near_check_tol<C>(arg));
     }
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 &halpha,
-                                                 A,
-                                                 B,
-                                                 &hbeta,
-                                                 C1,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 &halpha,
-                                                 A,
-                                                 B,
-                                                 &hbeta,
-                                                 C1,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_spmm,
+                                                                      handle,
+                                                                      trans_A,
+                                                                      trans_B,
+                                                                      &halpha,
+                                                                      mat_A,
+                                                                      mat_B,
+                                                                      &hbeta,
+                                                                      mat_C1,
+                                                                      ttype,
+                                                                      alg,
+                                                                      rocsparse_spmm_stage_compute,
+                                                                      &buffer_size,
+                                                                      dbuffer);
 
         double gflop_count
             = batch_count_C
@@ -463,9 +443,14 @@ void testing_spmm_batched_coo(const Arguments& arg)
     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
 }
 
-#define INSTANTIATE(ITYPE, TTYPE)                                                       \
-    template void testing_spmm_batched_coo_bad_arg<ITYPE, TTYPE>(const Arguments& arg); \
-    template void testing_spmm_batched_coo<ITYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE(ITYPE, TTYPE)                                                      \
+    template void testing_spmm_batched_coo_bad_arg<ITYPE, TTYPE, TTYPE, TTYPE, TTYPE>( \
+        const Arguments& arg);                                                         \
+    template void testing_spmm_batched_coo<ITYPE, TTYPE, TTYPE, TTYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE_MIXED(ITYPE, ATYPE, XTYPE, YTYPE, TTYPE)                           \
+    template void testing_spmm_batched_coo_bad_arg<ITYPE, ATYPE, XTYPE, YTYPE, TTYPE>( \
+        const Arguments& arg);                                                         \
+    template void testing_spmm_batched_coo<ITYPE, ATYPE, XTYPE, YTYPE, TTYPE>(const Arguments& arg)
 
 INSTANTIATE(int32_t, float);
 INSTANTIATE(int32_t, double);
@@ -475,4 +460,12 @@ INSTANTIATE(int64_t, float);
 INSTANTIATE(int64_t, double);
 INSTANTIATE(int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, rocsparse_double_complex);
+
+INSTANTIATE_MIXED(int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int32_t, _Float16, _Float16, float, float);
+INSTANTIATE_MIXED(int64_t, _Float16, _Float16, float, float);
+
 void testing_spmm_batched_coo_extra(const Arguments& arg) {}
