@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -286,24 +286,24 @@ void testing_spmm_bell(const Arguments& arg)
                 }
             }
 
-            host_coomm<T, I>(M,
-                             N,
-                             K,
-                             nnzb * block_dim * block_dim,
-                             trans_A,
-                             trans_B,
-                             *h_alpha,
-                             coo_row,
-                             coo_col,
-                             coo_val,
-                             hB,
-                             (order_B == rocsparse_order_column) ? hB.m : hB.n,
-                             order_B,
-                             *h_beta,
-                             hC,
-                             (order_C == rocsparse_order_column) ? hC.m : hC.n,
-                             order_C,
-                             base);
+            host_coomm<T, I, T, T, T>(M,
+                                      N,
+                                      K,
+                                      nnzb * block_dim * block_dim,
+                                      trans_A,
+                                      trans_B,
+                                      *h_alpha,
+                                      coo_row,
+                                      coo_col,
+                                      coo_val,
+                                      hB,
+                                      (order_B == rocsparse_order_column) ? hB.m : hB.n,
+                                      order_B,
+                                      *h_beta,
+                                      hC,
+                                      (order_C == rocsparse_order_column) ? hC.m : hC.n,
+                                      order_C,
+                                      base);
 
             delete[] coo_val;
             delete[] coo_col;
@@ -342,50 +342,25 @@ void testing_spmm_bell(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 h_alpha,
-                                                 A,
-                                                 B,
-                                                 h_beta,
-                                                 C,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_spmm,
+                                                                      handle,
+                                                                      trans_A,
+                                                                      trans_B,
+                                                                      h_alpha,
+                                                                      A,
+                                                                      B,
+                                                                      h_beta,
+                                                                      C,
+                                                                      ttype,
+                                                                      alg,
+                                                                      rocsparse_spmm_stage_compute,
+                                                                      &buffer_size,
+                                                                      dbuffer);
 
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 h_alpha,
-                                                 A,
-                                                 B,
-                                                 h_beta,
-                                                 C,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        gpu_time_used      = (get_time_us() - gpu_time_used) / number_hot_calls;
         double gflop_count = spmm_gflop_count(
             N, dA.nnz, (int64_t)dC.m * (int64_t)dC.n, *h_beta != static_cast<T>(0));
         double gpu_gflops = get_gpu_gflops(gpu_time_used, gflop_count);
