@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 #include <tuple>
 
-template <typename I, typename J, typename T>
+template <typename I, typename J, typename A, typename B, typename C, typename T>
 void testing_spmm_csr_bad_arg(const Arguments& arg)
 {
     static const size_t safe_size = 100;
@@ -43,8 +43,8 @@ void testing_spmm_csr_bad_arg(const Arguments& arg)
     void*                csr_val     = (void*)0x4;
     void*                csr_row_ptr = (void*)0x4;
     void*                csr_col_ind = (void*)0x4;
-    void*                B           = (void*)0x4;
-    void*                C           = (void*)0x4;
+    void*                dB          = (void*)0x4;
+    void*                dC          = (void*)0x4;
     rocsparse_operation  trans_A     = rocsparse_operation_none;
     rocsparse_operation  trans_B     = rocsparse_operation_none;
     rocsparse_index_base base        = rocsparse_index_base_zero;
@@ -55,13 +55,16 @@ void testing_spmm_csr_bad_arg(const Arguments& arg)
 
     rocsparse_indextype itype        = get_indextype<I>();
     rocsparse_indextype jtype        = get_indextype<J>();
+    rocsparse_datatype  atype        = get_datatype<A>();
+    rocsparse_datatype  btype        = get_datatype<B>();
+    rocsparse_datatype  ctype        = get_datatype<C>();
     rocsparse_datatype  compute_type = get_datatype<T>();
 
     // SpMM structures
     rocsparse_local_spmat local_mat_A(
-        m, k, nnz, csr_row_ptr, csr_col_ind, csr_val, itype, jtype, base, compute_type);
-    rocsparse_local_dnmat local_mat_B(k, n, k, B, compute_type, order_B);
-    rocsparse_local_dnmat local_mat_C(m, n, m, C, compute_type, order_C);
+        m, k, nnz, csr_row_ptr, csr_col_ind, csr_val, itype, jtype, base, atype);
+    rocsparse_local_dnmat local_mat_B(k, n, k, dB, btype, order_B);
+    rocsparse_local_dnmat local_mat_C(m, n, m, dC, ctype, order_C);
 
     rocsparse_spmat_descr mat_A = local_mat_A;
     rocsparse_dnmat_descr mat_B = local_mat_B;
@@ -99,7 +102,7 @@ void testing_spmm_csr_bad_arg(const Arguments& arg)
 #undef PARAMS
 }
 
-template <typename I, typename J, typename T>
+template <typename I, typename J, typename A, typename B, typename C, typename T>
 void testing_spmm_csr(const Arguments& arg)
 {
     J                    M               = arg.M;
@@ -120,6 +123,9 @@ void testing_spmm_csr(const Arguments& arg)
     // Index and data type
     rocsparse_indextype itype = get_indextype<I>();
     rocsparse_indextype jtype = get_indextype<J>();
+    rocsparse_datatype  atype = get_datatype<A>();
+    rocsparse_datatype  btype = get_datatype<B>();
+    rocsparse_datatype  ctype = get_datatype<C>();
     rocsparse_datatype  ttype = get_datatype<T>();
 
     // Create rocsparse handle
@@ -128,10 +134,10 @@ void testing_spmm_csr(const Arguments& arg)
     // Allocate host memory for matrix
     host_vector<I> hcsr_row_ptr;
     host_vector<J> hcsr_col_ind;
-    host_vector<T> hcsr_val;
+    host_vector<A> hcsr_val;
 
     // Allocate host memory for matrix
-    rocsparse_matrix_factory<T, I, J> matrix_factory(arg);
+    rocsparse_matrix_factory<A, I, J> matrix_factory(arg);
 
     I nnz_A;
     matrix_factory.init_csr(hcsr_row_ptr,
@@ -167,14 +173,14 @@ void testing_spmm_csr(const Arguments& arg)
     int64_t nnz_C = nrowC * ncolC;
 
     // Allocate host memory for vectors
-    host_vector<T> hB(nnz_B);
-    host_vector<T> hC_1(nnz_C);
-    host_vector<T> hC_2(nnz_C);
-    host_vector<T> hC_gold(nnz_C);
+    host_vector<B> hB(nnz_B);
+    host_vector<C> hC_1(nnz_C);
+    host_vector<C> hC_2(nnz_C);
+    host_vector<C> hC_gold(nnz_C);
 
     // Initialize data on CPU
-    rocsparse_init<T>(hB, nnz_B, 1, 1);
-    rocsparse_init<T>(hC_1, nnz_C, 1, 1);
+    rocsparse_init<B>(hB, nnz_B, 1, 1);
+    rocsparse_init<C>(hC_1, nnz_C, 1, 1);
 
     hC_2    = hC_1;
     hC_gold = hC_1;
@@ -182,10 +188,10 @@ void testing_spmm_csr(const Arguments& arg)
     // Allocate device memory
     device_vector<I> dcsr_row_ptr(A_m + 1);
     device_vector<J> dcsr_col_ind(nnz_A);
-    device_vector<T> dcsr_val(nnz_A);
-    device_vector<T> dB(nnz_B);
-    device_vector<T> dC_1(nnz_C);
-    device_vector<T> dC_2(nnz_C);
+    device_vector<A> dcsr_val(nnz_A);
+    device_vector<B> dB(nnz_B);
+    device_vector<C> dC_1(nnz_C);
+    device_vector<C> dC_2(nnz_C);
     device_vector<T> dalpha(1);
     device_vector<T> dbeta(1);
 
@@ -194,23 +200,23 @@ void testing_spmm_csr(const Arguments& arg)
         hipMemcpy(dcsr_row_ptr, hcsr_row_ptr.data(), sizeof(I) * (A_m + 1), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(
         hipMemcpy(dcsr_col_ind, hcsr_col_ind.data(), sizeof(J) * nnz_A, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dcsr_val, hcsr_val.data(), sizeof(T) * nnz_A, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dB, hB, sizeof(T) * nnz_B, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dC_1, hC_1, sizeof(T) * nnz_C, hipMemcpyHostToDevice));
-    CHECK_HIP_ERROR(hipMemcpy(dC_2, hC_2, sizeof(T) * nnz_C, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dcsr_val, hcsr_val.data(), sizeof(A) * nnz_A, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dB, hB, sizeof(B) * nnz_B, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dC_1, hC_1, sizeof(C) * nnz_C, hipMemcpyHostToDevice));
+    CHECK_HIP_ERROR(hipMemcpy(dC_2, hC_2, sizeof(C) * nnz_C, hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dalpha, &halpha, sizeof(T), hipMemcpyHostToDevice));
     CHECK_HIP_ERROR(hipMemcpy(dbeta, &hbeta, sizeof(T), hipMemcpyHostToDevice));
 
     // Create descriptors
-    rocsparse_local_spmat A(
-        A_m, A_n, nnz_A, dcsr_row_ptr, dcsr_col_ind, dcsr_val, itype, jtype, base, ttype);
+    rocsparse_local_spmat mat_A(
+        A_m, A_n, nnz_A, dcsr_row_ptr, dcsr_col_ind, dcsr_val, itype, jtype, base, atype);
 
     ldb = std::max(int64_t(1), ldb);
     ldc = std::max(int64_t(1), ldc);
 
-    rocsparse_local_dnmat B(B_m, B_n, ldb, dB, ttype, order_B);
-    rocsparse_local_dnmat C1(C_m, C_n, ldc, dC_1, ttype, order_C);
-    rocsparse_local_dnmat C2(C_m, C_n, ldc, dC_2, ttype, order_C);
+    rocsparse_local_dnmat mat_B(B_m, B_n, ldb, dB, btype, order_B);
+    rocsparse_local_dnmat mat_C1(C_m, C_n, ldc, dC_1, ctype, order_C);
+    rocsparse_local_dnmat mat_C2(C_m, C_n, ldc, dC_2, ctype, order_C);
 
     // Query SpMM buffer
     size_t buffer_size;
@@ -218,10 +224,10 @@ void testing_spmm_csr(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_buffer_size,
@@ -236,10 +242,10 @@ void testing_spmm_csr(const Arguments& arg)
                                          trans_A,
                                          trans_B,
                                          &halpha,
-                                         A,
-                                         B,
+                                         mat_A,
+                                         mat_B,
                                          &hbeta,
-                                         C1,
+                                         mat_C1,
                                          ttype,
                                          alg,
                                          rocsparse_spmm_stage_preprocess,
@@ -256,10 +262,10 @@ void testing_spmm_csr(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       &halpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       &hbeta,
-                                                      C1,
+                                                      mat_C1,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
@@ -276,10 +282,10 @@ void testing_spmm_csr(const Arguments& arg)
                                                       trans_A,
                                                       trans_B,
                                                       dalpha,
-                                                      A,
-                                                      B,
+                                                      mat_A,
+                                                      mat_B,
                                                       dbeta,
-                                                      C2,
+                                                      mat_C2,
                                                       ttype,
                                                       alg,
                                                       rocsparse_spmm_stage_compute,
@@ -292,79 +298,53 @@ void testing_spmm_csr(const Arguments& arg)
         }
 
         // Copy output to host
-        CHECK_HIP_ERROR(hipMemcpy(hC_1, dC_1, sizeof(T) * nnz_C, hipMemcpyDeviceToHost));
-        CHECK_HIP_ERROR(hipMemcpy(hC_2, dC_2, sizeof(T) * nnz_C, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hC_1, dC_1, sizeof(C) * nnz_C, hipMemcpyDeviceToHost));
+        CHECK_HIP_ERROR(hipMemcpy(hC_2, dC_2, sizeof(C) * nnz_C, hipMemcpyDeviceToHost));
 
         // CPU csrmm
-        host_csrmm<T, I, J>(A_m,
-                            N,
-                            A_n,
-                            trans_A,
-                            trans_B,
-                            halpha,
-                            hcsr_row_ptr,
-                            hcsr_col_ind,
-                            hcsr_val,
-                            hB,
-                            ldb,
-                            order_B,
-                            hbeta,
-                            hC_gold,
-                            ldc,
-                            order_C,
-                            base,
-                            false);
+        host_csrmm<T, I, J, A, B, C>(A_m,
+                                     N,
+                                     A_n,
+                                     trans_A,
+                                     trans_B,
+                                     halpha,
+                                     hcsr_row_ptr,
+                                     hcsr_col_ind,
+                                     hcsr_val,
+                                     hB,
+                                     ldb,
+                                     order_B,
+                                     hbeta,
+                                     hC_gold,
+                                     ldc,
+                                     order_C,
+                                     base,
+                                     false);
 
-        hC_gold.near_check(hC_1, get_near_check_tol<T>(arg));
-        hC_gold.near_check(hC_2, get_near_check_tol<T>(arg));
+        hC_gold.near_check(hC_1, get_near_check_tol<C>(arg));
+        hC_gold.near_check(hC_2, get_near_check_tol<C>(arg));
     }
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 &halpha,
-                                                 A,
-                                                 B,
-                                                 &hbeta,
-                                                 C1,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spmm(handle,
-                                                 trans_A,
-                                                 trans_B,
-                                                 &halpha,
-                                                 A,
-                                                 B,
-                                                 &hbeta,
-                                                 C1,
-                                                 ttype,
-                                                 alg,
-                                                 rocsparse_spmm_stage_compute,
-                                                 &buffer_size,
-                                                 dbuffer));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_spmm,
+                                                                      handle,
+                                                                      trans_A,
+                                                                      trans_B,
+                                                                      &halpha,
+                                                                      mat_A,
+                                                                      mat_B,
+                                                                      &hbeta,
+                                                                      mat_C1,
+                                                                      ttype,
+                                                                      alg,
+                                                                      rocsparse_spmm_stage_compute,
+                                                                      &buffer_size,
+                                                                      dbuffer);
 
         double gflop_count
             = spmm_gflop_count(N, nnz_A, (I)C_m * (I)C_n, hbeta != static_cast<T>(0));
@@ -399,9 +379,14 @@ void testing_spmm_csr(const Arguments& arg)
     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
 }
 
-#define INSTANTIATE(ITYPE, JTYPE, TTYPE)                                               \
-    template void testing_spmm_csr_bad_arg<ITYPE, JTYPE, TTYPE>(const Arguments& arg); \
-    template void testing_spmm_csr<ITYPE, JTYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE(ITYPE, JTYPE, TTYPE)                                              \
+    template void testing_spmm_csr_bad_arg<ITYPE, JTYPE, TTYPE, TTYPE, TTYPE, TTYPE>( \
+        const Arguments& arg);                                                        \
+    template void testing_spmm_csr<ITYPE, JTYPE, TTYPE, TTYPE, TTYPE, TTYPE>(const Arguments& arg)
+#define INSTANTIATE_MIXED(ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE)                   \
+    template void testing_spmm_csr_bad_arg<ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE>( \
+        const Arguments& arg);                                                        \
+    template void testing_spmm_csr<ITYPE, JTYPE, ATYPE, XTYPE, YTYPE, TTYPE>(const Arguments& arg)
 
 INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
@@ -415,4 +400,15 @@ INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
+
+INSTANTIATE_MIXED(int32_t, int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int32_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int64_t, int64_t, int8_t, int8_t, int32_t, int32_t);
+INSTANTIATE_MIXED(int32_t, int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int32_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int64_t, int64_t, int8_t, int8_t, float, float);
+INSTANTIATE_MIXED(int32_t, int32_t, _Float16, _Float16, float, float);
+INSTANTIATE_MIXED(int64_t, int32_t, _Float16, _Float16, float, float);
+INSTANTIATE_MIXED(int64_t, int64_t, _Float16, _Float16, float, float);
+
 void testing_spmm_csr_extra(const Arguments& arg) {}
