@@ -1,5 +1,5 @@
 /* ************************************************************************
-* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -89,13 +89,14 @@ void testing_spsv_coo_bad_arg(const Arguments& arg)
 template <typename I, typename T>
 void testing_spsv_coo(const Arguments& arg)
 {
-    I                    M       = arg.M;
-    I                    N       = arg.N;
-    rocsparse_operation  trans_A = arg.transA;
-    rocsparse_index_base base    = arg.baseA;
-    rocsparse_spsv_alg   alg     = arg.spsv_alg;
-    rocsparse_diag_type  diag    = arg.diag;
-    rocsparse_fill_mode  uplo    = arg.uplo;
+    I                     M           = arg.M;
+    I                     N           = arg.N;
+    rocsparse_operation   trans_A     = arg.transA;
+    rocsparse_index_base  base        = arg.baseA;
+    rocsparse_spsv_alg    alg         = arg.spsv_alg;
+    rocsparse_diag_type   diag        = arg.diag;
+    rocsparse_fill_mode   uplo        = arg.uplo;
+    rocsparse_matrix_type matrix_type = arg.matrix_type;
 
     rocsparse_spsv_stage buffersize = rocsparse_spsv_stage_buffer_size;
     rocsparse_spsv_stage preprocess = rocsparse_spsv_stage_preprocess;
@@ -172,6 +173,9 @@ void testing_spsv_coo(const Arguments& arg)
     CHECK_ROCSPARSE_ERROR(
         rocsparse_spmat_set_attribute(A, rocsparse_spmat_diag_type, &diag, sizeof(diag)));
 
+    CHECK_ROCSPARSE_ERROR(rocsparse_spmat_set_attribute(
+        A, rocsparse_spmat_matrix_type, &matrix_type, sizeof(matrix_type)));
+
     // Query SpSV buffer
     size_t buffer_size;
     CHECK_ROCSPARSE_ERROR(rocsparse_spsv(
@@ -242,28 +246,22 @@ void testing_spsv_coo(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
         CHECK_ROCSPARSE_ERROR(rocsparse_set_pointer_mode(handle, rocsparse_pointer_mode_host));
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spsv(
-                handle, trans_A, &halpha, A, x, y1, ttype, alg, compute, &buffer_size, dbuffer));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_spsv(
-                handle, trans_A, &halpha, A, x, y1, ttype, alg, compute, &buffer_size, dbuffer));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_spsv,
+                                                                      handle,
+                                                                      trans_A,
+                                                                      &halpha,
+                                                                      A,
+                                                                      x,
+                                                                      y1,
+                                                                      ttype,
+                                                                      alg,
+                                                                      compute,
+                                                                      &buffer_size,
+                                                                      dbuffer);
 
         double gflop_count = spsv_gflop_count(M, nnz_A, diag);
         double gpu_gflops  = get_gpu_gflops(gpu_time_used, gflop_count);
