@@ -1,4 +1,26 @@
 #!/usr/bin/env bash
+# ########################################################################
+# Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
+#
+# ########################################################################
 # Author: rocsparse-maintainer@amd.com
 
 #set -x #echo on
@@ -22,6 +44,8 @@ function display_help()
   echo "    [--hip-clang] build library for amdgpu backend using hip-clang"
   echo "    [-s|--static] build static library"
   echo "    [--memstat] build with memory statistics enabled."
+  echo "    [--offload-compress] Apply offload compression (enabled by default)."
+  echo "    [--no-offload-compress] Do not apply offload compression."
   echo "    [--rocsparse_ILP64] build with rocsparse_int equal to int64_t."
   echo "    [--address-sanitizer] build with address sanitizer"
   echo "    [--codecoverage] build with code coverage profiling enabled"
@@ -30,7 +54,8 @@ function display_help()
   echo "    [--matrices-dir] existing client matrices directory"
   echo "    [--matrices-dir-install] install client matrices directory"
   echo "    [--rm-legacy-include-dir] Remove legacy include dir Packaging added for file/folder reorg backward compatibility."
-  echo "    [--without-rocblas] Disable building rocSPARSE with rocBLAS."
+  echo "    [--no-rocblas] Disable building rocSPARSE with rocBLAS."
+  echo "    [--no-roctx] Disable building rocSPARSE with rocTX."
   echo "    [--cmake-arg] Forward the given argument to CMake when configuring the build."
 }
 
@@ -283,6 +308,8 @@ build_address_sanitizer=false
 build_memstat=false
 build_rocsparse_ILP64=false
 build_with_rocblas=true
+build_with_roctx=true
+build_with_offload_compress=true
 matrices_dir=
 matrices_dir_install=
 gpu_architecture=all
@@ -297,7 +324,7 @@ declare -a cmake_client_options
 # check if we have a modern version of getopt that can handle whitespace and long parameters
 getopt -T
 if [[ $? -eq 4 ]]; then
- GETOPT_PARSE=$(getopt --name "${0}" --longoptions help,install,clients,dependencies,debug,hip-clang,static,relocatable,codecoverage,relwithdebinfo,memstat,rocsparse_ILP64,rocprim-path:,rocblas-path:,without-rocblas,address-sanitizer,matrices-dir:,matrices-dir-install:,architecture:,rm-legacy-include-dir,cmake-arg: --options hicdgrska: -- "$@")
+ GETOPT_PARSE=$(getopt --name "${0}" --longoptions help,install,clients,dependencies,debug,hip-clang,static,relocatable,codecoverage,relwithdebinfo,memstat,rocsparse_ILP64,rocprim-path:,rocblas-path:,no-offload-compress,offload-compress,no-rocblas,no-roctx,address-sanitizer,matrices-dir:,matrices-dir-install:,architecture:,rm-legacy-include-dir,cmake-arg: --options hicdgrska: -- "$@")
 
 else
   echo "Need a new version of getopt"
@@ -353,9 +380,18 @@ while true; do
         --rocblas-path)
             rocblas_path=${2}
             shift 2 ;;
-        --without-rocblas)
+	--no-offload-compress)
+            build_with_offload_compress=false
+            shift ;;
+        --offload-compress)
+            build_with_offload_compress=true
+            shift ;;
+        --no-rocblas)
             build_with_rocblas=false
             shift ;;
+        --no-roctx)
+            build_with_roctx=false
+            shift ;;
         -k|--relwithdebinfo)
             build_release=false
             build_release_debug=true
@@ -515,7 +551,7 @@ pushd .
   # #################################################
   # configure & build
   # #################################################
-  cmake_common_options+=("--toolchain=toolchain-linux.cmake -DAMDGPU_TARGETS=${gpu_architecture}")
+  cmake_common_options+=("--toolchain=toolchain-linux.cmake -DGPU_TARGETS=${gpu_architecture}")
 
   # build type
   if [[ "${build_release}" == true ]]; then
@@ -545,13 +581,27 @@ pushd .
     cmake_common_options+=("-DBUILD_ROCSPARSE_ILP64=ON")
   fi
 
-  # without-rocblas
+  # offload-compress
+  if [[ "${build_with_offload_compress}" == true ]]; then
+    cmake_common_options+=("-DBUILD_WITH_OFFLOAD_COMPRESS=ON")
+  else
+    cmake_common_options+=("-DBUILD_WITH_OFFLOAD_COMPRESS=OFF")
+  fi
+
+  # no-rocblas
   if [[ "${build_with_rocblas}" == true ]]; then
     cmake_common_options+=("-DBUILD_WITH_ROCBLAS=ON")
   else
     cmake_common_options+=("-DBUILD_WITH_ROCBLAS=OFF")
   fi
 
+  # roctx
+  if [[ "${build_with_roctx}" == true ]]; then
+    cmake_common_options+=("-DBUILD_WITH_ROCTX=ON")
+  else
+    cmake_common_options+=("-DBUILD_WITH_ROCTX=OFF")
+  fi
+
   # freorg backward compatible support enable
   if [[ "${build_freorg_bkwdcomp}" == true ]]; then
     cmake_common_options+=("-DBUILD_FILE_REORG_BACKWARD_COMPATIBILITY=ON")
