@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -21,16 +21,15 @@
 * THE SOFTWARE.
 *
 * ************************************************************************ */
-#include "control.h"
-#include "utility.h"
+#include "rocsparse_control.hpp"
+#include "rocsparse_utility.hpp"
 
 #include "internal/conversion/rocsparse_coo2dense.h"
+#include "rocsparse_common.h"
 #include "rocsparse_coo2dense.hpp"
 
-#include "common.h"
 #include "coo2dense_device.h"
-
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_common.hpp"
 
 template <typename I, typename T>
 rocsparse_status rocsparse::coo2dense_template(rocsparse_handle          handle, //0
@@ -45,6 +44,7 @@ rocsparse_status rocsparse::coo2dense_template(rocsparse_handle          handle,
                                                int64_t                   lda, //9
                                                rocsparse_order           order) //10
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -90,17 +90,7 @@ rocsparse_status rocsparse::coo2dense_template(rocsparse_handle          handle,
     // RETURN_IF_HIP_ERROR(hipMemset2DAsync(A, sizeof(T) * lda, 0, sizeof(T) * mn, nm, stream));
 
     // Set memory to zero.
-    RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::memset2d_kernel<512>),
-                                       dim3((m * n - 1) / 512 + 1),
-                                       dim3(512),
-                                       0,
-                                       stream,
-                                       m,
-                                       n,
-                                       static_cast<T>(0),
-                                       A,
-                                       lda,
-                                       order);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset_2d(handle, m, n, lda, static_cast<T>(0), A, order));
 
     if(nnz > 0)
     {
@@ -143,10 +133,12 @@ rocsparse_status rocsparse::coo2dense_template(rocsparse_handle          handle,
         int64_t                   lda,                                     \
         rocsparse_order           order);
 
+INSTANTIATE(int32_t, _Float16);
 INSTANTIATE(int32_t, float);
 INSTANTIATE(int32_t, double);
 INSTANTIATE(int32_t, rocsparse_float_complex);
 INSTANTIATE(int32_t, rocsparse_double_complex);
+INSTANTIATE(int64_t, _Float16);
 INSTANTIATE(int64_t, float);
 INSTANTIATE(int64_t, double);
 INSTANTIATE(int64_t, rocsparse_float_complex);
@@ -172,6 +164,7 @@ INSTANTIATE(int64_t, rocsparse_double_complex);
                                      rocsparse_int             ld)                        \
     try                                                                                   \
     {                                                                                     \
+        ROCSPARSE_ROUTINE_TRACE;                                                          \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::coo2dense_template(handle,                   \
                                                                 m,                        \
                                                                 n,                        \
