@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,10 @@
  * ************************************************************************ */
 
 #include "rocsparse_coo2dense_aos.hpp"
-#include "common.h"
-#include "control.h"
-#include "utility.h"
+#include "rocsparse_common.h"
+#include "rocsparse_common.hpp"
+#include "rocsparse_control.hpp"
+#include "rocsparse_utility.hpp"
 
 #include "coo2dense_device.h"
 
@@ -41,6 +42,8 @@ rocsparse_status rocsparse::coo2dense_aos_template(rocsparse_handle          han
                                                    int64_t                   lda,
                                                    rocsparse_order           order)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for valid handle and matrix descriptor
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
@@ -97,17 +100,13 @@ rocsparse_status rocsparse::coo2dense_aos_template(rocsparse_handle          han
     // RETURN_IF_HIP_ERROR(hipMemset2DAsync(A, sizeof(T) * lda, 0, sizeof(T) * mn, nm, stream));
 
     // Set memory to zero.
-    RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::memset2d_kernel<512>),
-                                       dim3((m * n - 1) / 512 + 1),
-                                       dim3(512),
-                                       0,
-                                       stream,
-                                       static_cast<int64_t>(m),
-                                       static_cast<int64_t>(n),
-                                       static_cast<T>(0),
-                                       A,
-                                       lda,
-                                       order);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset_2d(handle,
+                                                   static_cast<int64_t>(m),
+                                                   static_cast<int64_t>(n),
+                                                   lda,
+                                                   static_cast<T>(0),
+                                                   A,
+                                                   order));
 
 #define COO2DENSE_DIM 512
     const int64_t num_blocks_x = rocsparse::min(
@@ -147,10 +146,12 @@ rocsparse_status rocsparse::coo2dense_aos_template(rocsparse_handle          han
         int64_t                   lda,                                         \
         rocsparse_order           order)
 
+INSTANTIATE(int32_t, _Float16);
 INSTANTIATE(int32_t, float);
 INSTANTIATE(int32_t, double);
 INSTANTIATE(int32_t, rocsparse_float_complex);
 INSTANTIATE(int32_t, rocsparse_double_complex);
+INSTANTIATE(int64_t, _Float16);
 INSTANTIATE(int64_t, float);
 INSTANTIATE(int64_t, double);
 INSTANTIATE(int64_t, rocsparse_float_complex);
