@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2024-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,7 @@
  * ************************************************************************ */
 
 #include "rocsparse_extract_alg_default.hpp"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -32,8 +32,10 @@ namespace rocsparse
                                                                   rocsparse_direction source_dir_,
                                                                   int64_t             source_m_,
                                                                   int64_t             source_n_,
-                                                                  size_t* __restrict__ buffer_size_)
+                                                                  size_t*             buffer_size_)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         if((source_m_ > std::numeric_limits<I>::max()))
         {
             RETURN_WITH_MESSAGE_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error,
@@ -61,13 +63,9 @@ namespace rocsparse
         }
         }
 
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(nullptr,
-                                                    buffer_size_[0],
-                                                    (I* __restrict__)nullptr,
-                                                    (I* __restrict__)nullptr,
-                                                    num_seq + 1,
-                                                    rocprim::plus<I>(),
-                                                    handle_->stream));
+        RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::inclusive_scan_buffer_size<I, I>(
+            handle_, num_seq + 1, buffer_size_)));
+
         return rocsparse_status_success;
     }
 
@@ -75,6 +73,8 @@ namespace rocsparse
     static rocsparse_status internal_extract_buffer_size_dispatch(rocsparse_indextype indextype_I,
                                                                   P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(indextype_I)
         {
         case rocsparse_indextype_u16:
@@ -93,7 +93,9 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
     template <typename I, typename J>
@@ -103,8 +105,10 @@ namespace rocsparse
                                                             size_t buffer_size_,
                                                             void* __restrict__ buffer_)
     {
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(
-            buffer_, buffer_size_, ptr_, ptr_, nseq_ + 1, rocprim::plus<I>(), handle_->stream));
+        ROCSPARSE_ROUTINE_TRACE;
+
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(
+            handle_, ptr_, ptr_, nseq_ + 1, buffer_size_, buffer_));
         return rocsparse_status_success;
     }
 
@@ -172,6 +176,7 @@ namespace rocsparse
                                            size_t               buffer_size_,
                                            void* __restrict__ buffer_)
     {
+        ROCSPARSE_ROUTINE_TRACE;
 
         if((source_m_ > std::numeric_limits<J>::max()))
         {
@@ -275,6 +280,7 @@ namespace rocsparse
                                            (const I* __restrict__)target_ptr_,
                                            target_base_,
                                            ((int64_t* __restrict__)target_nnz_));
+
         return rocsparse_status_success;
     }
 
@@ -282,6 +288,8 @@ namespace rocsparse
     static rocsparse_status internal_extract_analysis_dispatch_J(rocsparse_indextype indextype_J,
                                                                  P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(indextype_J)
         {
         case rocsparse_indextype_u16:
@@ -300,13 +308,17 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
     template <typename T, typename... P>
     static rocsparse_status internal_extract_analysis_dispatch_I(rocsparse_indextype indextype_I,
                                                                  P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(indextype_I)
         {
         case rocsparse_indextype_u16:
@@ -325,15 +337,24 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
     template <typename... P>
     static rocsparse_status internal_extract_analysis_dispatch(rocsparse_datatype datatype_T,
                                                                P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(datatype_T)
         {
+        case rocsparse_datatype_f16_r:
+        {
+            RETURN_IF_ROCSPARSE_ERROR((internal_extract_analysis_dispatch_I<_Float16>)(p...));
+            return rocsparse_status_success;
+        }
         case rocsparse_datatype_f32_r:
         {
             RETURN_IF_ROCSPARSE_ERROR((internal_extract_analysis_dispatch_I<float>)(p...));
@@ -377,7 +398,9 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
     template <uint32_t BLOCKSIZE, typename T, typename I, typename J>
@@ -436,6 +459,8 @@ namespace rocsparse
                                                        rocsparse_index_base target_base_,
                                                        void* __restrict__ buffer_)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         if((source_m_ > std::numeric_limits<J>::max()))
         {
             RETURN_WITH_MESSAGE_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error,
@@ -526,6 +551,8 @@ namespace rocsparse
     static rocsparse_status internal_extract_compute_dispatch_J(rocsparse_indextype indextype_J,
                                                                 P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(indextype_J)
         {
         case rocsparse_indextype_u16:
@@ -544,13 +571,17 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
     template <typename T, typename... P>
     static rocsparse_status internal_extract_compute_dispatch_I(rocsparse_indextype indextype_I,
                                                                 P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(indextype_I)
         {
         case rocsparse_indextype_u16:
@@ -569,15 +600,24 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
     template <typename... P>
     static rocsparse_status internal_extract_compute_dispatch(rocsparse_datatype datatype_T,
                                                               P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(datatype_T)
         {
+        case rocsparse_datatype_f16_r:
+        {
+            RETURN_IF_ROCSPARSE_ERROR((internal_extract_compute_dispatch_I<_Float16>)(p...));
+            return rocsparse_status_success;
+        }
         case rocsparse_datatype_f32_r:
         {
             RETURN_IF_ROCSPARSE_ERROR((internal_extract_compute_dispatch_I<float>)(p...));
@@ -621,7 +661,9 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 
 }
@@ -630,6 +672,8 @@ rocsparse_extract_descr_default_t::rocsparse_extract_descr_default_t(
     rocsparse_const_spmat_descr source, rocsparse_const_spmat_descr target)
     : _rocsparse_extract_descr(rocsparse_extract_alg_default, source, target)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     if(source->format != target->format)
     {
         THROW_WITH_MESSAGE_IF_ROCSPARSE_ERROR(
@@ -676,6 +720,7 @@ rocsparse_status
                                                    rocsparse_extract_stage     stage,
                                                    size_t* __restrict__ buffer_size_in_bytes)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     switch(stage)
     {
@@ -712,6 +757,8 @@ rocsparse_status rocsparse_extract_descr_default_t::run(rocsparse_handle
                                                         void* __restrict__ buffer)
 
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const rocsparse_fill_mode target_fill_mode = target->descr->fill_mode;
     const rocsparse_diag_type target_diag_type = target->descr->diag_type;
 
