@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,19 +23,20 @@
  * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_gebsr2gebsr.h"
-#include "control.h"
 #include "internal/conversion/rocsparse_coo2csr.h"
 #include "internal/conversion/rocsparse_csr2gebsr.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_gebsr2gebsr.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
-#include "common.h"
 #include "gebsr2csr_device.h"
 #include "gebsr2gebsr_device.h"
+#include "rocsparse_common.hpp"
 #include "rocsparse_csr2gebsr.hpp"
 #include "rocsparse_gebsr2csr.hpp"
 
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_common.h"
+#include "rocsparse_primitives.hpp"
 
 #define launch_gebsr2gebsr_fast_kernel(T, direction, block_size, segment_size)     \
     RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(                                            \
@@ -77,6 +78,7 @@ rocsparse_status rocsparse::gebsr2gebsr_buffer_size_template(rocsparse_handle
                                                              rocsparse_int col_block_dim_C, //12
                                                              size_t*       buffer_size) //13
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     rocsparse::log_trace(handle,
                          rocsparse::replaceX<T>("rocsparse_Xgebsr2csr_buffer_size"),
@@ -185,6 +187,8 @@ namespace rocsparse
                                                     rocsparse_int             col_block_dim_C,
                                                     void*                     temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         if(mb == 0 || nb == 0)
         {
             return rocsparse_status_success;
@@ -213,6 +217,7 @@ rocsparse_status rocsparse::gebsr2gebsr_template(rocsparse_handle          handl
                                                  rocsparse_int             col_block_dim_C, //16
                                                  void*                     temp_buffer) //17
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     rocsparse::log_trace(handle,
                          rocsparse::replaceX<T>("rocsparse_Xgebsr2gebsr"),
@@ -510,6 +515,8 @@ extern "C" rocsparse_status rocsparse_gebsr2csr_nnz(rocsparse_handle          ha
                                                     rocsparse_int*            csr_col_ind) //11
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for valid handle
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_ENUM(1, direction);
@@ -622,11 +629,13 @@ try
     }
 
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_gebsr2gebsr_nnz(rocsparse_handle          handle, //0
                                                       rocsparse_direction       dir, //1
@@ -646,6 +655,8 @@ extern "C" rocsparse_status rocsparse_gebsr2gebsr_nnz(rocsparse_handle
                                                       void*          temp_buffer) //15
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Logging
     rocsparse::log_trace(handle,
                          "rocsparse_gebsr2gebsr_nnz",
@@ -728,14 +739,8 @@ try
         {
             if(bsr_row_ptr_C != nullptr)
             {
-                RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                                   dim3(mb_c / 256 + 1),
-                                                   dim3(256),
-                                                   0,
-                                                   stream,
-                                                   (mb_c + 1),
-                                                   bsr_row_ptr_C,
-                                                   static_cast<rocsparse_int>(descr_C->base));
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                    handle, mb_c + 1, static_cast<rocsparse_int>(descr_C->base), bsr_row_ptr_C));
             }
 
             rocsparse_pointer_mode mode;
@@ -809,15 +814,10 @@ try
         }
 
         // Perform inclusive scan on bsr row pointer array
-        auto   op = rocprim::plus<rocsparse_int>();
         size_t temp_storage_size_bytes;
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(nullptr,
-                                                    temp_storage_size_bytes,
-                                                    bsr_row_ptr_C,
-                                                    bsr_row_ptr_C,
-                                                    mb_c + 1,
-                                                    op,
-                                                    handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::inclusive_scan_buffer_size<rocsparse_int, rocsparse_int>(
+                handle, mb_c + 1, &temp_storage_size_bytes)));
 
         bool  temp_alloc       = false;
         void* temp_storage_ptr = nullptr;
@@ -833,13 +833,12 @@ try
             temp_alloc = true;
         }
 
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(temp_storage_ptr,
-                                                    temp_storage_size_bytes,
-                                                    bsr_row_ptr_C,
-                                                    bsr_row_ptr_C,
-                                                    mb_c + 1,
-                                                    op,
-                                                    handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(handle,
+                                                                        bsr_row_ptr_C,
+                                                                        bsr_row_ptr_C,
+                                                                        mb_c + 1,
+                                                                        temp_storage_size_bytes,
+                                                                        temp_storage_ptr));
 
         if(temp_alloc)
         {
@@ -913,11 +912,13 @@ try
     }
 
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_sgebsr2gebsr_buffer_size(rocsparse_handle          handle,
                                                                rocsparse_direction       dir,
@@ -935,6 +936,8 @@ extern "C" rocsparse_status rocsparse_sgebsr2gebsr_buffer_size(rocsparse_handle
                                                                size_t*              buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_buffer_size_template(handle,
                                                                           dir,
                                                                           mb,
@@ -950,11 +953,13 @@ try
                                                                           col_block_dim_C,
                                                                           buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_dgebsr2gebsr_buffer_size(rocsparse_handle          handle,
                                                                rocsparse_direction       dir,
@@ -972,6 +977,8 @@ extern "C" rocsparse_status rocsparse_dgebsr2gebsr_buffer_size(rocsparse_handle
                                                                size_t*              buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_buffer_size_template(handle,
                                                                           dir,
                                                                           mb,
@@ -987,11 +994,13 @@ try
                                                                           col_block_dim_C,
                                                                           buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_cgebsr2gebsr_buffer_size(rocsparse_handle               handle,
@@ -1010,6 +1019,8 @@ extern "C" rocsparse_status
                                        size_t*                        buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_buffer_size_template(handle,
                                                                           dir,
                                                                           mb,
@@ -1025,11 +1036,13 @@ try
                                                                           col_block_dim_C,
                                                                           buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_zgebsr2gebsr_buffer_size(rocsparse_handle                handle,
@@ -1048,6 +1061,8 @@ extern "C" rocsparse_status
                                        size_t*                         buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_buffer_size_template(handle,
                                                                           dir,
                                                                           mb,
@@ -1063,11 +1078,13 @@ try
                                                                           col_block_dim_C,
                                                                           buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_sgebsr2gebsr(rocsparse_handle          handle,
                                                    rocsparse_direction       dir,
@@ -1089,6 +1106,8 @@ extern "C" rocsparse_status rocsparse_sgebsr2gebsr(rocsparse_handle          han
                                                    void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_template(handle,
                                                               dir,
                                                               mb,
@@ -1108,11 +1127,13 @@ try
                                                               col_block_dim_C,
                                                               temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_dgebsr2gebsr(rocsparse_handle          handle,
                                                    rocsparse_direction       dir,
@@ -1134,6 +1155,8 @@ extern "C" rocsparse_status rocsparse_dgebsr2gebsr(rocsparse_handle          han
                                                    void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_template(handle,
                                                               dir,
                                                               mb,
@@ -1153,11 +1176,13 @@ try
                                                               col_block_dim_C,
                                                               temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_cgebsr2gebsr(rocsparse_handle               handle,
                                                    rocsparse_direction            dir,
@@ -1179,6 +1204,8 @@ extern "C" rocsparse_status rocsparse_cgebsr2gebsr(rocsparse_handle
                                                    void*                          temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_template(handle,
                                                               dir,
                                                               mb,
@@ -1198,11 +1225,13 @@ try
                                                               col_block_dim_C,
                                                               temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_zgebsr2gebsr(rocsparse_handle                handle,
                                                    rocsparse_direction             dir,
@@ -1224,6 +1253,8 @@ extern "C" rocsparse_status rocsparse_zgebsr2gebsr(rocsparse_handle
                                                    void*                           temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsr_template(handle,
                                                               dir,
                                                               mb,
@@ -1243,8 +1274,10 @@ try
                                                               col_block_dim_C,
                                                               temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
