@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,13 +23,14 @@
  * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_hyb2csr.h"
-#include "control.h"
 #include "internal/conversion/rocsparse_coo2csr.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_hyb2csr.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "hyb2csr_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_common.h"
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -42,19 +43,15 @@ namespace rocsparse
                                                 void*                     temp_buffer)
 
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Quick return if possible
         if(hyb->m == 0 || hyb->n == 0 || (hyb->ell_nnz == 0 && hyb->coo_nnz == 0))
         {
             if(hyb->m >= 0 && csr_row_ptr != nullptr)
             {
-                RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                                   dim3(hyb->m / 256 + 1),
-                                                   dim3(256),
-                                                   0,
-                                                   handle->stream,
-                                                   hyb->m + 1,
-                                                   csr_row_ptr,
-                                                   static_cast<rocsparse_int>(descr->base));
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                    handle, hyb->m + 1, static_cast<rocsparse_int>(descr->base), csr_row_ptr));
             }
 
             return rocsparse_status_success;
@@ -72,6 +69,8 @@ rocsparse_status rocsparse::hyb2csr_template(rocsparse_handle          handle,
                                              rocsparse_int*            csr_col_ind,
                                              void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Logging
     rocsparse::log_trace(handle,
                          rocsparse::replaceX<T>("rocsparse_Xhyb2csr"),
@@ -151,22 +150,17 @@ rocsparse_status rocsparse::hyb2csr_template(rocsparse_handle          handle,
     size_t rocprim_size;
     void*  rocprim_buffer = reinterpret_cast<void*>(ptr);
 
-    RETURN_IF_HIP_ERROR(rocprim::exclusive_scan(nullptr,
-                                                rocprim_size,
-                                                csr_row_ptr,
-                                                csr_row_ptr,
-                                                static_cast<rocsparse_int>(descr->base),
-                                                hyb->m + 1,
-                                                rocprim::plus<rocsparse_int>(),
-                                                stream));
-    RETURN_IF_HIP_ERROR(rocprim::exclusive_scan(rocprim_buffer,
-                                                rocprim_size,
-                                                csr_row_ptr,
-                                                csr_row_ptr,
-                                                static_cast<rocsparse_int>(descr->base),
-                                                hyb->m + 1,
-                                                rocprim::plus<rocsparse_int>(),
-                                                stream));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::exclusive_scan_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, static_cast<rocsparse_int>(descr->base), hyb->m + 1, &rocprim_size)));
+    RETURN_IF_ROCSPARSE_ERROR(
+        rocsparse::primitives::exclusive_scan(handle,
+                                              csr_row_ptr,
+                                              csr_row_ptr,
+                                              static_cast<rocsparse_int>(descr->base),
+                                              hyb->m + 1,
+                                              rocprim_size,
+                                              rocprim_buffer));
 
     // Fill columns and values
     RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::hyb2csr_fill_kernel<HYB2CSR_DIM>),
@@ -206,6 +200,8 @@ static rocsparse_status rocsparse_hyb2csr_buffer_size_quickreturn(rocsparse_hand
                                                                   size_t*     buffer_size)
 
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     if(hyb->m == 0 || hyb->n == 0 || (hyb->ell_nnz == 0 && hyb->coo_nnz == 0))
     {
         *buffer_size = 0;
@@ -221,6 +217,7 @@ extern "C" rocsparse_status rocsparse_hyb2csr_buffer_size(rocsparse_handle
                                                           size_t*                   buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -254,9 +251,6 @@ try
 
     ROCSPARSE_CHECKARG_POINTER(3, csr_row_ptr);
 
-    // Stream
-    hipStream_t stream = handle->stream;
-
     // Initialize buffer size
     *buffer_size = 0;
 
@@ -267,27 +261,22 @@ try
     }
 
     // Exclusive scan
-    size_t         rocprim_size;
-    rocsparse_int* ptr = reinterpret_cast<rocsparse_int*>(buffer_size);
-
-    RETURN_IF_HIP_ERROR(rocprim::exclusive_scan(nullptr,
-                                                rocprim_size,
-                                                ptr,
-                                                ptr,
-                                                static_cast<rocsparse_int>(descr->base),
-                                                hyb->m + 1,
-                                                rocprim::plus<rocsparse_int>(),
-                                                stream));
+    size_t rocprim_size;
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::exclusive_scan_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, static_cast<rocsparse_int>(descr->base), hyb->m + 1, &rocprim_size)));
 
     // rocprim buffer
     *buffer_size += rocprim_size;
 
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_shyb2csr(rocsparse_handle          handle,
                                                const rocsparse_mat_descr descr,
@@ -298,14 +287,18 @@ extern "C" rocsparse_status rocsparse_shyb2csr(rocsparse_handle          handle,
                                                void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::hyb2csr_template(
         handle, descr, hyb, csr_val, csr_row_ptr, csr_col_ind, temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_dhyb2csr(rocsparse_handle          handle,
                                                const rocsparse_mat_descr descr,
@@ -316,14 +309,18 @@ extern "C" rocsparse_status rocsparse_dhyb2csr(rocsparse_handle          handle,
                                                void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::hyb2csr_template(
         handle, descr, hyb, csr_val, csr_row_ptr, csr_col_ind, temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_chyb2csr(rocsparse_handle          handle,
                                                const rocsparse_mat_descr descr,
@@ -334,14 +331,18 @@ extern "C" rocsparse_status rocsparse_chyb2csr(rocsparse_handle          handle,
                                                void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::hyb2csr_template(
         handle, descr, hyb, csr_val, csr_row_ptr, csr_col_ind, temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_zhyb2csr(rocsparse_handle          handle,
                                                const rocsparse_mat_descr descr,
@@ -352,11 +353,15 @@ extern "C" rocsparse_status rocsparse_zhyb2csr(rocsparse_handle          handle,
                                                void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::hyb2csr_template(
         handle, descr, hyb, csr_val, csr_row_ptr, csr_col_ind, temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
