@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,13 +23,13 @@
  * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_nnz.h"
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_nnz.hpp"
 #include "rocsparse_nnz_impl.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "nnz_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -42,6 +42,8 @@ namespace rocsparse
                                            rocsparse_order  order,
                                            I*               nnz_per_rows)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         hipStream_t stream = handle->stream;
 
         static constexpr rocsparse_int NNZ_DIM_X = 64;
@@ -75,6 +77,8 @@ namespace rocsparse
                                            rocsparse_order  order,
                                            I*               nnz_per_columns)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         hipStream_t stream = handle->stream;
 
         static constexpr rocsparse_int NB = 256;
@@ -106,6 +110,8 @@ rocsparse_status rocsparse::nnz_template(rocsparse_handle    handle,
                                          int64_t             ld,
                                          I*                  nnz_per_row_columns)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     if(m == 0 || n == 0)
     {
         return rocsparse_status_success;
@@ -130,7 +136,9 @@ rocsparse_status rocsparse::nnz_template(rocsparse_handle    handle,
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 namespace rocsparse
@@ -147,6 +155,8 @@ namespace rocsparse
                                   I*                        nnz_total_dev_host_ptr,
                                   rocsparse_order           order)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_ENUM(1, dir);
         ROCSPARSE_CHECKARG_SIZE(2, m);
@@ -223,6 +233,7 @@ rocsparse_status rocsparse::nnz_impl(rocsparse_handle          handle,
                                      I*                        nnz_per_row_columns,
                                      I*                        nnz_total_dev_host_ptr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     //
     // Loggings
@@ -258,16 +269,9 @@ rocsparse_status rocsparse::nnz_impl(rocsparse_handle          handle,
     //
     {
         J      mn = dir == rocsparse_direction_row ? m : n;
-        auto   op = rocprim::plus<I>();
         size_t temp_storage_size_bytes;
-        RETURN_IF_HIP_ERROR(rocprim::reduce(nullptr,
-                                            temp_storage_size_bytes,
-                                            nnz_per_row_columns,
-                                            nnz_total_dev_host_ptr,
-                                            0,
-                                            mn,
-                                            op,
-                                            handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::find_sum_buffer_size<I, I>(
+            handle, mn, &temp_storage_size_bytes)));
         temp_storage_size_bytes += sizeof(I);
         bool  temp_alloc       = false;
         void* temp_storage_ptr = nullptr;
@@ -293,14 +297,8 @@ rocsparse_status rocsparse::nnz_impl(rocsparse_handle          handle,
         //
         // Perform reduce
         //
-        RETURN_IF_HIP_ERROR(rocprim::reduce(temp_storage_ptr,
-                                            temp_storage_size_bytes,
-                                            nnz_per_row_columns,
-                                            d_nnz,
-                                            0,
-                                            mn,
-                                            op,
-                                            handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::find_sum(
+            handle, nnz_per_row_columns, d_nnz, mn, temp_storage_size_bytes, temp_storage_ptr));
 
         //
         // Extract nnz
@@ -342,14 +340,17 @@ rocsparse_status rocsparse::nnz_impl(rocsparse_handle          handle,
         ITYPE*                    nnz_per_row_columns,                  \
         ITYPE*                    nnz_total_dev_host_ptr);
 
+INSTANTIATE(int32_t, int32_t, _Float16);
 INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
 INSTANTIATE(int32_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int32_t, int32_t, rocsparse_double_complex);
+INSTANTIATE(int64_t, int32_t, _Float16);
 INSTANTIATE(int64_t, int32_t, float);
 INSTANTIATE(int64_t, int32_t, double);
 INSTANTIATE(int64_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int32_t, rocsparse_double_complex);
+INSTANTIATE(int64_t, int64_t, _Float16);
 INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
@@ -374,6 +375,7 @@ extern "C" {
     {                                                                               \
         try                                                                         \
         {                                                                           \
+            ROCSPARSE_ROUTINE_TRACE;                                                \
             RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_impl(handle,                   \
                                                           dir,                      \
                                                           rocsparse_order_column,   \
