@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,12 +23,12 @@
  * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_nnz_compress.h"
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_nnz_compress.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "nnz_compress_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -60,6 +60,7 @@ rocsparse_status rocsparse::nnz_compress_template(rocsparse_handle          hand
                                                   rocsparse_int*            nnz_C, //6
                                                   T                         tol) //7
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -355,10 +356,10 @@ rocsparse_status rocsparse::nnz_compress_template(rocsparse_handle          hand
     }
 
     // Perform inclusive scan on csr row pointer array
-    auto   op = rocprim::plus<rocsparse_int>();
     size_t temp_storage_size_bytes;
-    RETURN_IF_HIP_ERROR(
-        rocprim::reduce(nullptr, temp_storage_size_bytes, nnz_per_row, dnnz_C, m, op, stream));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::find_sum_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, m, &temp_storage_size_bytes)));
 
     bool  temp_alloc       = false;
     void* temp_storage_ptr = nullptr;
@@ -374,8 +375,8 @@ rocsparse_status rocsparse::nnz_compress_template(rocsparse_handle          hand
         temp_alloc = true;
     }
 
-    RETURN_IF_HIP_ERROR(rocprim::reduce(
-        temp_storage_ptr, temp_storage_size_bytes, nnz_per_row, dnnz_C, m, op, stream));
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::find_sum(
+        handle, nnz_per_row, dnnz_C, m, temp_storage_size_bytes, temp_storage_ptr));
 
     if(handle->pointer_mode == rocsparse_pointer_mode_host)
     {
@@ -409,14 +410,18 @@ extern "C" rocsparse_status rocsparse_snnz_compress(rocsparse_handle          ha
                                                     float                     tol)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
         handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_dnnz_compress(rocsparse_handle          handle,
                                                     rocsparse_int             m,
@@ -428,14 +433,18 @@ extern "C" rocsparse_status rocsparse_dnnz_compress(rocsparse_handle          ha
                                                     double                    tol)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
         handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_cnnz_compress(rocsparse_handle               handle,
                                                     rocsparse_int                  m,
@@ -447,14 +456,18 @@ extern "C" rocsparse_status rocsparse_cnnz_compress(rocsparse_handle
                                                     rocsparse_float_complex        tol)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
         handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_znnz_compress(rocsparse_handle                handle,
                                                     rocsparse_int                   m,
@@ -466,11 +479,15 @@ extern "C" rocsparse_status rocsparse_znnz_compress(rocsparse_handle
                                                     rocsparse_double_complex        tol)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::nnz_compress_template(
         handle, m, descr_A, csr_val_A, csr_row_ptr_A, nnz_per_row, nnz_C, tol));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
