@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -23,28 +23,28 @@
 * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_prune_dense2csr_by_percentage.h"
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_prune_dense2csr_by_percentage.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "csr2csr_compress_device.h"
 #include "prune_dense2csr_by_percentage_device.h"
 #include "prune_dense2csr_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_common.h"
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
-    template <rocsparse_int DIM_X, rocsparse_int DIM_Y, typename T, typename U>
+    template <rocsparse_int DIM_X, rocsparse_int DIM_Y, typename T>
     ROCSPARSE_KERNEL(DIM_X* DIM_Y)
     void prune_dense2csr_nnz_kernel2(rocsparse_int m,
                                      rocsparse_int n,
                                      const T* __restrict__ A,
-                                     int64_t lda,
-                                     U       threshold_device_host,
+                                     int64_t  lda,
+                                     const T* threshold,
                                      rocsparse_int* __restrict__ nnz_per_rows)
     {
-        auto threshold = rocsparse::load_scalar_device_host(threshold_device_host);
-        rocsparse::prune_dense2csr_nnz_device<DIM_X, DIM_Y>(m, n, A, lda, threshold, nnz_per_rows);
+        rocsparse::prune_dense2csr_nnz_device<DIM_X, DIM_Y>(m, n, A, lda, *threshold, nnz_per_rows);
     }
 
     template <rocsparse_int NUMROWS_PER_BLOCK, rocsparse_int WF_SIZE, typename T>
@@ -79,6 +79,7 @@ rocsparse_status rocsparse::prune_dense2csr_by_percentage_buffer_size_template(
     rocsparse_mat_info        info, //10
     size_t*                   buffer_size) //11
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(
@@ -138,6 +139,7 @@ rocsparse_status
                                                           rocsparse_mat_info info, //9
                                                           void*              temp_buffer) //10
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -180,14 +182,8 @@ rocsparse_status
     {
         if(nnz_total_dev_host_ptr != nullptr && csr_row_ptr != nullptr)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                               dim3(m / 256 + 1),
-                                               dim3(256),
-                                               0,
-                                               stream,
-                                               (m + 1),
-                                               csr_row_ptr,
-                                               static_cast<rocsparse_int>(descr->base));
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                handle, m + 1, static_cast<rocsparse_int>(descr->base), csr_row_ptr));
 
             if(handle->pointer_mode == rocsparse_pointer_mode_device)
             {
@@ -227,16 +223,14 @@ rocsparse_status
     size_t temp_storage_size_bytes_sort = 0;
     size_t temp_storage_size_bytes_scan = 0;
 
-    RETURN_IF_HIP_ERROR(rocprim::radix_sort_keys(
-        nullptr, temp_storage_size_bytes_sort, output, (output + nnz_A), nnz_A));
+    uint32_t startbit = 0;
+    uint32_t endbit   = 8 * sizeof(T);
+    RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::radix_sort_keys_buffer_size<T>(
+        handle, nnz_A, startbit, endbit, &temp_storage_size_bytes_sort)));
 
-    RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(nullptr,
-                                                temp_storage_size_bytes_scan,
-                                                csr_row_ptr,
-                                                csr_row_ptr,
-                                                m + 1,
-                                                rocprim::plus<rocsparse_int>(),
-                                                handle->stream));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::inclusive_scan_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, m + 1, &temp_storage_size_bytes_scan)));
 
     size_t temp_storage_size_bytes
         = rocsparse::max(temp_storage_size_bytes_sort, temp_storage_size_bytes_scan);
@@ -257,10 +251,11 @@ rocsparse_status
     }
 
     // perform sort on first half of output array and store result in second half of output array
-    rocprim::radix_sort_keys(
-        temp_storage_ptr, temp_storage_size_bytes_sort, output, (output + nnz_A), nnz_A);
+    rocsparse::primitives::double_buffer<T> keys(output, output + nnz_A);
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::radix_sort_keys(
+        handle, keys, nnz_A, startbit, endbit, temp_storage_size_bytes_sort, temp_storage_ptr));
 
-    const T* d_threshold = &output[nnz_A + pos];
+    const T* d_threshold = keys.current() + pos;
 
     static constexpr int NNZ_DIM_X = 64;
     static constexpr int NNZ_DIM_Y = 16;
@@ -269,42 +264,20 @@ rocsparse_status
         dim3 grid((m - 1) / (NNZ_DIM_X * 4) + 1);
         dim3 threads(NNZ_DIM_X, NNZ_DIM_Y);
 
-        if(handle->pointer_mode == rocsparse_pointer_mode_device)
-        {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
-                (rocsparse::prune_dense2csr_nnz_kernel2<NNZ_DIM_X, NNZ_DIM_Y>),
-                grid,
-                threads,
-                0,
-                stream,
-                m,
-                n,
-                A,
-                lda,
-                d_threshold,
-                &csr_row_ptr[1]);
-        }
-        else
-        {
-            T h_threshold = static_cast<T>(0);
-            RETURN_IF_HIP_ERROR(hipMemcpyAsync(
-                &h_threshold, d_threshold, sizeof(T), hipMemcpyDeviceToHost, handle->stream));
-            RETURN_IF_HIP_ERROR(hipStreamSynchronize(handle->stream));
-
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
-                (rocsparse::prune_dense2csr_nnz_kernel2<NNZ_DIM_X, NNZ_DIM_Y>),
-                grid,
-                threads,
-                0,
-                stream,
-                m,
-                n,
-                A,
-                lda,
-                h_threshold,
-                &csr_row_ptr[1]);
-        }
+        RETURN_IF_HIPLAUNCHKERNELGGL_ERROR(
+            (rocsparse::prune_dense2csr_nnz_kernel2<NNZ_DIM_X, NNZ_DIM_Y>),
+            grid,
+            threads,
+            0,
+            stream,
+            m,
+            n,
+            A,
+            lda,
+            d_threshold,
+            &csr_row_ptr[1]);
     }
+
     // Store threshold at first entry in output array
     RETURN_IF_HIP_ERROR(
         hipMemcpyAsync(output, d_threshold, sizeof(T), hipMemcpyDeviceToDevice, handle->stream));
@@ -316,13 +289,9 @@ rocsparse_status
     RETURN_IF_HIP_ERROR(hipStreamSynchronize(handle->stream));
 
     // Perform actual inclusive sum
-    RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(temp_storage_ptr,
-                                                temp_storage_size_bytes_scan,
-                                                csr_row_ptr,
-                                                csr_row_ptr,
-                                                m + 1,
-                                                rocprim::plus<rocsparse_int>(),
-                                                handle->stream));
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(
+        handle, csr_row_ptr, csr_row_ptr, m + 1, temp_storage_size_bytes_scan, temp_storage_ptr));
+
     // Free rocprim buffer, if allocated
     if(temp_alloc == true)
     {
@@ -371,6 +340,7 @@ rocsparse_status
                                                       rocsparse_mat_info        info, //10
                                                       void*                     temp_buffer) //11
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     rocsparse::log_trace(handle,
                          rocsparse::replaceX<T>("rocsparse_Xprune_dense2csr_by_percentage"),
@@ -512,6 +482,8 @@ extern "C" rocsparse_status
                                                          size_t*                   buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(
         rocsparse::prune_dense2csr_by_percentage_buffer_size_template(handle,
                                                                       m,
@@ -526,11 +498,13 @@ try
                                                                       info,
                                                                       buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_dprune_dense2csr_by_percentage_buffer_size(rocsparse_handle          handle,
@@ -547,6 +521,8 @@ extern "C" rocsparse_status
                                                          size_t*                   buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(
         rocsparse::prune_dense2csr_by_percentage_buffer_size_template(handle,
                                                                       m,
@@ -561,11 +537,13 @@ try
                                                                       info,
                                                                       buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_sprune_dense2csr_nnz_by_percentage(rocsparse_handle          handle,
@@ -581,6 +559,8 @@ extern "C" rocsparse_status
                                                  void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(
         rocsparse::prune_dense2csr_nnz_by_percentage_template(handle,
                                                               m,
@@ -594,11 +574,13 @@ try
                                                               info,
                                                               temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_dprune_dense2csr_nnz_by_percentage(rocsparse_handle          handle,
@@ -614,6 +596,8 @@ extern "C" rocsparse_status
                                                  void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(
         rocsparse::prune_dense2csr_nnz_by_percentage_template(handle,
                                                               m,
@@ -627,11 +611,13 @@ try
                                                               info,
                                                               temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_sprune_dense2csr_by_percentage(rocsparse_handle          handle,
@@ -648,6 +634,8 @@ extern "C" rocsparse_status
                                              void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::prune_dense2csr_by_percentage_template(handle,
                                                                                 m,
                                                                                 n,
@@ -661,11 +649,13 @@ try
                                                                                 info,
                                                                                 temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status
     rocsparse_dprune_dense2csr_by_percentage(rocsparse_handle          handle,
@@ -682,6 +672,8 @@ extern "C" rocsparse_status
                                              void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::prune_dense2csr_by_percentage_template(handle,
                                                                                 m,
                                                                                 n,
@@ -695,8 +687,10 @@ try
                                                                                 info,
                                                                                 temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
