@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,10 +23,11 @@
  * ************************************************************************ */
 
 #include "../conversion/rocsparse_identity.hpp"
-#include "control.h"
 #include "csrgemm_device.h"
 #include "internal/extra/rocsparse_bsrgemm.h"
-#include "utility.h"
+#include "rocsparse_common.h"
+#include "rocsparse_control.hpp"
+#include "rocsparse_utility.hpp"
 
 #include "rocsparse_bsrgemm.hpp"
 #include "rocsparse_csrgemm.hpp"
@@ -63,6 +64,7 @@ namespace rocsparse
                                            const rocsparse_mat_info  info_C, //23
                                            void*                     temp_buffer) //24
     {
+        ROCSPARSE_ROUTINE_TRACE;
 
         ROCSPARSE_CHECKARG_POINTER(23, info_C);
         ROCSPARSE_CHECKARG(
@@ -319,14 +321,8 @@ namespace rocsparse
 
             if(bsr_row_ptr_C != nullptr)
             {
-                RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<1024>),
-                                                   dim3(mb / 1024 + 1),
-                                                   dim3(1024),
-                                                   0,
-                                                   handle->stream,
-                                                   (mb + 1),
-                                                   bsr_row_ptr_C,
-                                                   static_cast<I>(descr_C->base));
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                    handle, mb + 1, static_cast<I>(descr_C->base), bsr_row_ptr_C));
             }
 
             return rocsparse_status_success;
@@ -361,6 +357,8 @@ rocsparse_status rocsparse::bsrgemm_nnzb_template(rocsparse_handle          hand
                                                   const rocsparse_mat_info  info_C,
                                                   void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_nnz_template(handle,
                                                               trans_A,
                                                               trans_B,
@@ -416,6 +414,8 @@ namespace rocsparse
                                               const rocsparse_mat_info  info_C,
                                               void*                     temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_nnz_core(handle,
                                                               trans_A,
                                                               trans_B,
@@ -445,6 +445,8 @@ namespace rocsparse
     template <typename... P>
     static inline rocsparse_status bsrgemm_nnzb_impl(P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         rocsparse::log_trace("rocsparse_bsrgemm_nnzb", p...);
 
         const rocsparse_status status = rocsparse::bsrgemm_nnzb_checkarg(p...);
@@ -527,6 +529,8 @@ extern "C" rocsparse_status rocsparse_bsrgemm_nnzb(rocsparse_handle          han
                                                    void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::bsrgemm_nnzb_impl(handle,
                                                            dir,
                                                            trans_A,
@@ -553,8 +557,10 @@ try
                                                            info_C,
                                                            temp_buffer));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
