@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,11 @@
  * ************************************************************************ */
 
 #include "../conversion/rocsparse_identity.hpp"
-#include "control.h"
 #include "csrgemm_device.h"
 #include "internal/extra/rocsparse_csrgemm.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csrgemm.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "rocsparse_csrgemm_mult.hpp"
 #include "rocsparse_csrgemm_multadd.hpp"
@@ -43,6 +43,8 @@ namespace rocsparse
                                                 const int64_t      nnz_B,
                                                 const int64_t      nnz_D)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         info->csrgemm_info->mul = (alpha != nullptr);
         info->csrgemm_info->add = (beta != nullptr);
 
@@ -82,6 +84,7 @@ namespace rocsparse
                                                          rocsparse_mat_info info_C, //20
                                                          size_t*            buffer_size) //21
     {
+        ROCSPARSE_ROUTINE_TRACE;
 
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_POINTER(20, info_C);
@@ -310,6 +313,7 @@ rocsparse_status rocsparse::csrgemm_buffer_size_template(rocsparse_handle
                                                          rocsparse_mat_info        info_C,
                                                          size_t*                   buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::destroy_csrgemm_info(info_C->csrgemm_info));
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::create_csrgemm_info(&info_C->csrgemm_info));
@@ -414,6 +418,8 @@ namespace rocsparse
                                                      rocsparse_mat_info        info_C,
                                                      size_t*                   buffer_size)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Logging
         rocsparse::log_trace(handle,
                              rocsparse::replaceX<T>("rocsparse_Xcsrgemm_buffer_size"),
@@ -633,6 +639,7 @@ INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
                                      size_t*                   buffer_size)          \
     try                                                                              \
     {                                                                                \
+        ROCSPARSE_ROUTINE_TRACE;                                                     \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_buffer_size_impl(handle,        \
                                                                       trans_A,       \
                                                                       trans_B,       \
