@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,77 +23,39 @@
  * ************************************************************************ */
 
 #include "internal/generic/rocsparse_axpby.h"
-#include "axpby_device.h"
-#include "control.h"
 #include "rocsparse_axpyi.hpp"
-#include "utility.h"
+#include "rocsparse_common.h"
+#include "rocsparse_control.hpp"
+#include "rocsparse_utility.hpp"
 
 namespace rocsparse
 {
-    template <uint32_t BLOCKSIZE, typename I, typename T, typename U>
-    ROCSPARSE_KERNEL(BLOCKSIZE)
-    void axpby_scale_kernel(I size, U alpha_device_host, T* __restrict__ x)
-    {
-        auto alpha = rocsparse::load_scalar_device_host(alpha_device_host);
-        if(alpha != static_cast<T>(1))
-        {
-            rocsparse::axpby_scale_device<BLOCKSIZE>(size, alpha, x);
-        }
-    }
-
-    template <typename I, typename T>
+    template <typename T, typename I, typename X, typename Y>
     rocsparse_status axpby_template(rocsparse_handle            handle,
                                     const void*                 alpha,
                                     rocsparse_const_spvec_descr x,
                                     const void*                 beta,
                                     rocsparse_dnvec_descr       y)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Quick return
         if(y->size == 0)
         {
             return rocsparse_status_success;
         }
 
-#define SCALE_DIM 256
-        dim3 scale_blocks((y->size - 1) / SCALE_DIM + 1);
-        dim3 scale_threads(SCALE_DIM);
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::scale_array(handle, (I)y->size, (const T*)beta, (Y*)y->values));
 
-        if(handle->pointer_mode == rocsparse_pointer_mode_device)
-        {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::axpby_scale_kernel<SCALE_DIM>),
-                                               scale_blocks,
-                                               scale_threads,
-                                               0,
-                                               handle->stream,
-                                               (I)y->size,
-                                               (const T*)beta,
-                                               (T*)y->values);
-        }
-        else
-        {
-            const T* beta_ptr = (const T*)beta;
-
-            if(*beta_ptr != static_cast<T>(1))
-            {
-                RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::axpby_scale_kernel<SCALE_DIM>),
-                                                   scale_blocks,
-                                                   scale_threads,
-                                                   0,
-                                                   handle->stream,
-                                                   (I)y->size,
-                                                   *beta_ptr,
-                                                   (T*)y->values);
-            }
-        }
-#undef SCALE_DIM
-
-        RETURN_IF_ROCSPARSE_ERROR((rocsparse::axpyi_template<I, T>)(handle,
-                                                                    (I)x->nnz,
-                                                                    (const T*)alpha,
-                                                                    (const T*)x->const_val_data,
-                                                                    (const I*)x->const_idx_data,
-                                                                    (T*)y->values,
-                                                                    x->idx_base));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::axpyi_template<T, I, X, Y>)(handle,
+                                                    (I)x->nnz,
+                                                    (const T*)alpha,
+                                                    (const X*)x->const_val_data,
+                                                    (const I*)x->const_idx_data,
+                                                    (Y*)y->values,
+                                                    x->idx_base));
         return rocsparse_status_success;
     }
 }
@@ -111,6 +73,8 @@ extern "C" rocsparse_status rocsparse_axpby(rocsparse_handle            handle,
                                             rocsparse_dnvec_descr       y)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for invalid handle
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
@@ -135,29 +99,39 @@ try
     // Check for matching types while we do not support mixed precision computation
     ROCSPARSE_CHECKARG(4, y, (y->data_type != x->data_type), rocsparse_status_not_implemented);
 
+    // half real ; i32
+    if(x->idx_type == rocsparse_indextype_i32 && x->data_type == rocsparse_datatype_f16_r)
+    {
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::axpby_template<float, int32_t, _Float16, _Float16>)(handle,
+                                                                            alpha,
+                                                                            x,
+                                                                            beta,
+                                                                            y));
+        return rocsparse_status_success;
+    }
     // single real ; i32
     if(x->idx_type == rocsparse_indextype_i32 && x->data_type == rocsparse_datatype_f32_r)
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int32_t, float>)(handle, alpha, x, beta, y));
+            (rocsparse::axpby_template<float, int32_t, float, float>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
     // double real ; i32
     if(x->idx_type == rocsparse_indextype_i32 && x->data_type == rocsparse_datatype_f64_r)
     {
-        RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int32_t, double>)(handle, alpha, x, beta, y));
+        RETURN_IF_ROCSPARSE_ERROR((
+            rocsparse::axpby_template<double, int32_t, double, double>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
     // single complex ; i32
     if(x->idx_type == rocsparse_indextype_i32 && x->data_type == rocsparse_datatype_f32_c)
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int32_t, rocsparse_float_complex>)(handle,
-                                                                          alpha,
-                                                                          x,
-                                                                          beta,
-                                                                          y));
+            (rocsparse::axpby_template<rocsparse_float_complex,
+                                       int32_t,
+                                       rocsparse_float_complex,
+                                       rocsparse_float_complex>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
 
@@ -165,53 +139,63 @@ try
     if(x->idx_type == rocsparse_indextype_i32 && x->data_type == rocsparse_datatype_f64_c)
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int32_t, rocsparse_double_complex>)(handle,
-                                                                           alpha,
-                                                                           x,
-                                                                           beta,
-                                                                           y));
+            (rocsparse::axpby_template<rocsparse_double_complex,
+                                       int32_t,
+                                       rocsparse_double_complex,
+                                       rocsparse_double_complex>)(handle, alpha, x, beta, y));
+        return rocsparse_status_success;
+    }
+    // half real ; i64
+    if(x->idx_type == rocsparse_indextype_i64 && x->data_type == rocsparse_datatype_f16_r)
+    {
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::axpby_template<float, int64_t, _Float16, _Float16>)(handle,
+                                                                            alpha,
+                                                                            x,
+                                                                            beta,
+                                                                            y));
         return rocsparse_status_success;
     }
     // single real ; i64
     if(x->idx_type == rocsparse_indextype_i64 && x->data_type == rocsparse_datatype_f32_r)
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int64_t, float>)(handle, alpha, x, beta, y));
+            (rocsparse::axpby_template<float, int64_t, float, float>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
     // double real ; i64
     if(x->idx_type == rocsparse_indextype_i64 && x->data_type == rocsparse_datatype_f64_r)
     {
-        RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int64_t, double>)(handle, alpha, x, beta, y));
+        RETURN_IF_ROCSPARSE_ERROR((
+            rocsparse::axpby_template<double, int64_t, double, double>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
     // single complex ; i64
     if(x->idx_type == rocsparse_indextype_i64 && x->data_type == rocsparse_datatype_f32_c)
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int64_t, rocsparse_float_complex>)(handle,
-                                                                          alpha,
-                                                                          x,
-                                                                          beta,
-                                                                          y));
+            (rocsparse::axpby_template<rocsparse_float_complex,
+                                       int64_t,
+                                       rocsparse_float_complex,
+                                       rocsparse_float_complex>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
     // double complex ; i64
     if(x->idx_type == rocsparse_indextype_i64 && x->data_type == rocsparse_datatype_f64_c)
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            (rocsparse::axpby_template<int64_t, rocsparse_double_complex>)(handle,
-                                                                           alpha,
-                                                                           x,
-                                                                           beta,
-                                                                           y));
+            (rocsparse::axpby_template<rocsparse_double_complex,
+                                       int64_t,
+                                       rocsparse_double_complex,
+                                       rocsparse_double_complex>)(handle, alpha, x, beta, y));
         return rocsparse_status_success;
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
