@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -129,12 +129,11 @@ namespace rocsparse
               typename J,
               typename A,
               typename X,
-              typename Y,
-              typename U>
+              typename Y>
     ROCSPARSE_KERNEL(BLOCKSIZE)
     void bsrxmvn_general_kernel(rocsparse_direction dir,
-                                U                   alpha_device_host,
-                                J                   size_of_mask,
+                                ROCSPARSE_DEVICE_HOST_SCALAR_PARAMS(T, alpha),
+                                J size_of_mask,
                                 const J* __restrict__ bsr_mask_ptr,
                                 const I* __restrict__ bsr_row_ptr,
                                 const I* __restrict__ bsr_end_ptr,
@@ -142,12 +141,14 @@ namespace rocsparse
                                 const A* __restrict__ bsr_val,
                                 J block_dim,
                                 const X* __restrict__ x,
-                                U beta_device_host,
+                                ROCSPARSE_DEVICE_HOST_SCALAR_PARAMS(T, beta),
                                 Y* __restrict__ y,
-                                rocsparse_index_base idx_base)
+                                rocsparse_index_base idx_base,
+                                bool                 is_host_mode)
     {
-        auto alpha = rocsparse::load_scalar_device_host(alpha_device_host);
-        auto beta  = rocsparse::load_scalar_device_host(beta_device_host);
+        ROCSPARSE_DEVICE_HOST_SCALAR_GET(alpha);
+        ROCSPARSE_DEVICE_HOST_SCALAR_GET(beta);
+
         if(alpha != static_cast<T>(0) || beta != static_cast<T>(1))
         {
             rocsparse::bsrxmvn_general_device<BLOCKSIZE, WFSIZE>(dir,
@@ -167,11 +168,11 @@ namespace rocsparse
     }
 }
 
-template <typename T, typename I, typename J, typename A, typename X, typename Y, typename U>
+template <typename T, typename I, typename J, typename A, typename X, typename Y>
 void rocsparse::bsrxmvn_general(rocsparse_handle     handle,
                                 rocsparse_direction  dir,
                                 J                    mb,
-                                U                    alpha_device_host,
+                                const T*             alpha_device_host,
                                 J                    size_of_mask,
                                 const J*             bsr_mask_ptr,
                                 const I*             bsr_row_ptr,
@@ -180,111 +181,104 @@ void rocsparse::bsrxmvn_general(rocsparse_handle     handle,
                                 const A*             bsr_val,
                                 J                    block_dim,
                                 const X*             x,
-                                U                    beta_device_host,
+                                const T*             beta_device_host,
                                 Y*                   y,
                                 rocsparse_index_base base)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const J size = (bsr_mask_ptr == nullptr) ? mb : size_of_mask;
     // Differentiate BSR block dimensions
     if(block_dim <= 8)
     {
-        THROW_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::bsrxmvn_general_kernel<64, 8, T>),
-                                          dim3(size),
-                                          dim3(8 * 8),
-                                          0,
-                                          handle->stream,
-                                          dir,
-                                          alpha_device_host,
-                                          size_of_mask,
-                                          bsr_mask_ptr,
-                                          bsr_row_ptr,
-                                          bsr_end_ptr,
-                                          bsr_col_ind,
-                                          bsr_val,
-                                          block_dim,
-                                          x,
-                                          beta_device_host,
-                                          y,
-                                          base);
+        THROW_IF_HIPLAUNCHKERNELGGL_ERROR(
+            (rocsparse::bsrxmvn_general_kernel<64, 8>),
+            dim3(size),
+            dim3(8 * 8),
+            0,
+            handle->stream,
+            dir,
+            ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, alpha_device_host),
+            size_of_mask,
+            bsr_mask_ptr,
+            bsr_row_ptr,
+            bsr_end_ptr,
+            bsr_col_ind,
+            bsr_val,
+            block_dim,
+            x,
+            ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, beta_device_host),
+            y,
+            base,
+            handle->pointer_mode == rocsparse_pointer_mode_host);
     }
     else if(block_dim <= 16)
     {
-        THROW_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::bsrxmvn_general_kernel<256, 16, T>),
-                                          dim3(size),
-                                          dim3(16 * 16),
-                                          0,
-                                          handle->stream,
-                                          dir,
-                                          alpha_device_host,
-                                          size_of_mask,
-                                          bsr_mask_ptr,
-                                          bsr_row_ptr,
-                                          bsr_end_ptr,
-                                          bsr_col_ind,
-                                          bsr_val,
-                                          block_dim,
-                                          x,
-                                          beta_device_host,
-                                          y,
-                                          base);
+        THROW_IF_HIPLAUNCHKERNELGGL_ERROR(
+            (rocsparse::bsrxmvn_general_kernel<256, 16>),
+            dim3(size),
+            dim3(16 * 16),
+            0,
+            handle->stream,
+            dir,
+            ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, alpha_device_host),
+            size_of_mask,
+            bsr_mask_ptr,
+            bsr_row_ptr,
+            bsr_end_ptr,
+            bsr_col_ind,
+            bsr_val,
+            block_dim,
+            x,
+            ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, beta_device_host),
+            y,
+            base,
+            handle->pointer_mode == rocsparse_pointer_mode_host);
     }
     else
     {
-        THROW_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::bsrxmvn_general_kernel<1024, 32, T>),
-                                          dim3(size),
-                                          dim3(32 * 32),
-                                          0,
-                                          handle->stream,
-                                          dir,
-                                          alpha_device_host,
-                                          size_of_mask,
-                                          bsr_mask_ptr,
-                                          bsr_row_ptr,
-                                          bsr_end_ptr,
-                                          bsr_col_ind,
-                                          bsr_val,
-                                          block_dim,
-                                          x,
-                                          beta_device_host,
-                                          y,
-                                          base);
+        THROW_IF_HIPLAUNCHKERNELGGL_ERROR(
+            (rocsparse::bsrxmvn_general_kernel<1024, 32>),
+            dim3(size),
+            dim3(32 * 32),
+            0,
+            handle->stream,
+            dir,
+            ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, alpha_device_host),
+            size_of_mask,
+            bsr_mask_ptr,
+            bsr_row_ptr,
+            bsr_end_ptr,
+            bsr_col_ind,
+            bsr_val,
+            block_dim,
+            x,
+            ROCSPARSE_DEVICE_HOST_SCALAR_ARGS(handle, beta_device_host),
+            y,
+            base,
+            handle->pointer_mode == rocsparse_pointer_mode_host);
     }
 }
 
 //
 // INSTANTIATE.
 //
-#define INSTANTIATE(T, I, J)                                                            \
-    template void rocsparse::bsrxmvn_general<T>(rocsparse_handle     handle,            \
-                                                rocsparse_direction  dir,               \
-                                                J                    mb,                \
-                                                const T*             alpha_device_host, \
-                                                J                    size_of_mask,      \
-                                                const J*             bsr_mask_ptr,      \
-                                                const I*             bsr_row_ptr,       \
-                                                const I*             bsr_end_ptr,       \
-                                                const J*             bsr_col_ind,       \
-                                                const T*             bsr_val,           \
-                                                J                    block_dim,         \
-                                                const T*             x,                 \
-                                                const T*             beta_device_host,  \
-                                                T*                   y,                 \
-                                                rocsparse_index_base base);             \
-    template void rocsparse::bsrxmvn_general<T>(rocsparse_handle     handle,            \
-                                                rocsparse_direction  dir,               \
-                                                J                    mb,                \
-                                                T                    alpha_device_host, \
-                                                J                    size_of_mask,      \
-                                                const J*             bsr_mask_ptr,      \
-                                                const I*             bsr_row_ptr,       \
-                                                const I*             bsr_end_ptr,       \
-                                                const J*             bsr_col_ind,       \
-                                                const T*             bsr_val,           \
-                                                J                    block_dim,         \
-                                                const T*             x,                 \
-                                                T                    beta_device_host,  \
-                                                T*                   y,                 \
-                                                rocsparse_index_base base)
+#define INSTANTIATE(T, I, J)                                                         \
+    template void rocsparse::bsrxmvn_general(rocsparse_handle     handle,            \
+                                             rocsparse_direction  dir,               \
+                                             J                    mb,                \
+                                             const T*             alpha_device_host, \
+                                             J                    size_of_mask,      \
+                                             const J*             bsr_mask_ptr,      \
+                                             const I*             bsr_row_ptr,       \
+                                             const I*             bsr_end_ptr,       \
+                                             const J*             bsr_col_ind,       \
+                                             const T*             bsr_val,           \
+                                             J                    block_dim,         \
+                                             const T*             x,                 \
+                                             const T*             beta_device_host,  \
+                                             T*                   y,                 \
+                                             rocsparse_index_base base)
 
 INSTANTIATE(float, int32_t, int32_t);
 INSTANTIATE(double, int32_t, int32_t);
@@ -302,37 +296,22 @@ INSTANTIATE(rocsparse_float_complex, int64_t, int64_t);
 INSTANTIATE(rocsparse_double_complex, int64_t, int64_t);
 #undef INSTANTIATE
 
-#define INSTANTIATE_MIXED(T, I, J, A, X, Y)                                             \
-    template void rocsparse::bsrxmvn_general<T>(rocsparse_handle     handle,            \
-                                                rocsparse_direction  dir,               \
-                                                J                    mb,                \
-                                                const T*             alpha_device_host, \
-                                                J                    size_of_mask,      \
-                                                const J*             bsr_mask_ptr,      \
-                                                const I*             bsr_row_ptr,       \
-                                                const I*             bsr_end_ptr,       \
-                                                const J*             bsr_col_ind,       \
-                                                const A*             bsr_val,           \
-                                                J                    block_dim,         \
-                                                const X*             x,                 \
-                                                const T*             beta_device_host,  \
-                                                Y*                   y,                 \
-                                                rocsparse_index_base base);             \
-    template void rocsparse::bsrxmvn_general<T>(rocsparse_handle     handle,            \
-                                                rocsparse_direction  dir,               \
-                                                J                    mb,                \
-                                                T                    alpha_device_host, \
-                                                J                    size_of_mask,      \
-                                                const J*             bsr_mask_ptr,      \
-                                                const I*             bsr_row_ptr,       \
-                                                const I*             bsr_end_ptr,       \
-                                                const J*             bsr_col_ind,       \
-                                                const A*             bsr_val,           \
-                                                J                    block_dim,         \
-                                                const X*             x,                 \
-                                                T                    beta_device_host,  \
-                                                Y*                   y,                 \
-                                                rocsparse_index_base base)
+#define INSTANTIATE_MIXED(T, I, J, A, X, Y)                                          \
+    template void rocsparse::bsrxmvn_general(rocsparse_handle     handle,            \
+                                             rocsparse_direction  dir,               \
+                                             J                    mb,                \
+                                             const T*             alpha_device_host, \
+                                             J                    size_of_mask,      \
+                                             const J*             bsr_mask_ptr,      \
+                                             const I*             bsr_row_ptr,       \
+                                             const I*             bsr_end_ptr,       \
+                                             const J*             bsr_col_ind,       \
+                                             const A*             bsr_val,           \
+                                             J                    block_dim,         \
+                                             const X*             x,                 \
+                                             const T*             beta_device_host,  \
+                                             Y*                   y,                 \
+                                             rocsparse_index_base base)
 
 INSTANTIATE_MIXED(int32_t, int32_t, int32_t, int8_t, int8_t, int32_t);
 INSTANTIATE_MIXED(int32_t, int64_t, int32_t, int8_t, int8_t, int32_t);
@@ -340,6 +319,9 @@ INSTANTIATE_MIXED(int32_t, int64_t, int64_t, int8_t, int8_t, int32_t);
 INSTANTIATE_MIXED(float, int32_t, int32_t, int8_t, int8_t, float);
 INSTANTIATE_MIXED(float, int64_t, int32_t, int8_t, int8_t, float);
 INSTANTIATE_MIXED(float, int64_t, int64_t, int8_t, int8_t, float);
+INSTANTIATE_MIXED(float, int32_t, int32_t, _Float16, _Float16, float);
+INSTANTIATE_MIXED(float, int64_t, int32_t, _Float16, _Float16, float);
+INSTANTIATE_MIXED(float, int64_t, int64_t, _Float16, _Float16, float);
 INSTANTIATE_MIXED(rocsparse_float_complex,
                   int32_t,
                   int32_t,
