@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,10 +23,9 @@
  * ************************************************************************ */
 
 #include "rocsparse_coosv.hpp"
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csrsv.hpp"
-#include "utility.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_utility.hpp"
 
 #include "../conversion/rocsparse_coo2csr.hpp"
 
@@ -42,6 +41,8 @@ rocsparse_status rocsparse::coosv_buffer_size_template(rocsparse_handle
                                                        rocsparse_mat_info        info,
                                                        size_t*                   buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for valid handle and matrix descriptor
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_POINTER(4, descr);
@@ -63,8 +64,11 @@ rocsparse_status rocsparse::coosv_buffer_size_template(rocsparse_handle
     ROCSPARSE_CHECKARG_ENUM(1, trans);
 
     // Check matrix type
-    ROCSPARSE_CHECKARG(
-        4, descr, (descr->type != rocsparse_matrix_type_general), rocsparse_status_not_implemented);
+    ROCSPARSE_CHECKARG(4,
+                       descr,
+                       (descr->type != rocsparse_matrix_type_general
+                        && descr->type != rocsparse_matrix_type_triangular),
+                       rocsparse_status_not_implemented);
 
     // Check matrix sorting mode
     ROCSPARSE_CHECKARG(4,
@@ -155,6 +159,8 @@ rocsparse_status rocsparse::coosv_analysis_template(rocsparse_handle          ha
                                                     rocsparse_solve_policy    solve,
                                                     void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for valid handle and matrix descriptor
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_POINTER(4, descr);
@@ -180,8 +186,11 @@ rocsparse_status rocsparse::coosv_analysis_template(rocsparse_handle          ha
     ROCSPARSE_CHECKARG_ENUM(10, solve);
 
     // Check matrix type
-    ROCSPARSE_CHECKARG(
-        4, descr, (descr->type != rocsparse_matrix_type_general), rocsparse_status_not_implemented);
+    ROCSPARSE_CHECKARG(4,
+                       descr,
+                       (descr->type != rocsparse_matrix_type_general
+                        && descr->type != rocsparse_matrix_type_triangular),
+                       rocsparse_status_not_implemented);
     // Check matrix sorting mode
     ROCSPARSE_CHECKARG(4,
                        descr,
@@ -308,6 +317,8 @@ rocsparse_status rocsparse::coosv_solve_template(rocsparse_handle          handl
                                                  rocsparse_solve_policy    policy,
                                                  void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Check for valid handle and matrix descriptor
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_POINTER(5, descr);
@@ -334,8 +345,11 @@ rocsparse_status rocsparse::coosv_solve_template(rocsparse_handle          handl
     ROCSPARSE_CHECKARG_ENUM(12, policy);
 
     // Check matrix type
-    ROCSPARSE_CHECKARG(
-        5, descr, (descr->type != rocsparse_matrix_type_general), rocsparse_status_not_implemented);
+    ROCSPARSE_CHECKARG(5,
+                       descr,
+                       (descr->type != rocsparse_matrix_type_general
+                        && descr->type != rocsparse_matrix_type_triangular),
+                       rocsparse_status_not_implemented);
 
     // Check matrix sorting mode
     ROCSPARSE_CHECKARG(5,
