@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,10 +22,10 @@
  *
  * ************************************************************************ */
 
-#include "control.h"
 #include "internal/level2/rocsparse_csritsv.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csritsv.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 template <typename I, typename J, typename T>
 rocsparse_status rocsparse::csritsv_buffer_size_template(rocsparse_handle          handle,
@@ -39,9 +39,10 @@ rocsparse_status rocsparse::csritsv_buffer_size_template(rocsparse_handle
                                                          rocsparse_mat_info        info,
                                                          size_t*                   buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Quick return if possible
-    if(m == 0 || nnz == 0)
+    if(m == 0)
     {
         *buffer_size = 0;
         return rocsparse_status_success;
@@ -49,7 +50,14 @@ rocsparse_status rocsparse::csritsv_buffer_size_template(rocsparse_handle
 
     if(rocsparse_diag_type_non_unit == descr->diag_type)
     {
-        *buffer_size = sizeof(T) * m * 2 + sizeof(T) * 4;
+        if(nnz == 0)
+        {
+            *buffer_size = 0;
+        }
+        else
+        {
+            *buffer_size = sizeof(T) * m * 2 + sizeof(T) * 4;
+        }
     }
     else
     {
@@ -72,6 +80,8 @@ namespace rocsparse
                                               rocsparse_mat_info        info,
                                               size_t*                   buffer_size)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Check for valid handle and matrix descriptor
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_POINTER(4, descr);
@@ -172,6 +182,7 @@ INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
                                      size_t*                   buffer_size)                       \
     try                                                                                           \
     {                                                                                             \
+        ROCSPARSE_ROUTINE_TRACE;                                                                  \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csritsv_buffer_size_impl(                            \
             handle, trans, m, nnz, descr, csr_val, csr_row_ptr, csr_col_ind, info, buffer_size)); \
         return rocsparse_status_success;                                                          \
