@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
  * ************************************************************************ */
 #pragma once
 
-#include "common.h"
+#include "rocsparse_common.hpp"
 
 namespace rocsparse
 {
@@ -355,12 +355,12 @@ namespace rocsparse
     }
 
     // Do the final block reduction of the block reduction buffers back into global memory
-    template <unsigned int BLOCKSIZE, typename I, typename J, typename T>
+    template <unsigned int BLOCKSIZE, typename I, typename J, typename C, typename T>
     ROCSPARSE_KERNEL(BLOCKSIZE)
     void csrmmnn_general_block_reduce(I nblocks,
                                       const J* __restrict__ row_block_red,
                                       const T* __restrict__ val_block_red,
-                                      T*              dense_C,
+                                      C*              dense_C,
                                       int64_t         ldc,
                                       rocsparse_order order_C)
     {
@@ -408,31 +408,6 @@ namespace rocsparse
         }
     }
 
-    // Scale kernel for beta != 1.0
-    template <unsigned int BLOCKSIZE, typename I, typename C, typename T>
-    ROCSPARSE_DEVICE_ILF void csrmmnn_nnz_split_scale_device(
-        I m, I n, T beta, C* __restrict__ data, int64_t ld, rocsparse_order order_C)
-    {
-        const I gid = hipBlockIdx_x * BLOCKSIZE + hipThreadIdx_x;
-
-        if(gid >= m * n)
-        {
-            return;
-        }
-
-        const I wid = (order_C == rocsparse_order_column) ? gid / m : gid / n;
-        const I lid = (order_C == rocsparse_order_column) ? gid % m : gid % n;
-
-        if(beta == 0)
-        {
-            data[lid + ld * wid] = 0;
-        }
-        else
-        {
-            data[lid + ld * wid] *= beta;
-        }
-    }
-
     template <unsigned int BLOCKSIZE, unsigned int NNZ_PER_BLOCK, typename I, typename J>
     ROCSPARSE_KERNEL(BLOCKSIZE)
     void csrmmnn_nnz_split_compute_row_limits(J m,
@@ -556,7 +531,8 @@ namespace rocsparse
                         for(unsigned int p = 0; p < LOOPS; p++)
                         {
                             rocsparse::atomic_add(
-                                &dense_C[(colB + p * WF_SIZE) * ldc + current_row], alpha * sum[p]);
+                                &dense_C[(colB + p * WF_SIZE) * ldc + current_row],
+                                static_cast<C>(alpha * sum[p]));
                         }
                     }
                     else
@@ -564,7 +540,7 @@ namespace rocsparse
                         for(unsigned int p = 0; p < LOOPS; p++)
                         {
                             rocsparse::atomic_add(&dense_C[current_row * ldc + colB + p * WF_SIZE],
-                                                  alpha * sum[p]);
+                                                  static_cast<C>(alpha * sum[p]));
                         }
                     }
 
@@ -588,7 +564,7 @@ namespace rocsparse
                 for(unsigned int p = 0; p < LOOPS; p++)
                 {
                     rocsparse::atomic_add(&dense_C[(colB + p * WF_SIZE) * ldc + current_row],
-                                          alpha * sum[p]);
+                                          static_cast<C>(alpha * sum[p]));
                 }
             }
             else
@@ -596,7 +572,7 @@ namespace rocsparse
                 for(unsigned int p = 0; p < LOOPS; p++)
                 {
                     rocsparse::atomic_add(&dense_C[current_row * ldc + colB + p * WF_SIZE],
-                                          alpha * sum[p]);
+                                          static_cast<C>(alpha * sum[p]));
                 }
             }
         }
@@ -684,11 +660,13 @@ namespace rocsparse
                     {
                         if(order_C == rocsparse_order_column)
                         {
-                            rocsparse::atomic_add(&dense_C[colB * ldc + current_row], alpha * sum);
+                            rocsparse::atomic_add(&dense_C[colB * ldc + current_row],
+                                                  static_cast<C>(alpha * sum));
                         }
                         else
                         {
-                            rocsparse::atomic_add(&dense_C[current_row * ldc + colB], alpha * sum);
+                            rocsparse::atomic_add(&dense_C[current_row * ldc + colB],
+                                                  static_cast<C>(alpha * sum));
                         }
                     }
 
@@ -738,12 +716,12 @@ namespace rocsparse
                         if(order_C == rocsparse_order_column)
                         {
                             rocsparse::atomic_add(&dense_C[(l + swid) * ldc + current_row],
-                                                  alpha * sum);
+                                                  static_cast<C>(alpha * sum));
                         }
                         else
                         {
                             rocsparse::atomic_add(&dense_C[current_row * ldc + (l + swid)],
-                                                  alpha * sum);
+                                                  static_cast<C>(alpha * sum));
                         }
                     }
                 }
@@ -758,12 +736,12 @@ namespace rocsparse
                         if(order_C == rocsparse_order_column)
                         {
                             rocsparse::atomic_add(&dense_C[(l + swid) * ldc + current_row],
-                                                  alpha * sum);
+                                                  static_cast<C>(alpha * sum));
                         }
                         else
                         {
                             rocsparse::atomic_add(&dense_C[current_row * ldc + (l + swid)],
-                                                  alpha * sum);
+                                                  static_cast<C>(alpha * sum));
                         }
                     }
                 }
