@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
-* Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+* Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -22,9 +22,9 @@
 *
 * ************************************************************************ */
 
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_coomm.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 namespace rocsparse
 {
@@ -42,6 +42,8 @@ namespace rocsparse
                                                        const I*                  coo_col_ind,
                                                        void*                     temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         if(m == 0 || n == 0 || k == 0)
         {
             return rocsparse_status_success;
@@ -63,6 +65,8 @@ namespace rocsparse
                                                     const I*                  coo_col_ind, //10
                                                     void*                     temp_buffer) //11
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_ENUM(1, trans_A);
         ROCSPARSE_CHECKARG_ENUM(2, alg);
@@ -121,6 +125,8 @@ namespace rocsparse
                                                 const I*                  coo_col_ind,
                                                 void*                     temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(alg)
         {
         case rocsparse_coomm_alg_default:
@@ -139,7 +145,9 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 }
 
@@ -157,6 +165,7 @@ rocsparse_status rocsparse::coomm_analysis_template(rocsparse_handle          ha
                                                     const I*                  coo_col_ind,
                                                     void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     const rocsparse_status status = rocsparse::coomm_analysis_quickreturn<T>(
         handle, trans_A, alg, m, n, k, nnz, descr, coo_val, coo_row_ind, coo_col_ind, temp_buffer);
@@ -189,6 +198,7 @@ namespace rocsparse
                                          const I*                  coo_col_ind,
                                          void*                     temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
 
         rocsparse::log_trace(handle,
                              "rocsparse_coomm_analysis",
@@ -266,6 +276,8 @@ INSTANTIATE_ANALYSIS(rocsparse_double_complex, int32_t, rocsparse_double_complex
 INSTANTIATE_ANALYSIS(rocsparse_double_complex, int64_t, rocsparse_double_complex);
 
 // Mixed precisions
+INSTANTIATE_ANALYSIS(float, int32_t, _Float16);
+INSTANTIATE_ANALYSIS(float, int64_t, _Float16);
 INSTANTIATE_ANALYSIS(int32_t, int32_t, int8_t);
 INSTANTIATE_ANALYSIS(int32_t, int64_t, int8_t);
 INSTANTIATE_ANALYSIS(float, int32_t, int8_t);
