@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,13 +27,14 @@
 
 #include "../level2/rocsparse_gebsrmv.hpp"
 #include "rocsparse_bsrmm.hpp"
+#include "rocsparse_common.h"
 
-#include "common.h"
-#include "utility.h"
+#include "rocsparse_common.hpp"
+#include "rocsparse_utility.hpp"
 
 namespace rocsparse
 {
-    template <typename T, typename U>
+    template <typename T>
     rocsparse_status gebsrmm_template_small(rocsparse_handle          handle,
                                             rocsparse_direction       dir,
                                             rocsparse_operation       trans_A,
@@ -42,7 +43,7 @@ namespace rocsparse
                                             rocsparse_int             n,
                                             rocsparse_int             kb,
                                             rocsparse_int             nnzb,
-                                            U                         alpha,
+                                            const T*                  alpha,
                                             const rocsparse_mat_descr descr,
                                             const T*                  bsr_val,
                                             const rocsparse_int*      bsr_row_ptr,
@@ -51,11 +52,11 @@ namespace rocsparse
                                             rocsparse_int             col_block_dim,
                                             const T*                  B,
                                             int64_t                   ldb,
-                                            U                         beta,
+                                            const T*                  beta,
                                             T*                        C,
                                             int64_t                   ldc);
 
-    template <typename T, typename U>
+    template <typename T>
     rocsparse_status gebsrmm_template_large_ext(rocsparse_handle          handle,
                                                 rocsparse_direction       dir,
                                                 rocsparse_operation       trans_A,
@@ -64,7 +65,7 @@ namespace rocsparse
                                                 rocsparse_int             n,
                                                 rocsparse_int             kb,
                                                 rocsparse_int             nnzb,
-                                                U                         alpha,
+                                                const T*                  alpha,
                                                 const rocsparse_mat_descr descr,
                                                 const T*                  bsr_val,
                                                 const rocsparse_int*      bsr_row_ptr,
@@ -73,11 +74,11 @@ namespace rocsparse
                                                 rocsparse_int             col_block_dim,
                                                 const T*                  B,
                                                 int64_t                   ldb,
-                                                U                         beta,
+                                                const T*                  beta,
                                                 T*                        C,
                                                 int64_t                   ldc);
 
-    template <typename T, typename U>
+    template <typename T>
     rocsparse_status gebsrmm_template_general(rocsparse_handle          handle,
                                               rocsparse_direction       dir,
                                               rocsparse_operation       trans_A,
@@ -86,7 +87,7 @@ namespace rocsparse
                                               rocsparse_int             n,
                                               rocsparse_int             kb,
                                               rocsparse_int             nnzb,
-                                              U                         alpha,
+                                              const T*                  alpha,
                                               const rocsparse_mat_descr descr,
                                               const T*                  bsr_val,
                                               const rocsparse_int*      bsr_row_ptr,
@@ -95,12 +96,12 @@ namespace rocsparse
                                               rocsparse_int             col_block_dim,
                                               const T*                  B,
                                               int64_t                   ldb,
-                                              U                         beta,
+                                              const T*                  beta,
                                               T*                        C,
                                               int64_t                   ldc);
 }
 
-template <typename T, typename U>
+template <typename T>
 rocsparse_status rocsparse::gebsrmm_template_dispatch(rocsparse_handle          handle,
                                                       rocsparse_direction       dir,
                                                       rocsparse_operation       trans_A,
@@ -109,7 +110,7 @@ rocsparse_status rocsparse::gebsrmm_template_dispatch(rocsparse_handle
                                                       rocsparse_int             n,
                                                       rocsparse_int             kb,
                                                       rocsparse_int             nnzb,
-                                                      U                         alpha,
+                                                      const T*                  alpha,
                                                       const rocsparse_mat_descr descr,
                                                       const T*                  bsr_val,
                                                       const rocsparse_int*      bsr_row_ptr,
@@ -118,10 +119,12 @@ rocsparse_status rocsparse::gebsrmm_template_dispatch(rocsparse_handle
                                                       rocsparse_int             col_block_dim,
                                                       const T*                  B,
                                                       int64_t                   ldb,
-                                                      U                         beta,
+                                                      const T*                  beta,
                                                       T*                        C,
                                                       int64_t                   ldc)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const rocsparse_int block_dim = rocsparse::max(row_block_dim, col_block_dim);
     if(row_block_dim == col_block_dim)
     {
@@ -129,10 +132,14 @@ rocsparse_status rocsparse::gebsrmm_template_dispatch(rocsparse_handle
                                                                      dir,
                                                                      trans_A,
                                                                      trans_B,
+                                                                     rocsparse_bsrmm_alg_default,
                                                                      mb,
                                                                      n,
                                                                      kb,
                                                                      nnzb,
+                                                                     static_cast<rocsparse_int>(1),
+                                                                     static_cast<int64_t>(0),
+                                                                     static_cast<int64_t>(0),
                                                                      alpha,
                                                                      descr,
                                                                      bsr_val,
@@ -141,9 +148,15 @@ rocsparse_status rocsparse::gebsrmm_template_dispatch(rocsparse_handle
                                                                      block_dim,
                                                                      B,
                                                                      ldb,
+                                                                     static_cast<rocsparse_int>(1),
+                                                                     static_cast<int64_t>(0),
+                                                                     rocsparse_order_column,
                                                                      beta,
                                                                      C,
-                                                                     ldc));
+                                                                     ldc,
+                                                                     static_cast<rocsparse_int>(1),
+                                                                     static_cast<int64_t>(0),
+                                                                     rocsparse_order_column));
         return rocsparse_status_success;
     }
 
@@ -253,29 +266,31 @@ namespace rocsparse
                                                 rocsparse_direction       dir,
                                                 rocsparse_operation       trans_A,
                                                 rocsparse_operation       trans_B,
-                                                rocsparse_int             mb,
-                                                rocsparse_int             n,
-                                                rocsparse_int             kb,
-                                                rocsparse_int             nnzb,
+                                                int64_t                   mb,
+                                                int64_t                   n,
+                                                int64_t                   kb,
+                                                int64_t                   nnzb,
                                                 const T*                  alpha,
                                                 const rocsparse_mat_descr descr,
-                                                const T*                  bsr_val,
-                                                const rocsparse_int*      bsr_row_ptr,
-                                                const rocsparse_int*      bsr_col_ind,
-                                                rocsparse_int             row_block_dim,
-                                                rocsparse_int             col_block_dim,
-                                                const T*                  B,
+                                                const void*               bsr_val,
+                                                const void*               bsr_row_ptr,
+                                                const void*               bsr_col_ind,
+                                                int64_t                   row_block_dim,
+                                                int64_t                   col_block_dim,
+                                                const void*               B,
                                                 int64_t                   ldb,
                                                 const T*                  beta,
                                                 T*                        C,
                                                 int64_t                   ldc)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Quick return if possible
         if(mb == 0 || n == 0 || kb == 0)
         {
             // matrix never accessed however still need to update C matrix
-            rocsparse_int m      = row_block_dim * mb;
-            rocsparse_int C_size = m * n;
+            const int64_t m      = row_block_dim * mb;
+            const int64_t C_size = m * n;
             if(C_size > 0)
             {
                 if(C == nullptr && beta == nullptr)
@@ -283,36 +298,8 @@ namespace rocsparse
                     return rocsparse_status_invalid_pointer;
                 }
 
-                if(handle->pointer_mode == rocsparse_pointer_mode_device)
-                {
-                    RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::scale_array_2d<256>),
-                                                       dim3((C_size - 1) / 256 + 1),
-                                                       dim3(256),
-                                                       0,
-                                                       handle->stream,
-                                                       m,
-                                                       n,
-                                                       ldc,
-                                                       0,
-                                                       C,
-                                                       beta,
-                                                       rocsparse_order_column);
-                }
-                else
-                {
-                    RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::scale_array_2d<256>),
-                                                       dim3((C_size - 1) / 256 + 1),
-                                                       dim3(256),
-                                                       0,
-                                                       handle->stream,
-                                                       m,
-                                                       n,
-                                                       ldc,
-                                                       0,
-                                                       C,
-                                                       *beta,
-                                                       rocsparse_order_column);
-                }
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse::scale_2d_array(
+                    handle, m, n, ldc, 1, 0, beta, C, rocsparse_order_column));
             }
 
             return rocsparse_status_success;
@@ -342,6 +329,8 @@ namespace rocsparse
                                              T*                        C, //18
                                              int64_t                   ldc) //19
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_POINTER(9, descr);
         ROCSPARSE_CHECKARG_ENUM(1, dir);
@@ -502,54 +491,29 @@ namespace rocsparse
                                          T*                        C,
                                          rocsparse_int             ldc)
     {
-        if(handle->pointer_mode == rocsparse_pointer_mode_device)
-        {
-            RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsrmm_template_dispatch(handle,
-                                                                           dir,
-                                                                           trans_A,
-                                                                           trans_B,
-                                                                           mb,
-                                                                           n,
-                                                                           kb,
-                                                                           nnzb,
-                                                                           alpha,
-                                                                           descr,
-                                                                           bsr_val,
-                                                                           bsr_row_ptr,
-                                                                           bsr_col_ind,
-                                                                           row_block_dim,
-                                                                           col_block_dim,
-                                                                           B,
-                                                                           ldb,
-                                                                           beta,
-                                                                           C,
-                                                                           ldc));
-            return rocsparse_status_success;
-        }
-        else
-        {
-            RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsrmm_template_dispatch(handle,
-                                                                           dir,
-                                                                           trans_A,
-                                                                           trans_B,
-                                                                           mb,
-                                                                           n,
-                                                                           kb,
-                                                                           nnzb,
-                                                                           *alpha,
-                                                                           descr,
-                                                                           bsr_val,
-                                                                           bsr_row_ptr,
-                                                                           bsr_col_ind,
-                                                                           row_block_dim,
-                                                                           col_block_dim,
-                                                                           B,
-                                                                           ldb,
-                                                                           *beta,
-                                                                           C,
-                                                                           ldc));
-            return rocsparse_status_success;
-        }
+        ROCSPARSE_ROUTINE_TRACE;
+
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsrmm_template_dispatch(handle,
+                                                                       dir,
+                                                                       trans_A,
+                                                                       trans_B,
+                                                                       mb,
+                                                                       n,
+                                                                       kb,
+                                                                       nnzb,
+                                                                       alpha,
+                                                                       descr,
+                                                                       bsr_val,
+                                                                       bsr_row_ptr,
+                                                                       bsr_col_ind,
+                                                                       row_block_dim,
+                                                                       col_block_dim,
+                                                                       B,
+                                                                       ldb,
+                                                                       beta,
+                                                                       C,
+                                                                       ldc));
+        return rocsparse_status_success;
     }
 
     template <typename... P>
@@ -588,6 +552,7 @@ namespace rocsparse
                                   T*                        C,
                                   rocsparse_int             ldc)
     {
+        ROCSPARSE_ROUTINE_TRACE;
 
         // Logging TODO bench logging
         rocsparse::log_trace(handle,
@@ -692,6 +657,7 @@ namespace rocsparse
                                      rocsparse_int             ldc)           \
     try                                                                       \
     {                                                                         \
+        ROCSPARSE_ROUTINE_TRACE;                                              \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsrmm_impl(handle,             \
                                                           dir,                \
                                                           trans_A,            \
