@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -21,15 +21,101 @@
  *
  * ************************************************************************ */
 
-#include "control.h"
-#include "handle.h"
+#include <map>
+#include <sstream>
+
 #include "rocsparse.h"
-#include "utility.h"
+#include "rocsparse_control.hpp"
+#include "rocsparse_enum_utils.hpp"
+#include "rocsparse_handle.hpp"
+#include "rocsparse_utility.hpp"
 
 #include "rocsparse_bellmm.hpp"
+#include "rocsparse_bsrmm.hpp"
 #include "rocsparse_coomm.hpp"
 #include "rocsparse_cscmm.hpp"
 #include "rocsparse_csrmm.hpp"
+#include "rocsparse_determine_indextype.hpp"
+
+template <>
+const char* rocsparse::enum_utils::to_string(rocsparse_spmm_alg value_)
+{
+#define CASE(C) \
+    case C:     \
+        return #C
+    switch(value_)
+    {
+        CASE(rocsparse_spmm_alg_default);
+        CASE(rocsparse_spmm_alg_csr);
+        CASE(rocsparse_spmm_alg_coo_segmented);
+        CASE(rocsparse_spmm_alg_coo_atomic);
+        CASE(rocsparse_spmm_alg_csr_row_split);
+        CASE(rocsparse_spmm_alg_csr_nnz_split);
+        CASE(rocsparse_spmm_alg_csr_merge_path);
+        CASE(rocsparse_spmm_alg_coo_segmented_atomic);
+        CASE(rocsparse_spmm_alg_bell);
+        CASE(rocsparse_spmm_alg_bsr);
+#undef CASE
+    }
+    // LCOV_EXCL_START
+    THROW_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
+}
+
+template <>
+const char* rocsparse::enum_utils::to_string(rocsparse_spmm_stage value_)
+{
+#define CASE(C) \
+    case C:     \
+        return #C
+    switch(value_)
+    {
+        CASE(rocsparse_spmm_stage_buffer_size);
+        CASE(rocsparse_spmm_stage_preprocess);
+        CASE(rocsparse_spmm_stage_compute);
+#undef CASE
+    }
+    // LCOV_EXCL_START
+    THROW_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
+}
+
+template <>
+bool rocsparse::enum_utils::is_invalid(rocsparse_spmm_alg value_)
+{
+    switch(value_)
+    {
+    case rocsparse_spmm_alg_default:
+    case rocsparse_spmm_alg_csr:
+    case rocsparse_spmm_alg_coo_segmented:
+    case rocsparse_spmm_alg_coo_atomic:
+    case rocsparse_spmm_alg_csr_row_split:
+    case rocsparse_spmm_alg_csr_nnz_split:
+    case rocsparse_spmm_alg_csr_merge_path:
+    case rocsparse_spmm_alg_coo_segmented_atomic:
+    case rocsparse_spmm_alg_bell:
+    case rocsparse_spmm_alg_bsr:
+    {
+        return false;
+    }
+    }
+    return true;
+}
+
+template <>
+bool rocsparse::enum_utils::is_invalid(rocsparse_spmm_stage value_)
+{
+    switch(value_)
+    {
+    case rocsparse_spmm_stage_buffer_size:
+    case rocsparse_spmm_stage_preprocess:
+    case rocsparse_spmm_stage_compute:
+    {
+        return false;
+    }
+    }
+    return true;
+}
 
 namespace rocsparse
 {
@@ -54,10 +140,12 @@ namespace rocsparse
         case rocsparse_spmm_alg_coo_atomic:
         case rocsparse_spmm_alg_coo_segmented_atomic:
         {
+            // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
         }
         }
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     rocsparse_status spmm_alg2csrmm_alg(rocsparse_spmm_alg spmm_alg, rocsparse_csrmm_alg& csrmm_alg)
@@ -95,10 +183,12 @@ namespace rocsparse
         case rocsparse_spmm_alg_coo_atomic:
         case rocsparse_spmm_alg_coo_segmented_atomic:
         {
+            // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
         }
         }
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     rocsparse_status spmm_alg2coomm_alg(rocsparse_spmm_alg spmm_alg, rocsparse_coomm_alg& coomm_alg)
@@ -136,10 +226,40 @@ namespace rocsparse
         case rocsparse_spmm_alg_csr_nnz_split:
         case rocsparse_spmm_alg_csr_merge_path:
         {
+            // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
         }
         }
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
+    }
+
+    rocsparse_status spmm_alg2bsrmm_alg(rocsparse_spmm_alg spmm_alg, rocsparse_bsrmm_alg& bsrmm_alg)
+    {
+        switch(spmm_alg)
+        {
+        case rocsparse_spmm_alg_default:
+        case rocsparse_spmm_alg_bsr:
+        {
+            bsrmm_alg = rocsparse_bsrmm_alg_default;
+            return rocsparse_status_success;
+        }
+
+        case rocsparse_spmm_alg_csr:
+        case rocsparse_spmm_alg_csr_row_split:
+        case rocsparse_spmm_alg_csr_nnz_split:
+        case rocsparse_spmm_alg_csr_merge_path:
+        case rocsparse_spmm_alg_bell:
+        case rocsparse_spmm_alg_coo_segmented:
+        case rocsparse_spmm_alg_coo_atomic:
+        case rocsparse_spmm_alg_coo_segmented_atomic:
+        {
+            // LCOV_EXCL_START
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        }
+        }
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     template <typename T, typename I, typename J, typename A, typename B, typename C>
@@ -156,6 +276,8 @@ namespace rocsparse
                                    size_t*                     buffer_size,
                                    void*                       temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         switch(mat_A->format)
         {
         case rocsparse_format_csr:
@@ -519,321 +641,317 @@ namespace rocsparse
             break;
         }
 
+        case rocsparse_format_bsr:
+        {
+            rocsparse_bsrmm_alg bsrmm_alg;
+            RETURN_IF_ROCSPARSE_ERROR((rocsparse::spmm_alg2bsrmm_alg(alg, bsrmm_alg)));
+
+            const J mb = (J)mat_A->rows;
+            const J n  = (J)mat_C->cols;
+            const J kb = (J)mat_A->cols;
+
+            switch(stage)
+            {
+            case rocsparse_spmm_stage_buffer_size:
+            {
+                RETURN_IF_ROCSPARSE_ERROR(
+                    rocsparse::bsrmm_buffer_size_template<T>(handle,
+                                                             trans_A,
+                                                             bsrmm_alg,
+                                                             mb,
+                                                             n,
+                                                             kb,
+                                                             (I)mat_A->nnz,
+                                                             mat_A->descr,
+                                                             (const A*)mat_A->const_val_data,
+                                                             (const I*)mat_A->const_row_data,
+                                                             (const J*)mat_A->const_col_data,
+                                                             (J)mat_A->block_dim,
+                                                             buffer_size));
+                return rocsparse_status_success;
+            }
+            case rocsparse_spmm_stage_preprocess:
+            {
+                RETURN_IF_ROCSPARSE_ERROR(
+                    rocsparse::bsrmm_analysis_template<T>(handle,
+                                                          trans_A,
+                                                          bsrmm_alg,
+                                                          mb,
+                                                          n,
+                                                          kb,
+                                                          (I)mat_A->nnz,
+                                                          mat_A->descr,
+                                                          (const A*)mat_A->const_val_data,
+                                                          (const I*)mat_A->const_row_data,
+                                                          (const J*)mat_A->const_col_data,
+                                                          (J)mat_A->block_dim,
+                                                          temp_buffer));
+                return rocsparse_status_success;
+            }
+            case rocsparse_spmm_stage_compute:
+            {
+                RETURN_IF_ROCSPARSE_ERROR(
+                    rocsparse::bsrmm_template(handle,
+                                              mat_A->block_dir,
+                                              trans_A,
+                                              trans_B,
+                                              bsrmm_alg,
+                                              mb,
+                                              n,
+                                              kb,
+                                              (I)mat_A->nnz,
+                                              (J)mat_A->batch_count,
+                                              mat_A->offsets_batch_stride,
+                                              mat_A->columns_values_batch_stride,
+                                              (const T*)alpha,
+                                              mat_A->descr,
+                                              (const A*)mat_A->const_val_data,
+                                              (const I*)mat_A->const_row_data,
+                                              (const J*)mat_A->const_col_data,
+                                              (J)mat_A->block_dim,
+                                              (const B*)mat_B->const_values,
+                                              mat_B->ld,
+                                              (J)mat_B->batch_count,
+                                              mat_B->batch_stride,
+                                              mat_B->order,
+                                              (const T*)beta,
+                                              (C*)mat_C->values,
+                                              mat_C->ld,
+                                              (J)mat_C->batch_count,
+                                              mat_C->batch_stride,
+                                              mat_C->order));
+                return rocsparse_status_success;
+            }
+            }
+        }
+
         case rocsparse_format_coo_aos:
         case rocsparse_format_ell:
-        case rocsparse_format_bsr:
         {
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
         }
+            // LCOV_EXCL_START
         }
 
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
-    template <typename... Ts>
-    static inline rocsparse_status spmm_dynamic_dispatch(rocsparse_indextype itype,
-                                                         rocsparse_indextype jtype,
-                                                         rocsparse_datatype  atype,
-                                                         rocsparse_datatype  btype,
-                                                         rocsparse_datatype  ctype,
-                                                         rocsparse_datatype  compute_type,
-                                                         Ts&&... ts)
+    typedef rocsparse_status (*spmm_template_t)(rocsparse_handle            handle,
+                                                rocsparse_operation         trans_A,
+                                                rocsparse_operation         trans_B,
+                                                const void*                 alpha,
+                                                rocsparse_const_spmat_descr mat_A,
+                                                rocsparse_const_dnmat_descr mat_B,
+                                                const void*                 beta,
+                                                const rocsparse_dnmat_descr mat_C,
+                                                rocsparse_spmm_alg          alg,
+                                                rocsparse_spmm_stage        stage,
+                                                size_t*                     buffer_size,
+                                                void*                       temp_buffer);
+
+    using spmm_template_tuple = std::tuple<rocsparse_datatype,
+                                           rocsparse_indextype,
+                                           rocsparse_indextype,
+                                           rocsparse_datatype,
+                                           rocsparse_datatype,
+                                           rocsparse_datatype>;
+    // clang-format off
+#define SPMM_TEMPLATE_CONFIG(T_, I_, J_, A_, B_, C_)                        \
+    {                                                                       \
+        spmm_template_tuple(T_, I_, J_, A_, B_, C_),                        \
+            spmm_template<typename rocsparse::datatype_traits<T_>::type_t,  \
+                          typename rocsparse::indextype_traits<I_>::type_t, \
+                          typename rocsparse::indextype_traits<J_>::type_t, \
+                          typename rocsparse::datatype_traits<A_>::type_t,  \
+                          typename rocsparse::datatype_traits<B_>::type_t,  \
+                          typename rocsparse::datatype_traits<C_>::type_t>  \
+    }
+    // clang-format on
+
+    static const std::map<spmm_template_tuple, spmm_template_t> s_spmm_template_dispatch{{
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_i32_r,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_i32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_i32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f32_r,
+                             rocsparse_datatype_f32_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f32_r,
+                             rocsparse_datatype_f32_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_f32_r,
+                             rocsparse_datatype_f32_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_i8_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f16_r,
+                             rocsparse_datatype_f16_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f16_r,
+                             rocsparse_datatype_f16_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_f16_r,
+                             rocsparse_datatype_f16_r,
+                             rocsparse_datatype_f32_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f64_r,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f64_r,
+                             rocsparse_datatype_f64_r,
+                             rocsparse_datatype_f64_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f64_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f64_r,
+                             rocsparse_datatype_f64_r,
+                             rocsparse_datatype_f64_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f64_r,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_f64_r,
+                             rocsparse_datatype_f64_r,
+                             rocsparse_datatype_f64_r),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f64_c,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f64_c,
+                             rocsparse_datatype_f64_c,
+                             rocsparse_datatype_f64_c),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f64_c,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f64_c,
+                             rocsparse_datatype_f64_c,
+                             rocsparse_datatype_f64_c),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f64_c,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_f64_c,
+                             rocsparse_datatype_f64_c,
+                             rocsparse_datatype_f64_c),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_c,
+                             rocsparse_indextype_i32,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f32_c,
+                             rocsparse_datatype_f32_c,
+                             rocsparse_datatype_f32_c),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_c,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i32,
+                             rocsparse_datatype_f32_c,
+                             rocsparse_datatype_f32_c,
+                             rocsparse_datatype_f32_c),
+
+        SPMM_TEMPLATE_CONFIG(rocsparse_datatype_f32_c,
+                             rocsparse_indextype_i64,
+                             rocsparse_indextype_i64,
+                             rocsparse_datatype_f32_c,
+                             rocsparse_datatype_f32_c,
+                             rocsparse_datatype_f32_c)}};
+
+    static rocsparse_status spmm_template_find(spmm_template_t*    spmm_function_,
+                                               rocsparse_datatype  compute_type_,
+                                               rocsparse_indextype i_type_,
+                                               rocsparse_indextype j_type_,
+                                               rocsparse_datatype  a_type_,
+                                               rocsparse_datatype  b_type_,
+                                               rocsparse_datatype  c_type_)
     {
-        rocsparse_host_assert(
-            compute_type == atype,
-            "This function is designed for atype and compute_type being the same.");
-        rocsparse_host_assert(
-            compute_type == btype,
-            "This function is designed for btype and compute_type being the same.");
-        rocsparse_host_assert(
-            compute_type == ctype,
-            "This function is designed for ctype and compute_type being the same.");
+        const auto& it = rocsparse::s_spmm_template_dispatch.find(rocsparse::spmm_template_tuple(
+            compute_type_, i_type_, j_type_, a_type_, b_type_, c_type_));
 
-        switch(compute_type)
+        if(it != rocsparse::s_spmm_template_dispatch.end())
         {
-        case rocsparse_datatype_f32_r:
+            spmm_function_[0] = it->second;
+        }
+        // LCOV_EXCL_START
+        else
         {
-            switch(itype)
-            {
-            case rocsparse_indextype_u16:
-            {
-                RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-            }
-            case rocsparse_indextype_i32:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<float, int32_t, int32_t, float, float, float>(
-                            ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            case rocsparse_indextype_i64:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<float, int64_t, int32_t, float, float, float>(
-                            ts...)));
-                    return rocsparse_status_success;
-                }
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<float, int64_t, int64_t, float, float, float>(
-                            ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            }
+            std::stringstream sstr;
+            sstr << "invalid precision configuration: "
+                 << "compute_type: " << rocsparse::enum_utils::to_string(compute_type_)
+                 << ", i_type: " << rocsparse::enum_utils::to_string(i_type_)
+                 << ", j_type: " << rocsparse::enum_utils::to_string(j_type_)
+                 << ", a_type: " << rocsparse::enum_utils::to_string(a_type_)
+                 << ", b_type: " << rocsparse::enum_utils::to_string(b_type_)
+                 << ", c_type: " << rocsparse::enum_utils::to_string(c_type_);
+
+            RETURN_WITH_MESSAGE_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value,
+                                                   sstr.str().c_str());
         }
-        case rocsparse_datatype_f64_r:
-        {
-            switch(itype)
-            {
-            case rocsparse_indextype_u16:
-            {
-                RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-            }
-            case rocsparse_indextype_i32:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<double, int32_t, int32_t, double, double, double>(
-                            ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            case rocsparse_indextype_i64:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<double, int64_t, int32_t, double, double, double>(
-                            ts...)));
-                    return rocsparse_status_success;
-                }
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<double, int64_t, int64_t, double, double, double>(
-                            ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            }
-        }
-        case rocsparse_datatype_f32_c:
-        {
-            switch(itype)
-            {
-            case rocsparse_indextype_u16:
-            {
-                RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-            }
-            case rocsparse_indextype_i32:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<rocsparse_float_complex,
-                                                  int32_t,
-                                                  int32_t,
-                                                  rocsparse_float_complex,
-                                                  rocsparse_float_complex,
-                                                  rocsparse_float_complex>(ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            case rocsparse_indextype_i64:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<rocsparse_float_complex,
-                                                  int64_t,
-                                                  int32_t,
-                                                  rocsparse_float_complex,
-                                                  rocsparse_float_complex,
-                                                  rocsparse_float_complex>(ts...)));
-                    return rocsparse_status_success;
-                }
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<rocsparse_float_complex,
-                                                  int64_t,
-                                                  int64_t,
-                                                  rocsparse_float_complex,
-                                                  rocsparse_float_complex,
-                                                  rocsparse_float_complex>(ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            }
-        }
-        case rocsparse_datatype_f64_c:
-        {
-            switch(itype)
-            {
-            case rocsparse_indextype_u16:
-            {
-                RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-            }
-            case rocsparse_indextype_i32:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<rocsparse_double_complex,
-                                                  int32_t,
-                                                  int32_t,
-                                                  rocsparse_double_complex,
-                                                  rocsparse_double_complex,
-                                                  rocsparse_double_complex>(ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            case rocsparse_indextype_i64:
-            {
-                switch(jtype)
-                {
-                case rocsparse_indextype_u16:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-                }
-                case rocsparse_indextype_i32:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<rocsparse_double_complex,
-                                                  int64_t,
-                                                  int32_t,
-                                                  rocsparse_double_complex,
-                                                  rocsparse_double_complex,
-                                                  rocsparse_double_complex>(ts...)));
-                    return rocsparse_status_success;
-                }
-                case rocsparse_indextype_i64:
-                {
-                    RETURN_IF_ROCSPARSE_ERROR(
-                        (rocsparse::spmm_template<rocsparse_double_complex,
-                                                  int64_t,
-                                                  int64_t,
-                                                  rocsparse_double_complex,
-                                                  rocsparse_double_complex,
-                                                  rocsparse_double_complex>(ts...)));
-                    return rocsparse_status_success;
-                }
-                }
-            }
-            }
-        }
-        case rocsparse_datatype_i8_r:
-        case rocsparse_datatype_u8_r:
-        case rocsparse_datatype_i32_r:
-        case rocsparse_datatype_u32_r:
-        {
-            RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
-        }
-        }
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
+
+        return rocsparse_status_success;
     }
-
-    rocsparse_indextype determine_I_index_type(rocsparse_const_spmat_descr mat)
-    {
-        switch(mat->format)
-        {
-        case rocsparse_format_coo:
-        case rocsparse_format_coo_aos:
-        case rocsparse_format_csr:
-        case rocsparse_format_ell:
-        case rocsparse_format_bell:
-        case rocsparse_format_bsr:
-        {
-            return mat->row_type;
-        }
-        case rocsparse_format_csc:
-        {
-            return mat->col_type;
-        }
-        }
-    }
-
-    rocsparse_indextype determine_J_index_type(rocsparse_const_spmat_descr mat)
-    {
-        switch(mat->format)
-        {
-        case rocsparse_format_coo:
-        case rocsparse_format_coo_aos:
-        case rocsparse_format_csr:
-        case rocsparse_format_ell:
-        case rocsparse_format_bell:
-        case rocsparse_format_bsr:
-        {
-            return mat->col_type;
-        }
-        case rocsparse_format_csc:
-        {
-            return mat->row_type;
-        }
-        }
-    }
-
 }
 
 /*
@@ -856,6 +974,8 @@ extern "C" rocsparse_status rocsparse_spmm(rocsparse_handle            handle, /
                                            void*                       temp_buffer) //12
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     rocsparse::log_trace(handle,
                          "rocsparse_spmm",
                          trans_A,
@@ -882,11 +1002,8 @@ try
     ROCSPARSE_CHECKARG_POINTER(7, mat_C);
     ROCSPARSE_CHECKARG(7, mat_C, mat_C->init == false, rocsparse_status_not_initialized);
     ROCSPARSE_CHECKARG_ENUM(8, compute_type);
-    ROCSPARSE_CHECKARG(8,
-                       compute_type,
-                       (compute_type != mat_A->data_type || compute_type != mat_B->data_type
-                        || compute_type != mat_C->data_type),
-                       rocsparse_status_not_implemented);
+    ROCSPARSE_CHECKARG(
+        8, compute_type, (compute_type != mat_C->data_type), rocsparse_status_not_implemented);
 
     ROCSPARSE_CHECKARG_ENUM(9, alg);
     ROCSPARSE_CHECKARG_ENUM(10, stage);
@@ -912,29 +1029,34 @@ try
         break;
     }
     }
-    RETURN_IF_ROCSPARSE_ERROR(
-        rocsparse::spmm_dynamic_dispatch(rocsparse::determine_I_index_type(mat_A),
-                                         rocsparse::determine_J_index_type(mat_A),
-                                         mat_A->data_type,
-                                         mat_B->data_type,
-                                         mat_C->data_type,
-                                         compute_type,
-                                         handle,
-                                         trans_A,
-                                         trans_B,
-                                         alpha,
-                                         mat_A,
-                                         mat_B,
-                                         beta,
-                                         mat_C,
-                                         alg,
-                                         stage,
-                                         buffer_size,
-                                         temp_buffer));
+
+    rocsparse::spmm_template_t spmm_function;
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::spmm_template_find(&spmm_function,
+                                                            compute_type,
+                                                            rocsparse::determine_I_indextype(mat_A),
+                                                            rocsparse::determine_J_indextype(mat_A),
+                                                            mat_A->data_type,
+                                                            mat_B->data_type,
+                                                            mat_C->data_type));
+
+    RETURN_IF_ROCSPARSE_ERROR(spmm_function(handle,
+                                            trans_A,
+                                            trans_B,
+                                            alpha,
+                                            mat_A,
+                                            mat_B,
+                                            beta,
+                                            mat_C,
+                                            alg,
+                                            stage,
+                                            buffer_size,
+                                            temp_buffer));
 
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
