@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,11 +24,11 @@
 
 #pragma once
 
-#include "common.h"
+#include "rocsparse_common.hpp"
 
 namespace rocsparse
 {
-    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, uint32_t BSRDIM, typename T, typename U>
+    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, uint32_t BSRDIM, typename T>
     ROCSPARSE_DEVICE_ILF void bsrilu0_2_8_device(rocsparse_direction  dir,
                                                  rocsparse_int        mb,
                                                  const rocsparse_int* bsr_row_ptr,
@@ -41,7 +41,7 @@ namespace rocsparse
                                                  rocsparse_int*       zero_pivot,
                                                  rocsparse_index_base idx_base,
                                                  int                  boost,
-                                                 U                    boost_tol,
+                                                 double               boost_tol,
                                                  T                    boost_val)
     {
         // Current row this wavefront is working on
@@ -172,7 +172,9 @@ namespace rocsparse
 
                         // Load BSR block from row k into shared memory
                         sdata1[threadIdx.y][threadIdx.x]
-                            = bsr_val[BSR_IND(k, threadIdx.x, threadIdx.y, dir)];
+                            = (threadIdx.x < block_dim && threadIdx.y < block_dim)
+                                  ? bsr_val[BSR_IND(k, threadIdx.x, threadIdx.y, dir)]
+                                  : static_cast<T>(0);
 
                         // Make sure all writes to shared memory are visible
                         __threadfence_block();
@@ -322,7 +324,7 @@ namespace rocsparse
         }
     }
 
-    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, uint32_t BSRDIM, typename T, typename U>
+    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, uint32_t BSRDIM, typename T>
     ROCSPARSE_DEVICE_ILF void bsrilu0_9_32_device(rocsparse_direction  dir,
                                                   rocsparse_int        mb,
                                                   const rocsparse_int* bsr_row_ptr,
@@ -335,7 +337,7 @@ namespace rocsparse
                                                   rocsparse_int*       zero_pivot,
                                                   rocsparse_index_base idx_base,
                                                   int                  boost,
-                                                  U                    boost_tol,
+                                                  double               boost_tol,
                                                   T                    boost_val)
     {
         constexpr static uint32_t DIMX = BSRDIM;
@@ -647,7 +649,7 @@ namespace rocsparse
         }
     }
 
-    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, uint32_t BSRDIM, typename T, typename U>
+    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, uint32_t BSRDIM, typename T>
     ROCSPARSE_DEVICE_ILF void bsrilu0_33_64_device(rocsparse_direction  dir,
                                                    rocsparse_int        mb,
                                                    const rocsparse_int* bsr_row_ptr,
@@ -660,7 +662,7 @@ namespace rocsparse
                                                    rocsparse_int*       zero_pivot,
                                                    rocsparse_index_base idx_base,
                                                    int                  boost,
-                                                   U                    boost_tol,
+                                                   double               boost_tol,
                                                    T                    boost_val)
     {
         constexpr static uint32_t DIMX = BSRDIM;
@@ -953,7 +955,7 @@ namespace rocsparse
         }
     }
 
-    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, bool SLEEP, typename T, typename U>
+    template <uint32_t BLOCKSIZE, uint32_t WFSIZE, bool SLEEP, typename T>
     ROCSPARSE_DEVICE_ILF void bsrilu0_general_device(rocsparse_direction  dir,
                                                      rocsparse_int        mb,
                                                      const rocsparse_int* bsr_row_ptr,
@@ -966,7 +968,7 @@ namespace rocsparse
                                                      rocsparse_int*       zero_pivot,
                                                      rocsparse_index_base idx_base,
                                                      int                  boost,
-                                                     U                    boost_tol,
+                                                     double               boost_tol,
                                                      T                    boost_val)
     {
         int lid = hipThreadIdx_x & (WFSIZE - 1);
