@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,8 @@
  *
  * ************************************************************************ */
 
-#include "common.h"
 #include "internal/precond/rocsparse_csritilu0.h"
+#include "rocsparse_common.hpp"
 #include "rocsparse_csritilu0_driver.hpp"
 
 namespace rocsparse
@@ -64,7 +64,9 @@ namespace rocsparse
         }
         }
 
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     template <typename T, typename I, typename J>
@@ -95,12 +97,13 @@ namespace rocsparse
             nmaxiter_[0] = 0;
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_zero_pivot);
         }
-
+        static constexpr J nfreeiter = 0;
         RETURN_IF_ROCSPARSE_ERROR((rocsparse::compute_dispatch<T, I, J>(alg_,
                                                                         handle_,
                                                                         alg_,
                                                                         options_,
                                                                         nmaxiter_,
+                                                                        nfreeiter,
                                                                         tol_,
                                                                         m_,
                                                                         nnz_,
