@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
  * ************************************************************************ */
 
 #include "rocsparse_csritilu0x_compute.hpp"
-#include "common.h"
+#include "rocsparse_common.hpp"
 #include "rocsparse_csritilu0x_driver.hpp"
 
 namespace rocsparse
@@ -61,7 +61,9 @@ namespace rocsparse
             return rocsparse_status_success;
         }
         }
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 }
 
@@ -70,6 +72,7 @@ rocsparse_status rocsparse::csritilu0x_compute_template(rocsparse_handle     han
                                                         rocsparse_itilu0_alg alg_,
                                                         rocsparse_int        options_,
                                                         J* __restrict__ nsweeps_,
+                                                        J                  nfreeiter_,
                                                         floating_data_t<T> tol_,
                                                         J                  m_,
                                                         I                  nnz_,
@@ -123,6 +126,7 @@ rocsparse_status rocsparse::csritilu0x_compute_template(rocsparse_handle     han
                                                                     handle_,
                                                                     options_,
                                                                     nsweeps_,
+                                                                    nfreeiter_,
                                                                     tol_,
                                                                     m_,
                                                                     nnz_,
@@ -158,6 +162,7 @@ rocsparse_status rocsparse::csritilu0x_compute_impl(rocsparse_handle     handle_
                                                     rocsparse_itilu0_alg alg_,
                                                     rocsparse_int        options_,
                                                     J* __restrict__ nsweeps_,
+                                                    J                  nfreeiter_,
                                                     floating_data_t<T> tol_,
                                                     J                  m_,
                                                     I                  nnz_,
@@ -197,6 +202,7 @@ rocsparse_status rocsparse::csritilu0x_compute_impl(rocsparse_handle     handle_
                          rocsparse::replaceX<T>("rocsparse_Xcsritilu0x"),
                          options_,
                          (const void*&)nsweeps_,
+                         nfreeiter_,
                          tol_,
                          m_,
                          nnz_,
@@ -266,33 +272,47 @@ rocsparse_status rocsparse::csritilu0x_compute_impl(rocsparse_handle     handle_
 
     if(rocsparse::enum_utils::is_invalid(base_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     if(rocsparse::enum_utils::is_invalid(ubase_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
     if(rocsparse::enum_utils::is_invalid(lbase_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     if(rocsparse::enum_utils::is_invalid(udiag_type_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
     if(rocsparse::enum_utils::is_invalid(udir_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
     if(rocsparse::enum_utils::is_invalid(ldiag_type_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
     if(rocsparse::enum_utils::is_invalid(ldir_))
     {
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     if((m_ > 0)
@@ -331,6 +351,7 @@ rocsparse_status rocsparse::csritilu0x_compute_impl(rocsparse_handle     handle_
                                                                      alg_,
                                                                      options_,
                                                                      nsweeps_,
+                                                                     nfreeiter_,
                                                                      tol_,
                                                                      m_,
                                                                      nnz_,
@@ -367,6 +388,7 @@ rocsparse_status rocsparse::csritilu0x_compute_impl(rocsparse_handle     handle_
         rocsparse_itilu0_alg alg_,                                          \
         J                    options_,                                      \
         J* __restrict__ nsweeps_,                                           \
+        J                  nfreeiter_,                                      \
         floating_data_t<T> tol_,                                            \
         J                  m_,                                              \
         I                  nnz_,                                            \
