@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,8 +24,8 @@
 
 #include "../conversion/rocsparse_csr2csc.hpp"
 #include "../conversion/rocsparse_identity.hpp"
-#include "common.h"
 #include "common.hpp"
+#include "rocsparse_common.hpp"
 #include "rocsparse_csritilu0x_driver.hpp"
 #include <iomanip>
 
@@ -407,9 +407,65 @@ public:
                                     size_t buffer_size_,
                                     void* __restrict__ buffer_)
         {
-            RETURN_IF_ROCSPARSE_ERROR(
-                (rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split_fusion>::
-                     history<T, J>::run(handle_, niter_, data_, buffer_size_, buffer_)));
+            if(buffer_size_ == 0)
+            {
+                *niter_ = static_cast<J>(0);
+                return rocsparse_status_success;
+            }
+
+            rocsparse::itilu0x_convergence_info_t<T, J> convergence_info;
+            buffer_ = convergence_info.init(handle_, buffer_);
+            J options;
+
+            RETURN_IF_HIP_ERROR(hipMemcpyAsync(&options,
+                                               convergence_info.info.options,
+                                               sizeof(J),
+                                               hipMemcpyDeviceToHost,
+                                               handle_->stream));
+
+            RETURN_IF_HIP_ERROR(hipMemcpyAsync(niter_,
+                                               convergence_info.info.iter,
+                                               sizeof(J),
+                                               hipMemcpyDeviceToHost,
+                                               handle_->stream));
+
+            RETURN_IF_HIP_ERROR(hipStreamSynchronize(handle_->stream));
+
+            J          niter = niter_[0];
+            const bool convergence_history
+                = (options & rocsparse_itilu0_option_convergence_history) > 0;
+            if(!convergence_history)
+            {
+                std::cerr << "convergence history has not been activated." << std::endl;
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error);
+            }
+
+            const bool compute_nrm_residual
+                = (options & rocsparse_itilu0_option_compute_nrm_residual) > 0;
+            const bool compute_nrm_corr
+                = (options & rocsparse_itilu0_option_compute_nrm_correction) > 0;
+
+            if(compute_nrm_corr)
+            {
+                RETURN_IF_HIP_ERROR(hipMemcpyAsync(data_,
+                                                   convergence_info.log_mxcorr,
+                                                   sizeof(T) * niter,
+                                                   hipMemcpyDeviceToHost,
+                                                   handle_->stream));
+            }
+
+            if(compute_nrm_residual)
+            {
+                RETURN_IF_HIP_ERROR(hipMemcpyAsync(data_ + niter,
+                                                   convergence_info.log_mxresidual,
+                                                   sizeof(T) * niter,
+                                                   hipMemcpyDeviceToHost,
+                                                   handle_->stream));
+            }
+
+            //
+            // No stream synchronization needed here,
+            //
             return rocsparse_status_success;
         }
     };
@@ -480,6 +536,7 @@ public:
             case rocsparse_datatype_u8_r:
             case rocsparse_datatype_i32_r:
             case rocsparse_datatype_u32_r:
+            case rocsparse_datatype_f16_r:
             {
                 RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
             }
@@ -506,6 +563,7 @@ public:
             case rocsparse_datatype_u8_r:
             case rocsparse_datatype_i32_r:
             case rocsparse_datatype_u32_r:
+            case rocsparse_datatype_f16_r:
             {
                 RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
             }
@@ -628,6 +686,7 @@ public:
         static rocsparse_status run(rocsparse_handle handle_,
                                     J                options_,
                                     J* __restrict__ nmaxiter_,
+                                    J                  nfreeiter_,
                                     floating_data_t<T> tol_,
                                     J                  m_,
                                     I                  nnz_,
@@ -708,8 +767,53 @@ public:
             //
             floating_data_t<T> nrm_indicator_previous = static_cast<floating_data_t<T>>(0);
             bool               converged              = false;
+
             for(J iter = 0; iter < nmaxiter; ++iter)
             {
+                for(J freeiter = 0; freeiter < nfreeiter_; ++freeiter)
+                {
+                    rocsparse::kernel_correction_dispatch<BLOCKSIZE, T, I, J>(
+                        m_,
+                        mean,
+                        handle_->wavefront_size,
+                        stream,
+                        m_,
+                        nnz_,
+                        ptr_begin_,
+                        ptr_end_,
+                        ind_,
+                        val_,
+                        base_,
+                        lptr_begin_,
+                        lptr_end_,
+                        lind_,
+                        lval_,
+                        layout_x_next.lval,
+                        lbase_,
+                        uptr_begin_,
+                        uptr_end_,
+                        uind_,
+                        uval_,
+                        layout_x_next.uval,
+                        ubase_,
+                        dval_,
+                        layout_x_next.dval);
+                    //
+                    // Copy (before stopping criteria).
+                    //
+                    RETURN_IF_ROCSPARSE_ERROR(copy_unkwowns(handle_,
+                                                            lnnz_,
+                                                            layout_x_next.lval,
+                                                            lval_,
+                                                            unnz_,
+                                                            layout_x_next.uval,
+                                                            uval_,
+                                                            m_,
+                                                            layout_x_next.dval,
+                                                            dval_,
+                                                            hipMemcpyDeviceToDevice));
+                }
+
                 floating_data_t<T> nrm_corr     = static_cast<floating_data_t<T>>(0);
                 floating_data_t<T> nrm_residual = static_cast<floating_data_t<T>>(0);
 
