@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,8 +24,8 @@
 
 #include "../conversion/rocsparse_csr2csc.hpp"
 #include "../conversion/rocsparse_identity.hpp"
-#include "common.h"
 #include "common.hpp"
+#include "rocsparse_common.hpp"
 #include "rocsparse_csritilu0x_driver.hpp"
 #include <iomanip>
 
@@ -432,90 +432,120 @@ namespace rocsparse
                         T* __restrict__ dval0_,
                         T* __restrict__ dval_)
     {
-        static constexpr uint32_t nid = BLOCKSIZE / WFSIZE;
-        const J                   lid = hipThreadIdx_x & (WFSIZE - 1);
-        const J                   wid = hipThreadIdx_x / WFSIZE;
-        const J                   row = nid * hipBlockIdx_x + wid;
-        if(row < m_)
+
+        static constexpr uint32_t nid  = BLOCKSIZE / WFSIZE;
+        const J                   lid  = hipThreadIdx_x & (WFSIZE - 1);
+        const J                   wid  = hipThreadIdx_x / WFSIZE;
+        const J                   row0 = BLOCKSIZE * hipBlockIdx_x + wid;
+        J                         iter = 0;
+        if(row0 < m_)
         {
-            for(J iter = 0; iter < niter_; ++iter)
+            for(; iter < niter_; ++iter)
             {
-                const J nl     = lptr_end_[row] - lptr_begin_[row];
-                const I lshift = lptr_begin_[row] - lbase_;
-                const I begin  = ((ptr_begin_[row] - base_) + lid);
-                const I end    = (ptr_end_[row] - base_);
 
-                for(I k = begin; k < end; k += WFSIZE)
+                for(J l = 0; l < WFSIZE; ++l)
                 {
-                    const J    col    = ind_[k] - base_;
-                    const I    ushift = uptr_begin_[col] - ubase_;
-                    const bool in_L   = (row > col);
-                    const bool in_U   = (row < col);
-                    const J    nu     = uptr_end_[col] - uptr_begin_[col];
-                    J          i = 0, j = 0;
-                    T          sum = rocsparse::sparse_dotproduct(nl,
-                                                         lind_ + lshift,
-                                                         lval0_ + lshift,
-                                                         lbase_,
-                                                         nu,
-                                                         uind_ + ushift,
-                                                         uval0_ + ushift,
-                                                         ubase_,
-                                                         i,
-                                                         j);
+                    J row = row0 + nid * l;
 
-                    T s = val_[k] - sum;
-                    if(in_L)
+                    if(row < m_)
                     {
-                        s /= dval0_[col];
-                    }
+                        const J nl     = lptr_end_[row] - lptr_begin_[row];
+                        const I lshift = lptr_begin_[row] - lbase_;
+                        const I begin  = ((ptr_begin_[row] - base_) + lid);
+                        const I end    = (ptr_end_[row] - base_);
 
-                    //
-                    // Assign.
-                    //
-                    const bool assignable = !std::isinf(std::abs(s));
-                    if(assignable)
-                    {
-                        if(in_L)
+                        for(I k = begin; k < end; k += WFSIZE)
                         {
-                            for(J h = i; h < nl; ++h)
+                            const J    col    = ind_[k] - base_;
+                            const I    ushift = uptr_begin_[col] - ubase_;
+                            const bool in_L   = (row > col);
+                            const bool in_U   = (row < col);
+                            const J    nu     = uptr_end_[col] - uptr_begin_[col];
+                            J          i = 0, j = 0;
+                            T          sum = rocsparse::sparse_dotproduct(nl,
+                                                                 lind_ + lshift,
+                                                                 lval0_ + lshift,
+                                                                 lbase_,
+                                                                 nu,
+                                                                 uind_ + ushift,
+                                                                 uval0_ + ushift,
+                                                                 ubase_,
+                                                                 i,
+                                                                 j);
+
+                            T s = val_[k] - sum;
+                            if(in_L)
                             {
-                                if((lind_[lshift + h] - lbase_) == col)
+                                if(std::abs(dval0_[col]) > 0)
                                 {
-                                    lval_[lshift + h] = s;
-                                    break;
+                                    s /= dval0_[col];
+                                }
+                                else
+                                {
+                                    s = 0;
                                 }
                             }
-                        }
-                        else if(in_U)
-                        {
-                            for(J h = j; h < nu; ++h)
+
+                            //
+                            // Assign.
+                            //
+                            floating_data_t<T> tmp        = std::abs(s);
+                            const bool         assignable = (!std::isinf(tmp) && !std::isnan(tmp));
+                            if(assignable)
                             {
-                                if((uind_[ushift + h] - ubase_) == row)
+                                if(in_L)
                                 {
-                                    uval_[ushift + h] = s;
-                                    break;
+                                    for(J h = i; h < nl; ++h)
+                                    {
+                                        if((lind_[lshift + h] - lbase_) == col)
+                                        {
+                                            lval_[lshift + h] = s;
+                                            break;
+                                        }
+                                    }
+                                }
+                                else if(in_U)
+                                {
+                                    for(J h = j; h < nu; ++h)
+                                    {
+                                        if((uind_[ushift + h] - ubase_) == row)
+                                        {
+                                            uval_[ushift + h] = s;
+                                            break;
+                                        }
+                                    }
+                                }
+                                else
+                                {
+                                    dval_[col] = s;
                                 }
                             }
                         }
-                        else
-                        {
-                            dval_[col] = s;
-                        }
                     }
                 }
 
-                for(I i = uptr_begin_[row] + lid; i < uptr_end_[row]; i += WFSIZE)
+                //
+                // COPY
+                //
+                for(J row = row0; row < BLOCKSIZE * (hipBlockIdx_x + 1); row += nid)
                 {
-                    uval0_[i] = uval_[i];
-                }
-                for(I i = lptr_begin_[row] + lid; i < lptr_end_[row]; i += WFSIZE)
-                {
-                    lval0_[i] = lval_[i];
-                }
-                if(lid == 0)
-                {
-                    dval0_[row] = dval_[row];
+                    if(row < m_)
+                    {
+                        for(I ii = lptr_begin_[row] - lbase_ + lid; ii < lptr_end_[row] - lbase_;
+                            ii += WFSIZE)
+                        {
+                            lval0_[ii] = lval_[ii];
+                        }
+                        for(I ii = uptr_begin_[row] - ubase_ + lid; ii < uptr_end_[row] - ubase_;
+                            ii += WFSIZE)
+                        {
+                            uval0_[ii] = uval_[ii];
+                        }
+                        if(lid == 0)
+                        {
+                            dval0_[row] = dval_[row];
+                        }
+                    }
                 }
             }
         }
@@ -600,65 +630,9 @@ struct rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split_fusion>
                                     size_t buffer_size_,
                                     void* __restrict__ buffer_)
         {
-            if(buffer_size_ == 0)
-            {
-                *niter_ = static_cast<J>(0);
-                return rocsparse_status_success;
-            }
-
-            rocsparse::itilu0x_convergence_info_t<T, J> convergence_info;
-            buffer_ = convergence_info.init(handle_, buffer_);
-            J options;
-
-            RETURN_IF_HIP_ERROR(hipMemcpyAsync(&options,
-                                               convergence_info.info.options,
-                                               sizeof(J),
-                                               hipMemcpyDeviceToHost,
-                                               handle_->stream));
-
-            RETURN_IF_HIP_ERROR(hipMemcpyAsync(niter_,
-                                               convergence_info.info.iter,
-                                               sizeof(J),
-                                               hipMemcpyDeviceToHost,
-                                               handle_->stream));
-
-            RETURN_IF_HIP_ERROR(hipStreamSynchronize(handle_->stream));
-
-            J          niter = niter_[0];
-            const bool convergence_history
-                = (options & rocsparse_itilu0_option_convergence_history) > 0;
-            if(!convergence_history)
-            {
-                std::cerr << "convergence history has not been activated." << std::endl;
-                RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error);
-            }
-
-            const bool compute_nrm_residual
-                = (options & rocsparse_itilu0_option_compute_nrm_residual) > 0;
-            const bool compute_nrm_corr
-                = (options & rocsparse_itilu0_option_compute_nrm_correction) > 0;
-
-            if(compute_nrm_corr)
-            {
-                RETURN_IF_HIP_ERROR(hipMemcpyAsync(data_,
-                                                   convergence_info.log_mxcorr,
-                                                   sizeof(T) * niter,
-                                                   hipMemcpyDeviceToHost,
-                                                   handle_->stream));
-            }
-
-            if(compute_nrm_residual)
-            {
-                RETURN_IF_HIP_ERROR(hipMemcpyAsync(data_ + niter,
-                                                   convergence_info.log_mxresidual,
-                                                   sizeof(T) * niter,
-                                                   hipMemcpyDeviceToHost,
-                                                   handle_->stream));
-            }
-
-            //
-            // No stream synchronization needed here,
-            //
+            RETURN_IF_ROCSPARSE_ERROR((
+                rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split>::history<T, J>::run(
+                    handle_, niter_, data_, buffer_size_, buffer_)));
             return rocsparse_status_success;
         }
     };
@@ -728,6 +702,7 @@ struct rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split_fusion>
             case rocsparse_datatype_u8_r:
             case rocsparse_datatype_i32_r:
             case rocsparse_datatype_u32_r:
+            case rocsparse_datatype_f16_r:
             {
                 RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
             }
@@ -754,6 +729,7 @@ struct rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split_fusion>
             case rocsparse_datatype_u8_r:
             case rocsparse_datatype_i32_r:
             case rocsparse_datatype_u32_r:
+            case rocsparse_datatype_f16_r:
             {
                 RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
             }
@@ -812,6 +788,7 @@ struct rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split_fusion>
         static rocsparse_status run(rocsparse_handle handle_,
                                     J                options_,
                                     J* __restrict__ nmaxiter_,
+                                    J                  nfreeiter_,
                                     floating_data_t<T> tol_,
                                     J                  m_,
                                     I                  nnz_,
@@ -914,6 +891,41 @@ struct rocsparse::csritilu0x_driver_t<rocsparse_itilu0_alg_sync_split_fusion>
             bool               converged              = false;
             for(J iter = 0; iter < nmaxiter; ++iter)
             {
+
+                for(J freeiter = 0; freeiter < nfreeiter_; ++freeiter)
+                {
+                    rocsparse::kernel_freerun_dispatch<BLOCKSIZE, T, I, J>(m_,
+                                                                           mean,
+                                                                           handle_->wavefront_size,
+                                                                           handle_->stream,
+                                                                           1,
+                                                                           //
+                                                                           m_,
+                                                                           nnz_,
+                                                                           ptr_begin_,
+                                                                           ptr_end_,
+                                                                           ind_,
+                                                                           val_,
+                                                                           base_,
+                                                                           //
+                                                                           lptr_begin_,
+                                                                           lptr_end_,
+                                                                           lind_,
+                                                                           lval_,
+                                                                           layout_next.lval,
+                                                                           lbase_,
+                                                                           //
+                                                                           uptr_begin_,
+                                                                           uptr_end_,
+                                                                           uind_,
+                                                                           uval_,
+                                                                           layout_next.uval,
+                                                                           ubase_,
+                                                                           //
+                                                                           dval_,
+                                                                           layout_next.dval);
+                }
+
                 if(!stopping_criteria && !compute_nrm_corr && !compute_nrm_residual)
                 {
                     rocsparse::kernel_freerun_dispatch<BLOCKSIZE, T, I, J>(m_,
