@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,11 @@
  * ************************************************************************ */
 
 #include "internal/reordering/rocsparse_csrcolor.h"
-#include "control.h"
 #include "csrcolor_device.hpp"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csrcolor.hpp"
-#include "utility.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
+#include "rocsparse_utility.hpp"
 
 namespace rocsparse
 {
@@ -183,6 +183,8 @@ namespace rocsparse
                                                       J                colors_length,
                                                       J*               colors)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         hipStream_t stream = handle->stream;
         J           m, n;
 
@@ -196,7 +198,7 @@ namespace rocsparse
         // Allocation.
         //
         RETURN_IF_HIP_ERROR(
-            rocsparse_hipMallocAsync((void**)&seq_ptr, sizeof(J) * (n + 1), handle->stream));
+            rocsparse_hipMallocAsync(&seq_ptr, sizeof(J) * (n + 1), handle->stream));
 
         //
         // Set to 0.
@@ -229,8 +231,8 @@ namespace rocsparse
         //
         // Obtain rocprim buffer size
         //
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(
-            nullptr, temp_storage_bytes, seq_ptr, seq_ptr, n + 1, rocprim::plus<J>(), stream));
+        RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::inclusive_scan_buffer_size<J, J>(
+            handle, n + 1, &temp_storage_bytes)));
 
         //
         // Get rocprim buffer
@@ -256,13 +258,9 @@ namespace rocsparse
         //
         // Perform actual inclusive sum.
         //
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(d_temp_storage,
-                                                    temp_storage_bytes,
-                                                    seq_ptr,
-                                                    seq_ptr,
-                                                    n + 1,
-                                                    rocprim::plus<J>(),
-                                                    handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(
+            handle, seq_ptr, seq_ptr, n + 1, temp_storage_bytes, d_temp_storage));
+
         //
         // Free rocprim buffer, if allocated.
         //
@@ -318,6 +316,8 @@ namespace rocsparse
                 colors,
                 seq_ptr);
         }
+
+        RETURN_IF_HIP_ERROR(rocsparse_hipFreeAsync(seq_ptr, handle->stream));
         return rocsparse_status_success;
     }
 }
@@ -336,6 +336,8 @@ rocsparse_status rocsparse::csrcolor_core(rocsparse_handle          handle,
                                           J*                        reordering,
                                           rocsparse_mat_info        info)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     static constexpr rocsparse_int blocksize = 256;
 
     hipStream_t stream = handle->stream;
@@ -349,7 +351,7 @@ rocsparse_status rocsparse::csrcolor_core(rocsparse_handle          handle,
     //
     J* workspace;
     RETURN_IF_HIP_ERROR(
-        rocsparse_hipMallocAsync((void**)&workspace, sizeof(J) * blocksize, handle->stream));
+        rocsparse_hipMallocAsync(&workspace, sizeof(J) * blocksize, handle->stream));
 
     //
     // Initialize colors
@@ -481,47 +483,42 @@ rocsparse_status rocsparse::csrcolor_core(rocsparse_handle          handle,
             rocsparse_hipMallocAsync(&sorted_colors, sizeof(J) * m, handle->stream));
 
         {
-            rocsparse_int* keys_input    = colors;
-            rocsparse_int* values_input  = reordering_identity;
-            rocsparse_int* keys_output   = sorted_colors;
-            rocsparse_int* values_output = reordering;
-
             size_t temporary_storage_size_bytes;
             void*  temporary_storage_ptr = nullptr;
 
             //
             // Get required size of the temporary storage
             //
-            rocprim::radix_sort_pairs(temporary_storage_ptr,
-                                      temporary_storage_size_bytes,
-                                      keys_input,
-                                      keys_output,
-                                      values_input,
-                                      values_output,
-                                      m,
-                                      0,
-                                      sizeof(rocsparse_int) * 8,
-                                      stream);
+            static constexpr bool s_using_double_buffers = false;
+            RETURN_IF_ROCSPARSE_ERROR(
+                (rocsparse::primitives::radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int>(
+                    handle,
+                    m,
+                    0,
+                    sizeof(rocsparse_int) * 8,
+                    &temporary_storage_size_bytes,
+                    s_using_double_buffers)));
 
             //
             // allocate temporary storage
             //
-            rocsparse_hipMallocAsync(
-                &temporary_storage_ptr, temporary_storage_size_bytes, handle->stream);
+            RETURN_IF_HIP_ERROR(rocsparse_hipMallocAsync(
+                &temporary_storage_ptr, temporary_storage_size_bytes, handle->stream));
 
             //
             // perform sort
             //
-            rocprim::radix_sort_pairs(temporary_storage_ptr,
-                                      temporary_storage_size_bytes,
-                                      keys_input,
-                                      keys_output,
-                                      values_input,
-                                      values_output,
-                                      m,
-                                      0,
-                                      sizeof(rocsparse_int) * 8,
-                                      stream);
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::primitives::radix_sort_pairs(handle,
+                                                        colors,
+                                                        sorted_colors,
+                                                        reordering_identity,
+                                                        reordering,
+                                                        m,
+                                                        0,
+                                                        sizeof(rocsparse_int) * 8,
+                                                        temporary_storage_size_bytes,
+                                                        temporary_storage_ptr));
 
             RETURN_IF_HIP_ERROR(rocsparse_hipFreeAsync(temporary_storage_ptr, handle->stream));
         }
@@ -546,6 +543,8 @@ rocsparse_status rocsparse::csrcolor_quickreturn(rocsparse_handle          handl
                                                  void*                     reordering,
                                                  rocsparse_mat_info        info)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     if(m == 0 || nnz == 0)
     {
         return rocsparse_status_success;
@@ -568,6 +567,8 @@ namespace rocsparse
                                               void*                     reordering, //10
                                               rocsparse_mat_info        info) //11
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_SIZE(1, m);
         ROCSPARSE_CHECKARG_SIZE(2, nnz);
@@ -623,6 +624,8 @@ namespace rocsparse
                                           J*                        reordering,
                                           rocsparse_mat_info        info)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Logging
         rocsparse::log_trace(handle,
                              rocsparse::replaceX<T>("rocsparse_Xcsrcolor"),
@@ -695,6 +698,7 @@ namespace rocsparse
                                      rocsparse_mat_info        info)              \
     try                                                                           \
     {                                                                             \
+        ROCSPARSE_ROUTINE_TRACE;                                                  \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrcolor_impl(handle,                \
                                                            m,                     \
                                                            nnz,                   \
