@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,12 +24,12 @@
 #include "internal/util/rocsparse_check_matrix_csr.h"
 
 #include "rocsparse_check_matrix_csr.hpp"
-#include "to_string.hpp"
-#include "utility.h"
+#include "rocsparse_enum_utils.hpp"
+#include "rocsparse_utility.hpp"
 
 #include "check_matrix_csr_device.h"
 
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 template <typename T, typename I, typename J>
 rocsparse_status rocsparse::check_matrix_csr_buffer_size_core(rocsparse_handle       handle,
@@ -45,25 +45,17 @@ rocsparse_status rocsparse::check_matrix_csr_buffer_size_core(rocsparse_handle
                                                               rocsparse_storage_mode storage,
                                                               size_t*                buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     *buffer_size = 0;
     *buffer_size += ((sizeof(rocsparse_data_status) - 1) / 256 + 1) * 256; // data status
 
     if(storage == rocsparse_storage_mode_unsorted)
     {
         // Determine required rocprim buffer size
-        size_t                    rocprim_buffer_size;
-        rocprim::double_buffer<J> dummy(nullptr, nullptr);
-        RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_pairs(nullptr,
-                                                                rocprim_buffer_size,
-                                                                dummy,
-                                                                dummy,
-                                                                nnz,
-                                                                m,
-                                                                csr_row_ptr,
-                                                                csr_row_ptr + 1,
-                                                                0,
-                                                                rocsparse::clz(n),
-                                                                handle->stream));
+        size_t rocprim_buffer_size;
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::sort_csr_column_indices_buffer_size(
+            handle, m, n, nnz, csr_row_ptr, &rocprim_buffer_size));
         *buffer_size += ((rocprim_buffer_size - 1) / 256 + 1) * 256;
 
         // offset buffer
@@ -94,6 +86,8 @@ namespace rocsparse
                                                  rocsparse_storage_mode storage,
                                                  size_t*                buffer_size)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         return rocsparse_status_continue;
     }
 }
@@ -113,6 +107,8 @@ rocsparse_status
                                                      rocsparse_storage_mode storage, //10
                                                      size_t*                buffer_size) //11
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
     ROCSPARSE_CHECKARG_SIZE(1, m);
@@ -133,7 +129,7 @@ rocsparse_status
         {
             rocsparse::log_debug(handle,
                                  ("Matrix was specified to be "
-                                  + std::string(rocsparse::to_string(matrix_type))
+                                  + std::string(rocsparse::enum_utils::to_string(matrix_type))
                                   + " but m != n"));
         }
     }
@@ -226,6 +222,7 @@ INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
                                      size_t*                buffer_size)                       \
     try                                                                                        \
     {                                                                                          \
+        ROCSPARSE_ROUTINE_TRACE;                                                               \
         RETURN_IF_ROCSPARSE_ERROR(                                                             \
             (rocsparse::check_matrix_csr_buffer_size_impl<TYPE, rocsparse_int, rocsparse_int>( \
                 handle,                                                                        \
