@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,9 @@
 #include "internal/util/rocsparse_check_matrix_gebsr.h"
 
 #include "rocsparse_check_matrix_gebsr.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
-#include "rocsparse_primitives.h"
+#include "rocsparse_primitives.hpp"
 
 #include "check_matrix_gebsr_device.h"
 
@@ -47,6 +47,8 @@ rocsparse_status rocsparse::check_matrix_gebsr_buffer_size_core(rocsparse_handle
                                                                 rocsparse_storage_mode storage,
                                                                 size_t*                buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     *buffer_size = 0;
     *buffer_size += ((sizeof(rocsparse_data_status) - 1) / 256 + 1) * 256; // data status
 
@@ -89,6 +91,8 @@ namespace rocsparse
                                                    rocsparse_storage_mode storage,
                                                    size_t*                buffer_size)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         return rocsparse_status_continue;
     }
 }
@@ -111,6 +115,8 @@ rocsparse_status
                                                        rocsparse_storage_mode storage, //13
                                                        size_t*                buffer_size) //14
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_ENUM(1, dir);
     ROCSPARSE_CHECKARG_SIZE(2, mb);
@@ -229,6 +235,7 @@ INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
                                      size_t*                buffer_size)                      \
     try                                                                                       \
     {                                                                                         \
+        ROCSPARSE_ROUTINE_TRACE;                                                              \
         RETURN_IF_ROCSPARSE_ERROR(                                                            \
             (rocsparse::check_matrix_gebsr_buffer_size_impl<T, rocsparse_int, rocsparse_int>( \
                 handle,                                                                       \
